module sim_ltc2123_jesd204b_logic_v #
(
parameter   DATA_A_PLUS   =    8'h00,//gyu test
parameter   DATA_B_PLUS   =    8'h00,//gyu test
parameter   SYSREF_SHIFT  =    4'h6//gyu test
)
(
  output wire [2:0]   L_,//gyu pass parameter
  output wire [4:0]   K_,//gyu pass parameter

  input wire          sync,//gyu handled by FPGA
  output wire         div2_rstb,
  output wire         PLL_ref,
  output wire [19:0]  DAT_A1,
  output wire [19:0]  DAT_A0,
  output wire [19:0]  DAT_B0,
  output wire [19:0]  DAT_B1,
  output wire         ser_DAT_A1,
  output wire         ser_DAT_A0,
  output wire         ser_DAT_B0,
  output wire         ser_DAT_B1,
  output wire         ser_A1_en,
  output wire         ser_A0_en,
  output wire         ser_B0_en,
  output wire         ser_B1_en,
  output wire         two_adc_per_ln,
  output wire         two_ln_per_adc
);

reg   [19:0] shift_DAT_A0;
reg   [19:0] shift_DAT_A1;
reg   [19:0] shift_DAT_B0;
reg   [19:0] shift_DAT_B1;

wire         scan_mode;
wire         static_clk;
wire         ser_clk;
wire         dev_clk;
wire         adc_clk;
wire         CLK_frm;
wire         CLK_2char;
wire [15:0]  DATA_A;
wire [15:0]  DATA_B;
wire         nap;
wire         pd_all;
wire         pda;
wire         pdb;
wire         single;
wire [7:0]   DID;
wire [3:0]   BID;
wire [2:0]   LID_A1;
wire [2:0]   LID_A0;
wire [2:0]   LID_B0;
wire [2:0]   LID_B1;
wire [1:0]   CID_A1;
wire [1:0]   CID_A0;
wire [1:0]   CID_B0;
wire [1:0]   CID_B1;
wire         SCR;
//wire [2:0]   L_;//gyu pass parameter
//wire [4:0]   K_;//gyu pass parameter
wire [2:0]   M_;
wire [1:0]   CS;
wire [3:0]   N_;
wire [1:0]   SUBCLASSV;
wire         LAS_dis;
wire         LAM_dis;
wire         FAM_dis;
wire         TX_SYNC_en;
wire [2:0]   PAT;
wire [2:0]   R_;
wire         RST_dis;
wire         alert_en;
//wire         sync//gyu handled by FPGA
wire         sysref;

assign ser_DAT_A0 = shift_DAT_A0[19];
assign ser_DAT_A1 = shift_DAT_A1[19];
assign ser_DAT_B0 = shift_DAT_B0[19];
assign ser_DAT_B1 = shift_DAT_B1[19];

always @(posedge CLK_2char)
begin
        shift_DAT_A0 <= DAT_A0;
        shift_DAT_A1 <= DAT_A1;
        shift_DAT_B0 <= DAT_B0;
        shift_DAT_B1 <= DAT_B1;
end
        
always @(posedge ser_clk)
begin
        shift_DAT_A0 = shift_DAT_A0 << 1;
        shift_DAT_A1 = shift_DAT_A1 << 1;
        shift_DAT_B0 = shift_DAT_B0 << 1;
        shift_DAT_B1 = shift_DAT_B1 << 1;
end
        
ltc2123_jesd204b_logic ltc2123_jesd204b_logic
(

  .div2_rstb(div2_rstb),
  .PLL_ref(PLL_ref),
  .DAT_A1(DAT_A1),
  .DAT_A0(DAT_A0),
  .DAT_B0(DAT_B0),
  .DAT_B1(DAT_B1),
  .ser_A1_en(ser_A1_en),
  .ser_A0_en(ser_A0_en),
  .ser_B0_en(ser_B0_en),
  .ser_B1_en(ser_B1_en),
  .two_adc_per_ln(two_adc_per_ln),
  .two_ln_per_adc(two_ln_per_adc),

  .scan_mode(scan_mode),
  .static_clk(static_clk),

  .dev_clk(dev_clk),
  .adc_clk(adc_clk),
  .CLK_frm(CLK_frm),
  .CLK_2char(CLK_2char),
  .DATA_A(DATA_A),
  .DATA_B(DATA_B),
  .nap(nap),
  .pd_all(pd_all),
  .pda(pda),
  .pdb(pdb),
  .single(single),
  .DID(DID),
  .BID(BID),
  .LID_A1(LID_A1),
  .LID_A0(LID_A0),
  .LID_B0(LID_B0),
  .LID_B1(LID_B1),
  .CID_A1(CID_A1),
  .CID_A0(CID_A0),
  .CID_B0(CID_B0),
  .CID_B1(CID_B1),
  .SCR(SCR),
  .L_(L_),
  .K_(K_),
  .M_(M_),
  .CS(CS),
  .N_(N_),
  .SUBCLASSV(SUBCLASSV),
  .LAS_dis(LAS_dis),
  .LAM_dis(LAM_dis),
  .FAM_dis(FAM_dis),
  .TX_SYNC_en(TX_SYNC_en),
  .PAT(PAT),
  .R_(R_),
  .RST_dis(RST_dis),
  .alert_en(alert_en),
  .sync(sync),
  .sysref(sysref)
);

test_vectors #
(
   .DATA_A_PLUS (DATA_A_PLUS),//gyu test
   .DATA_B_PLUS (DATA_B_PLUS),//gyu test
   .SYSREF_SHIFT (SYSREF_SHIFT)//gyu test
)
vectors
(

   .scan_mode(scan_mode),
   .static_clk(static_clk),
   .ser_clk(ser_clk),
   .dev_clk(dev_clk),
   .adc_clk(adc_clk),
   .CLK_frm(CLK_frm),
   .CLK_2char(CLK_2char),
   .DATA_A(DATA_A),
   .DATA_B(DATA_B),
   .nap(nap),
   .pd_all(pd_all),
   .pda(pda),
   .pdb(pdb),
   .single(single),
   .DID(DID),
   .BID(BID),
   .LID_A1(LID_A1),
   .LID_A0(LID_A0),
   .LID_B0(LID_B0),
   .LID_B1(LID_B1),
   .CID_A1(CID_A1),
   .CID_A0(CID_A0),
   .CID_B0(CID_B0),
   .CID_B1(CID_B1),
   .SCR(SCR),
   .L_(L_),
   .K_(K_),
   .M_(M_),
   .CS(CS),
   .N_(N_),
   .SUBCLASSV(SUBCLASSV),
   .LAS_dis(LAS_dis),
   .LAM_dis(LAM_dis),
   .FAM_dis(FAM_dis),
   .TX_SYNC_en(TX_SYNC_en),
   .PAT(PAT),
   .R_(R_),
   .RST_dis(RST_dis),
   .alert_en(alert_en),
   .sync(sync),
   .sysref(sysref)
   );

  endmodule
