`timescale 10ps / 1ps
/* For this subclass 1 case, SYNC is performed before SYSREF is asserted.
This causes the K28.5 commas to be sent up to the start of the first Multiframe
after SYNC is De-asserted.  SYSREF is asserted twice which keeps the K28.5
commas flowing until the start of next full MF after the SYSREF occurs.

At about 312ns, after the Lane alignment sequence is completed, SYSREF is 
asserted again causing another change of Multi-frame.  Since TX_SYNC_en is 
de-asserted, K28.5 commas are not sent and a new lane alignment sequence is not 
initiated.
 
If TX_SYNC_en had been asserted, K28.5 commas would have been sent for one MF
and then a lane alignment sequence would have been sent.  When the receiver
sees the K28.5 commas, it will either ignore them and wait for the new Lane
Alignment Sequence or it will assert the SYNC signal in order to take control
of this alignment process. 

At about 310ns to 350ns, and constant "D3 HEX" is sent as OCTET2.  This is to
set up a condition where K28.7 characters may be sent for Frame Alignment
Monitoring, and K28.3 characters may be sent for Lane Alignment Monitoring.

At about 730-750ns, the built in patterns are sent, beginning with pattern 1
and ending with pattern 7.
*/
module test_vectors #
(
parameter   DATA_A_PLUS   =    8'h00,//gyu test
parameter   DATA_B_PLUS   =    8'h00,//gyu test
parameter   SYSREF_SHIFT  =    4'h6//gyu test
)
(

   output reg          scan_mode,
   output reg          static_clk,
   output reg          ser_clk,
   output reg          dev_clk,
   output reg          adc_clk,
   output reg          CLK_frm,
   output reg          CLK_2char,
   output reg  [15:0]  DATA_A,
   output reg  [15:0]  DATA_B,

   output reg          nap,
   output reg          pd_all,
   output reg          pda,
   output reg          pdb,
   output reg          single,

   output reg  [7:0]   DID,
   output reg  [3:0]   BID,
   output reg  [2:0]   LID_A1,
   output reg  [2:0]   LID_A0,
   output reg  [2:0]   LID_B0,
   output reg  [2:0]   LID_B1,
   output reg  [1:0]   CID_A1,
   output reg  [1:0]   CID_A0,
   output reg  [1:0]   CID_B0,
   output reg  [1:0]   CID_B1,
   output reg          SCR,
   output reg  [2:0]   L_,
   output reg  [4:0]   K_,
   output reg  [2:0]   M_,
   output reg  [1:0]   CS,
   output reg  [3:0]   N_,
   output reg  [1:0]   SUBCLASSV,
   output reg          LAS_dis,
   output reg          LAM_dis,
   output reg          FAM_dis,
   output reg          TX_SYNC_en,
   output reg  [2:0]   PAT,
   output reg  [2:0]   R_,
   output reg          RST_dis,
   output reg          alert_en,
   output reg          sysref,
   //output reg          sync//gyu handled by FPGA
   input               sync//gyu handled by FPGA
   );


   reg  [4:0]   sysref_cnt;//gyu msysref
   reg          sysref_sta;//gyu msysref
   
   reg          reverse_a; //reverse_a disparity
   reg          reverse_b; //reverse_b disparity
   
   //gyu create a counter to idendify the edge of MF
   reg          [3:0] mf_cnt = 4'b0;
   always @ (posedge CLK_2char)
     if (mf_cnt == 4'h9)
        mf_cnt <= 4'h0;
     else    
        mf_cnt <= mf_cnt + 1;
    

   /******************************************************************************/
   /** INSTANCES *****************************************************************/
   /******************************************************************************/
    initial
    begin
      sysref_sta = 1'b0;
      //#3502005;
      //sysref_sta = 1'b1;//gyu msysref//gyu test
      #2000;
      sysref_sta = 1'b1;//gyu msysref//gyu test
            
    end
    
    //always @ (posedge demo_tb.refclk_p or negedge sysref_sta) //gyu msysref//gyu fixed 312Mhz ref input simualtion issue 
    //always @ (posedge demo_tb.DUT.refclk or negedge sysref_sta) //gyu msysref//gyu fixed 312Mhz ref input simualtion issue
    always @ (posedge demo_tb.DUT.i_jd204bv6p0l4sc0_support_block.inst.refclk or negedge sysref_sta) //gyu msysref//gyu fixed 312Mhz ref input simualtion issue
      if (~sysref_sta)
      begin
        sysref_cnt  <= #1 5'b0;
        sysref  <= #1 1'b0;
      end
      //else if (sysref_cnt == 5'h6)//gyu test
      else if (sysref_cnt == {1'b0, SYSREF_SHIFT})//gyu test
      begin
        sysref_cnt  <= #1 sysref_cnt + 1;
        sysref  <= #1 1'b1;
      end
      else if (sysref_cnt == K_)
      begin
        sysref_cnt  <= #1 5'b0;
        sysref  <= #1 1'b0;
      end
      else
      begin
        sysref_cnt  <= #1 sysref_cnt + 1;
        sysref  <= #1 1'b0;
      end
    

    //initial #2000000 $finish;//gyu test
    //initial #12000000 $finish;//gyu test
    initial
      begin
      
        scan_mode = 0;
        static_clk = 0;

        DATA_A[15:8] = 8'h00 + DATA_A_PLUS * 2;//gyu test;
        DATA_A[7:0] = 8'h01 + DATA_A_PLUS;//gyu test
        DATA_B[15:8] = 8'h10 + DATA_B_PLUS * 4;//gyu test
        DATA_B[7:0] = 8'h11 + DATA_B_PLUS;//gyu test

        nap = 0;
        pd_all = 0;
        pda = 0;
        pdb = 0;
        single = 0;

        DID = 8'h23;
        BID = 4'h6;
        LID_A1 = 3'h1;
        LID_A0 = 3'h0;
        LID_B0 = 3'h1;
        LID_B1 = 3'h3;
        CID_A1 = 2'h0;
        CID_A0 = 2'h0;
        CID_B0 = 2'h1;
        CID_B1 = 2'h1;
        SCR = 1'b0;
        L_ = 3'h1;
        //K_ = 5'h9;//gyu test
        K_ = 5'hf;//gyu test
        M_ = 3'h1;
        CS = 2'h1;
        N_ = 4'hD;
        SUBCLASSV = 2'h1;
        LAS_dis = 1'b0;
        
        //LAM_dis = 1'b0;//gyu test
        LAM_dis = 1'b1;//gyu test
        //FAM_dis = 1'b0;//gyu test
        FAM_dis = 1'b1;//gyu test
        
        TX_SYNC_en = 1'b0;
        PAT = 3'h0;
        R_ = 3'h2;
        RST_dis = 0;
        alert_en = 0;
        //sysref = 0;//gyu msysref
        //sync = 1;//gyu handled by FPGA

        reverse_a = 0;  //reverse disparity variable for channel a
        reverse_b = 0;  //reverse disparity variable for channel b
        pd_all = 0; pda = 0; pdb = 0;
        #20 pd_all = 1; pda = 1; pdb = 1;
        #190 pd_all = 0; pda = 0; pdb = 0;
           
        //while (demo_tb.rx_aresetn !== 1'b1) @(posedge dev_clk);//gyu waiting FPGA ready
        //while (demo_tb.rx_aresetn !== 1'b0) @(posedge dev_clk);//gyu recover Kintex-7 aresetn issue
        //while (demo_tb.rx_aresetn !== 1'b1) @(posedge dev_clk);//gyu recover Kintex-7 aresetn issue
        
        //#2000 sync = 0;//gyu handled by FPGA
        //#4000 sync = 1;//gyu handled by FPGA
        #6000;//gyu handled by FPGA
   /*   
        //gyu msysref   
        #2802005 sysref = 1;   
        #1000 sysref = 0; 
        #3000 sysref = 1;   
        #1000 sysref = 0;
   */

    end
    
parameter   DEVCLK_PERIOD    =   320; //gyu 312.5M
//parameter   DEVCLK_PERIOD    =   360; //gyu 277.8M
//parameter   DEVCLK_PERIOD    =   400; //gyu 250.0M
//parameter   DEVCLK_PERIOD    =   625; //gyu 160.0M


    //initial dev_clk=1;
    //always
    //begin
    //  #80 dev_clk = ~dev_clk;
    //  #80 dev_clk = ~dev_clk;
    //end
     
    initial ser_clk=1;
    always
    begin
      #(DEVCLK_PERIOD/40) ser_clk = ~ser_clk;//gyu
      //#8 ser_clk = ~ser_clk;//gyu
    end

    initial dev_clk=1;
    always
    begin
      #(DEVCLK_PERIOD/2) dev_clk = ~dev_clk;//gyu
      #(DEVCLK_PERIOD/2) dev_clk = ~dev_clk;
      //#160 dev_clk = ~dev_clk;//gyu
      //#160 dev_clk = ~dev_clk;
    end

    initial adc_clk=1;
    always
    begin
      #(DEVCLK_PERIOD/2) adc_clk = ~adc_clk;//gyu
      #(DEVCLK_PERIOD/2) adc_clk = ~adc_clk;
      //#160 adc_clk = ~adc_clk;//gyu
      //#160 adc_clk = ~adc_clk;
    end

    initial CLK_2char=1;
    always
    begin
      #(DEVCLK_PERIOD * 3/5) CLK_2char = ~CLK_2char;//gyu
      #(DEVCLK_PERIOD * 2/5) CLK_2char = ~CLK_2char;
      //#192 CLK_2char = ~CLK_2char;//gyu
      //#128 CLK_2char = ~CLK_2char;
    end

    initial CLK_frm=1;
    always
    begin
      #(DEVCLK_PERIOD * 3/5) CLK_frm = ~CLK_frm;//gyu
      #(DEVCLK_PERIOD * 2/5) CLK_frm = ~CLK_frm;
      //#192 CLK_frm = ~CLK_frm;//gyu
      //#128 CLK_frm = ~CLK_frm;
    end
    
    //initial CLK_frm=1;
    //always
    //begin
    //  #320 CLK_frm = ~CLK_frm;
    //  #320  CLK_frm = ~CLK_frm;
    //end
    
    always @ (posedge adc_clk)
    begin        //Check 8b10b
        #10
      if (!pd_all && !pda)
      begin
        if (DATA_A[15:8] < 8'hD2)
        begin
          DATA_A[15:8] = DATA_A[15:8]+2;
          DATA_A[7:0] = DATA_A[7:0]+2;
        end
        else
        if ((DATA_A[15:8] >= 8'hD2) && (DATA_A[15:8] < 8'hFE))
        begin  
          DATA_A[15:8] = DATA_A[15:8]+2;
          DATA_A[7:0] = DATA_A[7:0];
        end
        else
        begin 
          DATA_A[15:8] = 8'h03;
          DATA_A[7:0] = 8'h04;
          reverse_a = 1;                //reverse_a the disparity
        end
        if (reverse_a && DATA_A[15:8] == 8'h05)
        begin  
          DATA_A[15:8] = 8'h00;
          DATA_A[7:0] = 8'h01;
          reverse_a = 0;                //once disparity is reverse_ad,
        end                             //initialize to zero and run all codes again
      end
      if (!pd_all && !pdb)
      begin
        if (DATA_B[15:8] < 8'hD2)
        begin 
          DATA_B[15:8] = DATA_B[15:8]+2;
          DATA_B[7:0] = DATA_B[7:0]+2;
        end
        else
        if ((DATA_B[15:8] >= 8'hD2) && (DATA_B[15:8] < 8'hFE))
        begin 
          DATA_B[15:8] = DATA_B[15:8]+2;
          DATA_B[7:0] = DATA_B[7:0];
        end
        else
        begin 
          DATA_B[15:8] = 8'h13;
          DATA_B[7:0] = 8'h14;
          reverse_b = 1;                //reverse_a the disparity
        end
        if (reverse_b && DATA_B[15:8] == 8'h15)
        begin 
          DATA_B[15:8] = 8'h10;
          DATA_B[7:0] = 8'h11;
          reverse_b = 0;                //once disparity is reverse_ad,
        end                             //initialize to zero and run all codes again
      end
    end
endmodule

