/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2015                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.0     10:08:58 09/09/2015 Gary Yu                                *
* File Created                                                                *
*                                                                             *
* Additional Comments:                                                        *
*                                                                             *
******************************************************************************/

`timescale 10ps/1ps
    
module ftdi_sim #
(
parameter   FIFOCLK_PERIOD   =    1666.6667, //60M
parameter   I2C_ADDRESS      =    7'h25
)
(
K_,//gyu pass parameter

PWR_RESET,

rx_sysref_en,

sck,
scs,
win_1st_word,
win_2nd_word,
win_3td_word,
win_4th_word,

ACT,
RpWn,
ApDn,
RST,

ADhp,

wrn_in,
txen_gen,
fifo_din,
fifoclk_gen

);

parameter MPSSE_PERIOD   = 3.75 * FIFOCLK_PERIOD;

input [4:0]   K_;//gyu pass parameter





      // 0x008: Enable ILA
    parameter JESD204B008WD = 32'h00000001;

      // 0x00C: Scrambling dissabled
    parameter JESD204B00CWD = 32'h00000000;

      // 0x010: Sysref once
    parameter JESD204B010WD = 32'h00000001;//gyu different with spec.  

      // 0x014: Tx Only register

      // 0x018: Test mode : Normal operation
    parameter JESD204B018WD = 32'h00000000;  

      // 0x020: Octets per Frame F = 2
    parameter JESD204B020WD = 32'h00000001;  
        
      // 0x024: Frames per Multiframe K
      
      // 0x028: Lanes in use 4 lines
    parameter JESD204B028WD = 32'h00000003;

      // 0x02C: Device subclass 1
    parameter JESD204B02CWD = 32'h00000001;

      // 0x030: Rx buffer delay
    parameter JESD204B030WD = 32'h00000000;

      // 0x034: Error reporting via ~sync
    parameter JESD204B034WD = 32'h00000000;


parameter LTC2123X1WD    = 32'h00000082;//32'h80000002
parameter LTC2123X2WD    = 32'h00000001;//32'h40000001
parameter LTC2123X3WD    = 32'h00000000;//32'ha0000003
parameter LTC2123X4WD    = 32'h00000009;//32'hf0000009

`include "/home/gyu/p2123_080315/sim/testbench/add_sim.h"
`include "/home/gyu/p2123_080315/src/fpga/cfg_define.h"

reg [7:0] RSV;
 

reg [7:0] ID_REG;
`define   ID_REG_VALU             ID_REG


reg [3:0] MEMSIZE_SET;
reg [2:0] CHSEL_SET;
`define   CAP_CFIG_REG_VALU           {MEMSIZE_SET, CHSEL_SET, RSV[0]}


reg       START_SET;
reg       FETONLY_SET;
`define   CAP_CTRL_REG_VALU           {RSV[7:2], FETONLY_SET, START_SET}

reg       CTRLRST_SET;
`define   CAP_RST_REG_VALU            {RSV[7:1], CTRLRST_SET}


reg [3:0] CHVLID_DET;

reg       SYNCERR_DET;
reg       FETDONE_DET;
reg       CAPDONE_DET;

`define   CAP_STATUS_REG_VALU         {CHVLID_DET, RSV[3], SYNCERR_DET, FETDONE_DET, CAPDONE_DET}

reg [2:0] SPI_SET;
reg       RCLK_SYNC_SET;
reg       SREF_SYNC_SET;

`define   SPI_CFIG_REG_VALU           {RSV[7:5], RCLK_SYNC_SET, SREF_SYNC_SET, SPI_SET}

reg       ARST_DET;
reg       SYNC_DET;
reg       SYSLK_DET;     
reg       CORELK_DET;
reg       FIFOLK_DET;
`define   CLK_STATUS_REG_VALU         {RSV[7:5], ARST_DET, SYNC_DET, SYSLK_DET, CORELK_DET, FIFOLK_DET}

reg [31:0] JESD204B_READ_VALU; 
      
reg [7:0] JESD204B_WB0_REG;
`define   JESD204B_WB0_REG_VALU       JESD204B_WB0_REG

reg [7:0] JESD204B_WB1_REG;
`define   JESD204B_WB1_REG_VALU       JESD204B_WB1_REG

reg [7:0] JESD204B_WB2_REG;
`define   JESD204B_WB2_REG_VALU       JESD204B_WB2_REG

reg [7:0] JESD204B_WB3_REG;
`define   JESD204B_WB3_REG_VALU       JESD204B_WB3_REG

reg [5:0] WINDEX_SET;
reg       WRCE_SET;
reg       WRACK_DET;
`define   JESD204B_CFIG_REG_VALU      {WINDEX_SET, WRCE_SET, WRACK_DET}

reg [7:0] JESD204B_RB0_REG;
`define   JESD204B_RB0_REG_VALU       JESD204B_RB0_REG

reg [7:0] JESD204B_RB1_REG;
`define   JESD204B_RB1_REG_VALU       JESD204B_RB1_REG

reg [7:0] JESD204B_RB2_REG;
`define   JESD204B_RB2_REG_VALU       JESD204B_RB2_REG

reg [7:0] JESD204B_RB3_REG;
`define   JESD204B_RB3_REG_VALU       JESD204B_RB3_REG

reg [5:0] RINDEX_SET;
reg       RDCE_SET;
reg       RDACK_DET;
`define   JESD204B_CHK_REG_VALU      {RINDEX_SET, RDCE_SET, RDACK_DET}

reg       SDA_SDT;
reg       SCL_SET;
`define   I2C_ACSS_REG_VALU          {RSV[7:2], SCL_SET, SDA_SDT}

reg [5:0] W2INDEX_SET;
`define   JESD204B_W2I_REG_VALU      {RSV[7:6], W2INDEX_SET}

reg [5:0] R2INDEX_SET;
`define   JESD204B_R2I_REG_VALU      {RSV[7:6], R2INDEX_SET}

input     PWR_RESET;

output    rx_sysref_en;

input     sck;
output    scs;

output [31:0] win_1st_word;
output [31:0] win_2nd_word;
output [31:0] win_3td_word;
output [31:0] win_4th_word;

output       ACT;
output       RpWn;
output       ApDn;
output       RST;


inout  [7:0] ADhp;


input        wrn_in;
output       txen_gen;
input  [7:0] fifo_din;  
output       fifoclk_gen;

reg          rx_sysref_en;

reg           scs;
reg    [31:0] win_1st_word;
reg    [31:0] win_2nd_word;
reg    [31:0] win_3td_word;
reg    [31:0] win_4th_word;
                                                                                                             
reg          ACT;
reg          RpWn;
reg          ApDn;
reg          RST;

reg          ADwr;

reg          FIFOmd;

wire  [7:0]  ADhp_in; 
reg   [7:0]  ADhp_out;

assign #4.5  ADhp    = (ADwr) ? ADhp_out : 8'hz;
assign #1    ADhp_in =  ADhp;

reg         fifoclk_reg;
wire        fifoclk_gen;
                         
integer i, j, seed0, seed1, seed2;


always  
  #(FIFOCLK_PERIOD/2) fifoclk_reg = !fifoclk_reg;
  
sqclkd sqclkd_fifoclk(fifoclk_reg, FIFOmd, fifoclk_gen);


reg         rdclk_reg;
wire        rdclk;

always  
  #(FIFOCLK_PERIOD * 1.5) rdclk_reg = !rdclk_reg;

    
sqclkd sqclkd_rdclk(rdclk_reg, FIFOmd, rdclk);


wire fifo_full;
reg FIFOrd;
wire [7:0] fifo_dout;

wire empty;
 
wire #1 wr_en = ~(wrn_in | fifo_full);//gyu handshake 
reg rd_en;

wire [9:0] rd_data_count, wr_data_count;


reg rd_en_disable;
always @ (posedge rdclk or negedge FIFOrd)
   if (~FIFOrd)
     rd_en   <= #1 1'b0;
   else if (rd_en_disable)
     rd_en   <= #1 1'b0;    
   else 
     rd_en   <= #1 1'b1;
         
always @ (posedge rdclk or negedge FIFOrd)
   if (~FIFOrd)
     rd_en_disable   <= #1 1'b0;  
   else if (rd_data_count ==  10'h003)
     rd_en_disable   <= #1 1'b1;  
        
                      
xlfifo_cp0 xlfifo_cp0(

//independent clk fifo  
  .rst(PWR_RESET),
  
//write  
  .wr_clk(fifoclk_gen),
  .wr_en(wr_en),
  .din(fifo_din),
  
  .full(),
  .prog_full(fifo_full),
  .wr_data_count(wr_data_count),


//read      
  .rd_clk(rdclk),
  .rd_en(rd_en),
  .dout(fifo_dout),

  .empty(empty),  
  .rd_data_count(rd_data_count)

);

reg [7:0] check_mem [0:4096];

reg [63:0] mem_ckin [0:32768 - 1];//256K bytes/128K samples
reg [63:0] mem1_ckin [0:32768 - 1];//256K bytes/128K samples

     
wire mem_ckin_clk          = demo_tb.DUT.bram_a_clk; 
wire mem_ckin_wren         = demo_tb.DUT.bram_wren;

wire [63:0] test_tb;

wire [14:0] mem_ckin_addr  = demo_tb.DUT.blk_mem_gen_gcp.addra;
                             
always @(posedge mem_ckin_clk)
  if(mem_ckin_wren)
  begin
    mem_ckin[mem_ckin_addr] <= #1 demo_tb.DUT.blk_mem_gen_gcp.dina [63 : 0];
    mem1_ckin[mem_ckin_addr] <= #1 demo_tb.DUT.blk_mem_gen_gcp.dina [127 : 64];
  end
    
reg [7:0]  mem_ckout [0:32768 * 8 - 1];//256K bytes/128K samples

reg        rd_en_reg;
reg [11:0] check_cnt;

always @(posedge rdclk or posedge PWR_RESET)
  if(PWR_RESET)
    rd_en_reg <= #1 1'b0;
  else
    rd_en_reg <= #1 rd_en;      

    
always @(posedge rdclk or negedge rd_en_reg)
  if(~rd_en_reg)
    check_cnt <= #1 12'b0;
  else
  begin
      check_mem[check_cnt] <= #1 fifo_dout;
      check_cnt            <= #1 check_cnt + 1;
  end             

assign #1 txen_gen = FIFOmd ? fifo_full : 1'bz;
      
initial 
begin
//constant initial value

  i                      = 0;
  j                      = 0;
  seed0                  = 3;
  seed1                  = 1;
  seed2                  = 2;

//memory initial value
  for (i = 0; i < 4097; i = i + 1)
    check_mem[i]  = 8'h55;
 
  for (i = 0; i < 32768; i = i + 1)
    mem_ckin[i]   = 64'b0;
  
  for (i = 0; i < 32768; i = i + 1)
    mem1_ckin[i]   = 64'b0;
    
  for (i = 0; i < 32768 * 8; i = i + 1)  
    mem_ckout[i]  = 8'b0;
  
    
    
//Registers initial value after power up
  rx_sysref_en           = 1'b0;
  fifoclk_reg            = 1'b1;
  rdclk_reg              = 1'b1; 
  
  scs                    = 1'b1;
  
  win_1st_word           = 32'b0;
  win_2nd_word           = 32'b0;
  win_3td_word           = 32'b0;
  win_4th_word           = 32'b0;
  
  ACT                    = 1'bz;
  RpWn                   = 1'bz;
  ApDn                   = 1'bz;
  RST                    = 1'bz;

  ADhp_out               = 8'b0;
      
  ADwr                   = 1'b0;  
  FIFOmd                 = 1'b0;
  FIFOrd                 = 1'b0;

  RSV                    = 8'b0;

  ID_REG                 = 8'b0;

  MEMSIZE_SET            = buf_128SPS;
  CHSEL_SET              = ch01_sel;//channel 0&1

  START_SET              = 1'b0;
  FETONLY_SET            = 1'b0;
  CTRLRST_SET            = 1'b0;
  
  CHVLID_DET             = 4'b0;
  
  SYNCERR_DET            = 1'b0;
  FETDONE_DET            = 1'b0;
  CAPDONE_DET            = 1'b0;

  SPI_SET                = adca_sel;
  RCLK_SYNC_SET          = 1'b0;
  SREF_SYNC_SET          = 1'b0;

  SYSLK_DET              = 1'b0;     
  CORELK_DET             = 1'b0;
  FIFOLK_DET             = 1'b0;
                                           
  JESD204B_READ_VALU     = 32'b0;
                                                                    
  JESD204B_WB0_REG       = 8'b0;
  JESD204B_WB1_REG       = 8'b0;
  JESD204B_WB2_REG       = 8'b0;
  JESD204B_WB3_REG       = 8'b0;
  WINDEX_SET             = 6'b0;
  W2INDEX_SET            = 6'b0;
  WRCE_SET               = 1'b0;
  WRACK_DET              = 1'b0;
  
  JESD204B_RB0_REG       = 8'b0;
  JESD204B_RB1_REG       = 8'b0;
  JESD204B_RB2_REG       = 8'b0;
  JESD204B_RB3_REG       = 8'b0;
  RINDEX_SET             = 6'b0;
  R2INDEX_SET            = 6'b0;
  RDCE_SET               = 1'b0;
  RDACK_DET              = 1'b0;

  SDA_SDT                = 1'b1;
  SCL_SET                = 1'b1;     
    
//FTDI Initialization -------------------------------------------------------------------
       
//Configure FTDI as MPSSE mode
//  #(108 * MPSSE_PERIOD);//gyu v6p0
  #(5 * MPSSE_PERIOD);//gyu v6p0
  FIFOmd                 = 1'b0;
  
  #(8 * MPSSE_PERIOD);
  ADwr                   = 1'b0;
  
  #(MPSSE_PERIOD);
  ACT                    = 1'b0;
  RpWn                   = 1'b0;
  ApDn                   = 1'b0;
  RST                    = 1'b1;

//Generate a reset pulse to FPGA on GPIOL3/BDBUS7 of FTDI
//  #(508 * MPSSE_PERIOD);//gyu v6p0
  #(4 * MPSSE_PERIOD);//gyu v6p0
  RST                    = 1'b0;
  
  #(8 * MPSSE_PERIOD);
  RST                    = 1'b1;

//Check FPGA ID register by FTDI configuration port 
//to verify FPGA image is correct  
  #(8 * MPSSE_PERIOD);
  ftdi_rd_seq (ID_REG_ADDR, `ID_REG_VALU);
  ftdi_wr_seq (ID_REG_ADDR, 8'h68);
  ftdi_rd_seq (ID_REG_ADDR, `ID_REG_VALU);

//Check FPGA Clock status register by FTDI configuration port 
//to verify system clock from K7 board is available 
  #(8 * MPSSE_PERIOD);
  SYSLK_DET = 1'b0;
  while (SYSLK_DET == 1'b0)
  begin
    # (MPSSE_PERIOD);
    ftdi_rd_seq (CLK_STATUS_REG_ADDR, `CLK_STATUS_REG_VALU);
  end
      
//Check board ID by I2C interface
  #(8 * MPSSE_PERIOD);   
  //start condition
  SDA_SDT = 1'b1;
  SCL_SET = 1'b1;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SDA_SDT = 1'b0;
  SCL_SET = 1'b1;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SDA_SDT = 1'b0;
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
     
  //1st bit of address
  SDA_SDT = I2C_ADDRESS[6];
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b1;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);   
     
  //2nd bit of address
  SDA_SDT = I2C_ADDRESS[5];
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b1;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
     
  //3rd bit of address
  SDA_SDT = I2C_ADDRESS[4];
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b1;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);    
     
  //4th bit of address
  SDA_SDT = I2C_ADDRESS[3];
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b1;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);       
     
  //5th bit of address
  SDA_SDT = I2C_ADDRESS[2];
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b1;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);  
     
  //6th bit of address
  SDA_SDT = I2C_ADDRESS[1];
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b1;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);  
       
  //7th bit of address
  SDA_SDT = I2C_ADDRESS[0];
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b1;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
     
  //issue read command
  SDA_SDT = I2C_READ_BIT;
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b1;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);  
  
  //read ack
  SDA_SDT = 1'b1;
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b1;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  ftdi_rd_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  SCL_SET = 1'b0;
  ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  if (SDA_SDT)
  begin
    //stop condition
    SDA_SDT = 1'b0;
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SDA_SDT = 1'b0;
    SCL_SET = 1'b1;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SDA_SDT = 1'b1;
    SCL_SET = 1'b1;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  end
  else
  begin
    //read 1st bit
    SDA_SDT = 1'b1;
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b1;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    ftdi_rd_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  
    //read 2nd bit
    SDA_SDT = 1'b1;
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b1;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    ftdi_rd_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  
    //read 3rd bit
    SDA_SDT = 1'b1;
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b1;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    ftdi_rd_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);      
  
    //read 4th bit
    SDA_SDT = 1'b1;
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b1;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    ftdi_rd_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    
    //read 5th bit
    SDA_SDT = 1'b1;
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b1;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    ftdi_rd_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    
    //read 6th bit
    SDA_SDT = 1'b1;
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b1;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    ftdi_rd_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  
    //read 7th bit
    SDA_SDT = 1'b1;
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b1;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    ftdi_rd_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    
    //read 8th bit
    SDA_SDT = 1'b1;
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b1;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    ftdi_rd_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    
    //send ack
    SDA_SDT = 1'b0;
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b1;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  
    //stop condition
    SDA_SDT = 1'b0;
    SCL_SET = 1'b0;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SDA_SDT = 1'b0;
    SCL_SET = 1'b1;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
    SDA_SDT = 1'b1;
    SCL_SET = 1'b1;
    ftdi_wr_seq (I2C_ACSS_REG_ADDR, `I2C_ACSS_REG_VALU);
  end
         
  
    

//Set reference clock generator as SPI slave by FTDI configuration port
  #(8 * MPSSE_PERIOD);       
  SPI_SET                = clkgen_sel;
  ftdi_wr_seq (SPI_CFIG_REG_ADDR, `SPI_CFIG_REG_VALU);
//Configure reference clock generator through SPI by FTDI  
  win_1st_word           = LTC2123X1WD;
  win_2nd_word           = LTC2123X4WD;
  win_3td_word           = LTC2123X2WD;
  win_4th_word           = LTC2123X3WD;
  #(8 * MPSSE_PERIOD);        
  @(negedge sck);
  scs                    = 1'b0;
  #885000;
  scs                    = 1'b1;
  
//Turn on reference clock           
  #(8 * MPSSE_PERIOD);       
  RCLK_SYNC_SET          = 1'b1;
  ftdi_wr_seq (SPI_CFIG_REG_ADDR, `SPI_CFIG_REG_VALU);
  
//Reset capture engine and FCLK PLL
  #(8 * MPSSE_PERIOD);
  CTRLRST_SET                = 1'b1;
  ftdi_wr_seq (CAP_RST_REG_ADDR, `CAP_RST_REG_VALU);
    
//Set sysref generator as SPI slave by FTDI configuration port
  #(8 * MPSSE_PERIOD);       
  SPI_SET                = sysref_sel;
  ftdi_wr_seq (SPI_CFIG_REG_ADDR, `SPI_CFIG_REG_VALU);
//Configure sysref generator through SPI by FTDI  
  win_1st_word           = LTC2123X1WD;
  win_2nd_word           = LTC2123X3WD;
  win_3td_word           = LTC2123X4WD;
  win_4th_word           = LTC2123X2WD;
  #(8 * MPSSE_PERIOD);        
  @(negedge sck);
  scs                    = 1'b0;
  #885000;
  scs                    = 1'b1;
  
//Turn on sysref             
  #(8 * MPSSE_PERIOD);       
  SREF_SYNC_SET          = 1'b1;
  ftdi_wr_seq (SPI_CFIG_REG_ADDR, `SPI_CFIG_REG_VALU);
  
//Set ADC2 as SPI slave by FTDI configuration port
  #(8 * MPSSE_PERIOD);       
  SPI_SET                = adcb_sel;
  ftdi_wr_seq (SPI_CFIG_REG_ADDR, `SPI_CFIG_REG_VALU);
//Configure ADC2 through SPI by FTDI  
  win_1st_word           = LTC2123X4WD;
  win_2nd_word           = LTC2123X3WD;
  win_3td_word           = LTC2123X2WD;
  win_4th_word           = LTC2123X1WD;
  #(8 * MPSSE_PERIOD);        
  @(negedge sck);
  scs                    = 1'b0;
  #885000;
  scs                    = 1'b1;
  

//Set ADC1 as SPI slave by FTDI configuration port
  #(8 * MPSSE_PERIOD);       
  SPI_SET                = adca_sel;
  ftdi_wr_seq (SPI_CFIG_REG_ADDR, `SPI_CFIG_REG_VALU);
//Configure ADC1 through SPI by FTDI  
  win_1st_word           = LTC2123X1WD;
  win_2nd_word           = LTC2123X2WD;
  win_3td_word           = LTC2123X3WD;
  win_4th_word           = LTC2123X4WD;
  #(8 * MPSSE_PERIOD);        
  @(negedge sck);
  scs                    = 1'b0;
  #885000;
  scs                    = 1'b1;
  
  
$display("Configure JESD204B core by FTDI configuration port through AXI interface");

      // 0x000: Read Version
      jesd204b_rd_seq(12'h000, JESD204B_READ_VALU);
      $display("Version = Major %d Minor %d Rev %d", JESD204B_READ_VALU[31:24],  JESD204B_READ_VALU[23:16],  JESD204B_READ_VALU[15:8]);

      // 0x004: Reset later once configured

      // 0x008: 
      jesd204b_wr_seq(12'h008, JESD204B008WD);

      // 0x00C: 
      jesd204b_wr_seq(12'h00c, JESD204B00CWD);

      // 0x010: 
      jesd204b_wr_seq(12'h010, JESD204B010WD);//gyu different with spec.

      // 0x014: 

      // 0x018: Test mode : Normal operation
      jesd204b_wr_seq(12'h018, JESD204B018WD);

      // 0x020: 
      jesd204b_wr_seq(12'h020, JESD204B020WD);

      // 0x024: 
      jesd204b_wr_seq(12'h024, {27'b0, K_});//gyu pass parameter

      // 0x028: 
      jesd204b_wr_seq(12'h028, JESD204B028WD);

      // 0x02C: 
      jesd204b_wr_seq(12'h02c, JESD204B02CWD);

      // 0x030: 
      jesd204b_wr_seq(12'h030, JESD204B030WD);

      // 0x034: 
      jesd204b_wr_seq(12'h034, JESD204B034WD);

        
//Check Capture status register by FTDI configuration 
//port to verify JESD204B data outputs are valid
  #(8 * MPSSE_PERIOD);
  CHVLID_DET = 4'b0;
  
    
  i = 0;    
  while ((CHVLID_DET == 4'b0) && (i != 7))
  begin
    # (MPSSE_PERIOD);
    if (( i == 0) || ( i == 3) || ( i == 6))
    begin
      // Link configuration has changed so reset the interface
      // 0x004: Write reset
      jesd204b_wr_seq(12'h004,32'h00000001);
      // Now poll register until reset has cleared
      JESD204B_READ_VALU = 32'h00000001;
      while (JESD204B_READ_VALU !== 32'b0 )
      begin
        # (MPSSE_PERIOD); //wait for a time then read
        jesd204b_rd_seq(12'h004, JESD204B_READ_VALU);
      end
      $display("%d Reset complete..", i);

      // 0x038 Poll status until Rx is in sync
      JESD204B_READ_VALU = 32'h00000000;
      while (JESD204B_READ_VALU[0] !== 1'b1 )
      begin
        # (MPSSE_PERIOD); //wait for a time then read
        jesd204b_rd_seq(12'h038, JESD204B_READ_VALU);
      end
      $display("%d Sync complete..", i);    
    end
    ftdi_rd_seq (CAP_STATUS_REG_ADDR, `CAP_STATUS_REG_VALU);
    i = i + 1;
  end
  
  if (i!= 7)
  begin
      #(8 * MPSSE_PERIOD);   
      //read RX registers to check link configuration data
      JESD204B_READ_VALU = 0;
      jesd204b_rd_seq(12'h01c, JESD204B_READ_VALU);
      $display("Address 0x01c: %h", JESD204B_READ_VALU);
      
      
      JESD204B_READ_VALU = 0;
      jesd204b_rd_seq(12'h03c, JESD204B_READ_VALU);
      $display("Address 0x03c: %h", JESD204B_READ_VALU);
            
      JESD204B_READ_VALU = 0;       
      jesd204b_rd_seq(12'h800, JESD204B_READ_VALU);
      $display("Address 0x800: %h", JESD204B_READ_VALU);    
        
      JESD204B_READ_VALU = 0;      
      jesd204b_rd_seq(12'h804, JESD204B_READ_VALU);
      $display("Address 0x804: %h", JESD204B_READ_VALU);      
      
      JESD204B_READ_VALU = 0;
      jesd204b_rd_seq(12'h808, JESD204B_READ_VALU);
      $display("Address 0x808: %h", JESD204B_READ_VALU);      
      
      JESD204B_READ_VALU = 0;
      jesd204b_rd_seq(12'h80c, JESD204B_READ_VALU);
      $display("Address 0x80c: %h", JESD204B_READ_VALU);      
      
      JESD204B_READ_VALU = 0;
      jesd204b_rd_seq(12'h810, JESD204B_READ_VALU);
      $display("Address 0x810: %h", JESD204B_READ_VALU);      
      
      JESD204B_READ_VALU = 0;
      jesd204b_rd_seq(12'h814, JESD204B_READ_VALU);
      $display("Address 0x814: %h", JESD204B_READ_VALU);      
      
      JESD204B_READ_VALU = 0;
      jesd204b_rd_seq(12'h818, JESD204B_READ_VALU);
      $display("Address 0x818: %h", JESD204B_READ_VALU);      
      
      JESD204B_READ_VALU = 0;
      jesd204b_rd_seq(12'h840, JESD204B_READ_VALU);
      $display("Address 0x840: %h", JESD204B_READ_VALU);      
      
      JESD204B_READ_VALU = 0;
      jesd204b_rd_seq(12'h844, JESD204B_READ_VALU);
      $display("Address 0x844: %h", JESD204B_READ_VALU);      
      
      JESD204B_READ_VALU = 0;
      jesd204b_rd_seq(12'h848, JESD204B_READ_VALU);
      $display("Address 0x868: %h", JESD204B_READ_VALU);      
      
      JESD204B_READ_VALU = 0;
      jesd204b_rd_seq(12'h84c, JESD204B_READ_VALU);
      $display("Address 0x84c: %h", JESD204B_READ_VALU);      
      
      JESD204B_READ_VALU = 0;
      jesd204b_rd_seq(12'h850, JESD204B_READ_VALU);
      $display("Address 0x850: %h", JESD204B_READ_VALU);      
      
      JESD204B_READ_VALU = 0;
      jesd204b_rd_seq(12'h854, JESD204B_READ_VALU);
      $display("Address 0x854: %h", JESD204B_READ_VALU);      
      
      JESD204B_READ_VALU = 0;
      jesd204b_rd_seq(12'h858, JESD204B_READ_VALU);
      $display("Address 0x858: %h", JESD204B_READ_VALU);      
  end
  else
      $display("Can't make valid date ouput..");  

            
//Check Clock status register by to verify JESD204B core embedded PLL was locked and sync signal was set

  #(8 * MPSSE_PERIOD);
  ARST_DET               = 1'b0;
  SYNC_DET               = 1'b0;  
  SYSLK_DET              = 1'b0;     
  CORELK_DET             = 1'b0;
  while ((ARST_DET == 1'b0)||(SYNC_DET == 1'b0)||(SYSLK_DET == 1'b0)||(CORELK_DET == 1'b0))
  begin
    # (MPSSE_PERIOD);
    ftdi_rd_seq (CLK_STATUS_REG_ADDR, `CLK_STATUS_REG_VALU);
  end

        
//==================================================================================

  
//Set buffer size, channel number and capture channel -----------------------------------
//by FTDI configuration port
  #(8 * MPSSE_PERIOD);
  MEMSIZE_SET            = buf_128SPS;
  CHSEL_SET              = ch01_sel;
  ftdi_wr_seq (CAP_CFIG_REG_ADDR, `CAP_CFIG_REG_VALU);
        
//Start to capture data to buffer by FTDI configuration port
  #(8 * MPSSE_PERIOD);
  START_SET              = 1'b1;
  FETONLY_SET            = 1'b0;
  ftdi_wr_seq (CAP_CTRL_REG_ADDR, `CAP_CTRL_REG_VALU);
  
//Check Capture status register by FTDI configuration 
//port to verify buffer was full
  #(8 * MPSSE_PERIOD);
  CAPDONE_DET = 1'b0;
  while (CAPDONE_DET == 1'b0)
  begin
    # (MPSSE_PERIOD); 
    ftdi_rd_seq (CAP_STATUS_REG_ADDR, `CAP_STATUS_REG_VALU);
  end

//Configure FTDI as Sync FIFO mode  
  #(8 * MPSSE_PERIOD);
  FIFOmd                 = 1'b1;

  #(8 * MPSSE_PERIOD);
  ADwr                   = 1'b0;
  
  #(MPSSE_PERIOD);
  ACT                    = 1'bz;
  RpWn                   = 1'bz;
  ApDn                   = 1'bz;
  RST                    = 1'bz;

    
//Check if all data is in fifo
  #(8 * MPSSE_PERIOD);
  while (({9'b0, wr_data_count} != FIFO_128SPS - 1) && (~fifo_full))
    @(posedge fifoclk_gen);
  
//Pull data from FIFO of FTDI untill reach the set size   
  #(8 * MPSSE_PERIOD); 
  FIFOrd                 = 1'b1;
    
  #(8 * MPSSE_PERIOD);
  while (empty == 1'b0)
  begin
    # (MPSSE_PERIOD);
  end

  #(8 * MPSSE_PERIOD);
  FIFOrd                 = 1'b0;
  
for(i = 0; i < FIFO_128SPS/8; i = i + 1)
 begin
   mem_ckout[i * 8 + 7]        = mem_ckin[i][63:56];
   mem_ckout[i * 8 + 6]        = mem_ckin[i][55:48];
   mem_ckout[i * 8 + 5]        = mem_ckin[i][47:40];
   mem_ckout[i * 8 + 4]        = mem_ckin[i][39:32];
   mem_ckout[i * 8 + 3]        = mem_ckin[i][31:24];
   mem_ckout[i * 8 + 2]        = mem_ckin[i][23:16];
   mem_ckout[i * 8 + 1]        = mem_ckin[i][15:8];
   mem_ckout[i * 8 + 0]        = mem_ckin[i][7:0];  
 end
 
//Compare reference data and the data from fifo of FTDI. this comparing can't present the problem of JESD204B transition  
j = 0;
for (i = 0; i < FIFO_128SPS; i = i + 1)
  if (mem_ckout[i] == check_mem[i])
    j = j + 1;
  else  
    $display ("Data Error: addr %h, ref %h, data %h",i,mem_ckout[i],check_mem[i]);

if(j == FIFO_128SPS)
  $display ("Capture is successful!");
                  
//Configure FTDI as MPSSE mode
  #(108 * MPSSE_PERIOD);
  FIFOmd                 = 1'b0;
  
  #(8 * MPSSE_PERIOD);
  ADwr                   = 1'b0;
  
  #(MPSSE_PERIOD);
  ACT                    = 1'b0;
  RpWn                   = 1'b0;
  ApDn                   = 1'b0;
  RST                    = 1'b1;

  
  
  
//Reset capture engine and FCLK PLL
  #(8 * MPSSE_PERIOD);
  CTRLRST_SET                = 1'b1;
  ftdi_wr_seq (CAP_RST_REG_ADDR, `CAP_RST_REG_VALU);
  
      

//Set buffer size, channel number and capture channel -----------------------------------
//by FTDI configuration port
  #(8 * MPSSE_PERIOD);
  MEMSIZE_SET            = buf_128SPS;
  CHSEL_SET              = ch23_sel;
  ftdi_wr_seq (CAP_CFIG_REG_ADDR, `CAP_CFIG_REG_VALU);
        
//Start to capture data to buffer by FTDI configuration port
  #(8 * MPSSE_PERIOD);
  START_SET              = 1'b1;
  FETONLY_SET            = 1'b1;
  ftdi_wr_seq (CAP_CTRL_REG_ADDR, `CAP_CTRL_REG_VALU);
  
//Check Capture status register by FTDI configuration 
//port to verify buffer was full
  #(8 * MPSSE_PERIOD);
  CAPDONE_DET = 1'b0;
  while (CAPDONE_DET == 1'b0)
  begin
    # (MPSSE_PERIOD); 
    ftdi_rd_seq (CAP_STATUS_REG_ADDR, `CAP_STATUS_REG_VALU);
  end

//Configure FTDI as Sync FIFO mode  
  #(8 * MPSSE_PERIOD);
  FIFOmd                 = 1'b1;

  #(8 * MPSSE_PERIOD);
  ADwr                   = 1'b0;
  
  #(MPSSE_PERIOD);
  ACT                    = 1'bz;
  RpWn                   = 1'bz;
  ApDn                   = 1'bz;
  RST                    = 1'bz;

    
//Check if all data is in fifo
  #(8 * MPSSE_PERIOD);
  while (({9'b0, wr_data_count} != FIFO_128SPS - 1) && (~fifo_full))
    @(posedge fifoclk_gen);
  
//Pull data from FIFO of FTDI untill reach the set size   
  #(8 * MPSSE_PERIOD); 
  FIFOrd                 = 1'b1;
    
  #(8 * MPSSE_PERIOD);
  while (empty == 1'b0)
  begin
    # (MPSSE_PERIOD);
  end

  #(8 * MPSSE_PERIOD);
  FIFOrd                 = 1'b0;
  
for(i = 0; i < FIFO_128SPS/8; i = i + 1)
 begin
   mem_ckout[i * 8 + 7]        = mem1_ckin[i][63:56];
   mem_ckout[i * 8 + 6]        = mem1_ckin[i][55:48];
   mem_ckout[i * 8 + 5]        = mem1_ckin[i][47:40];
   mem_ckout[i * 8 + 4]        = mem1_ckin[i][39:32];
   mem_ckout[i * 8 + 3]        = mem1_ckin[i][31:24];
   mem_ckout[i * 8 + 2]        = mem1_ckin[i][23:16];
   mem_ckout[i * 8 + 1]        = mem1_ckin[i][15:8];
   mem_ckout[i * 8 + 0]        = mem1_ckin[i][7:0];  
 end
 
//Compare reference data and the data from fifo of FTDI. this comparing can't present the problem of JESD204B transition    
j = 0;
for (i = 0; i < FIFO_128SPS; i = i + 1)
  if (mem_ckout[i] == check_mem[i])
    j = j + 1;
  else  
    $display ("Data Error: addr %h, ref %h, data %h",i,mem_ckout[i],check_mem[i]);

if(j == FIFO_128SPS)
  $display ("Capture is successful!");
                  
//Configure FTDI as MPSSE mode
  #(108 * MPSSE_PERIOD);
  FIFOmd                 = 1'b0;
  
  #(8 * MPSSE_PERIOD);
  ADwr                   = 1'b0;
  
  #(MPSSE_PERIOD);
  ACT                    = 1'b0;
  RpWn                   = 1'b0;
  ApDn                   = 1'b0;
  RST                    = 1'b1;


  
  
  
//Reset capture engine and FCLK PLL
  #(8 * MPSSE_PERIOD);
  CTRLRST_SET                = 1'b1;
  ftdi_wr_seq (CAP_RST_REG_ADDR, `CAP_RST_REG_VALU);
       
  
//==================================================================================

  
end


task ftdi_wr_seq;
input [7:0] FTDI_WRITE_ADDR;
input [7:0] FTDI_WRITE_IN;   
begin
//write data
        //initial value
        //ACT  = 1'b0;
        //RpWn = 1'b0;
        //ApDn = 1'b0;
        //RST  = 1'b1;
        //ADwr = 1'b0;//ADhp = 8'bz;
        // -------------------------------------
        #(MPSSE_PERIOD);
        ADwr = 1'b1;//host drive AD
        // -------------------------------------
        #(MPSSE_PERIOD);
        ADhp_out = FTDI_WRITE_ADDR;//send address
        // -------------------------------------
        #(MPSSE_PERIOD);
        ApDn = 1'b1;//address flag
        // -------------------------------------
        #(MPSSE_PERIOD);
        ACT  = 1'b1;//action
        // -------------------------------------
        #(MPSSE_PERIOD);
        ApDn = 1'b0;//address de-flag
        ACT  = 1'b0;//de-action
        // -------------------------------------
        #(MPSSE_PERIOD);
        ADwr = 1'b0;//host release AD
        
        
        // -------------------------------------
        #(4 * MPSSE_PERIOD);
        ADwr = 1'b1;//host drive AD
        // -------------------------------------
        #(MPSSE_PERIOD);
        ADhp_out = FTDI_WRITE_IN;//Send data
        // -------------------------------------
        #(MPSSE_PERIOD);
        ACT  = 1'b1;//action
        // -------------------------------------
        #(MPSSE_PERIOD);
        ACT  = 1'b0;//de-action
        // -------------------------------------
        #(MPSSE_PERIOD);
        ADwr = 1'b0;//host release AD      
end
endtask




task ftdi_rd_seq;
input  [7:0] FTDI_READ_ADDR;
output [7:0] FTDI_READ_OUT;
begin
//read data
        //initial value
        //ACT  = 1'b0;
        //RpWn = 1'b0;
        //ApDn = 1'b0;
        //RST  = 1'b1;
        //ADwr = 1'b0;//ADhp = 8'bz;
        // -------------------------------------
        #(MPSSE_PERIOD);
        ADwr = 1'b1;//host drive AD
        // -------------------------------------
        #(MPSSE_PERIOD);
        ADhp_out = FTDI_READ_ADDR;//send address
        // -------------------------------------
        #(MPSSE_PERIOD);
        ApDn = 1'b1;//address flag
        // -------------------------------------
        #(MPSSE_PERIOD);
        ACT  = 1'b1;//action
        // -------------------------------------
        #(MPSSE_PERIOD);
        ApDn = 1'b0;//address de-flag
        ACT  = 1'b0;//de-action
        // -------------------------------------
        #(MPSSE_PERIOD);
        ADwr = 1'b0;//host release AD
        
        
        // -------------------------------------
        #(4 * MPSSE_PERIOD);
        RpWn = 1'b1;//FPGA drive AD
        // -------------------------------------
        #(MPSSE_PERIOD);
        ACT  = 1'b1;//action
        // -------------------------------------
        #(MPSSE_PERIOD);
        FTDI_READ_OUT = demo_tb.DUT.adport_out;//read data
        // -------------------------------------
        #(MPSSE_PERIOD);
        RpWn = 1'b0;//FPGA release AD
        ACT  = 1'b0;//de-action
end
endtask


task jesd204b_wr_seq;
input [11:0] jesd204b_wr_addr;
input [31:0] jesd204b_wr_word;
begin
//Set configuration word  
  JESD204B_WB0_REG                = jesd204b_wr_word[7:0];
  ftdi_wr_seq (JESD204B_WB0_REG_ADDR, `JESD204B_WB0_REG_VALU);
  #(MPSSE_PERIOD);
  JESD204B_WB1_REG                = jesd204b_wr_word[15:8];
  ftdi_wr_seq (JESD204B_WB1_REG_ADDR, `JESD204B_WB1_REG_VALU);
  #(MPSSE_PERIOD);
  JESD204B_WB2_REG                = jesd204b_wr_word[23:16];
  ftdi_wr_seq (JESD204B_WB2_REG_ADDR, `JESD204B_WB2_REG_VALU);
  #(MPSSE_PERIOD);
  JESD204B_WB3_REG                = jesd204b_wr_word[31:24];
  ftdi_wr_seq (JESD204B_WB3_REG_ADDR, `JESD204B_WB3_REG_VALU);

//Write out the word
  #(MPSSE_PERIOD);
  W2INDEX_SET                     = jesd204b_wr_addr[11:6]; 
  ftdi_wr_seq (JESD204B_W2I_REG_ADDR, `JESD204B_W2I_REG_VALU); 
  #(MPSSE_PERIOD);
  WINDEX_SET                      = jesd204b_wr_addr[5:0];
  WRCE_SET                        = 1'b1; 
  ftdi_wr_seq (JESD204B_CFIG_REG_ADDR, `JESD204B_CFIG_REG_VALU);
  
//Check if the writing is valid
  #(MPSSE_PERIOD);
  WRACK_DET = 1'b0;
  while (WRACK_DET == 1'b0)
  begin
    # (MPSSE_PERIOD); 
    ftdi_rd_seq (JESD204B_CFIG_REG_ADDR, `JESD204B_CFIG_REG_VALU);
  end
  #(MPSSE_PERIOD);
end  
endtask

task jesd204b_rd_seq;
input [11:0] jesd204b_rd_addr;
output [31:0] jesd204b_rd_word;
begin 
//read back the word
   R2INDEX_SET                     = jesd204b_rd_addr[11:6]; 
   ftdi_wr_seq (JESD204B_R2I_REG_ADDR, `JESD204B_R2I_REG_VALU);
    
   #(MPSSE_PERIOD);
   RINDEX_SET                      = jesd204b_rd_addr[5:0];
   RDCE_SET                        = 1'b1; 
   ftdi_wr_seq (JESD204B_CHK_REG_ADDR, `JESD204B_CHK_REG_VALU);
  
//Check if the reading is valid
   #(MPSSE_PERIOD);
   RDACK_DET = 1'b0;
   while (RDACK_DET == 1'b0)
   begin
     # (MPSSE_PERIOD); 
     ftdi_rd_seq (JESD204B_CHK_REG_ADDR, `JESD204B_CHK_REG_VALU);
   end
      
   #(MPSSE_PERIOD);  
   ftdi_rd_seq (JESD204B_RB0_REG_ADDR, `JESD204B_RB0_REG_VALU);
   
   #(MPSSE_PERIOD);
   ftdi_rd_seq (JESD204B_RB1_REG_ADDR, `JESD204B_RB1_REG_VALU);
   
   #(MPSSE_PERIOD);
   ftdi_rd_seq (JESD204B_RB2_REG_ADDR, `JESD204B_RB2_REG_VALU);
   
   #(MPSSE_PERIOD);
   ftdi_rd_seq (JESD204B_RB3_REG_ADDR, `JESD204B_RB3_REG_VALU);
   jesd204b_rd_word = {JESD204B_RB3_REG, JESD204B_RB2_REG, JESD204B_RB1_REG, JESD204B_RB0_REG};
   #(MPSSE_PERIOD);
end   
endtask






endmodule
