/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2015                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.0     10:08:58 09/09/2015 Gary Yu                                *
* File Created                                                                *
*                                                                             *
* Additional Comments:                                                        *
*                                                                             *
******************************************************************************/
module add_dec
(
actpls, 
adflag,
addr, 
rst,

ID_SEL,
CAP_CFIG_SEL,       
CAP_CTRL_SEL,
CAP_RST_SEL,
CAP_STATUS_SEL,
SPI_CFIG_SEL,
CLK_STATUS_SEL,

JESD204B_WB0_SEL,
JESD204B_WB1_SEL,
JESD204B_WB2_SEL,
JESD204B_WB3_SEL,
JESD204B_CFIG_SEL,

JESD204B_RB0_SEL,
JESD204B_RB1_SEL,
JESD204B_RB2_SEL,
JESD204B_RB3_SEL,
JESD204B_CHK_SEL,

I2C_ACSS_SEL,

JESD204B_W2I_SEL,
JESD204B_R2I_SEL
);

parameter my_register_number          = 20;
    
input       actpls; 
input       adflag;
input       rst;

input [7:0] addr; 

output ID_SEL;
output CAP_CFIG_SEL;       
output CAP_CTRL_SEL;
output CAP_RST_SEL;
output CAP_STATUS_SEL;
output SPI_CFIG_SEL;
output CLK_STATUS_SEL;

output JESD204B_WB0_SEL;
output JESD204B_WB1_SEL;
output JESD204B_WB2_SEL;
output JESD204B_WB3_SEL;
output JESD204B_CFIG_SEL;

output JESD204B_RB0_SEL;
output JESD204B_RB1_SEL;
output JESD204B_RB2_SEL;
output JESD204B_RB3_SEL;
output JESD204B_CHK_SEL;

output I2C_ACSS_SEL;

output JESD204B_W2I_SEL;
output JESD204B_R2I_SEL;

reg ID_SEL;
reg CAP_CFIG_SEL;       
reg CAP_CTRL_SEL;
reg CAP_RST_SEL;
reg CAP_STATUS_SEL;
reg SPI_CFIG_SEL;
reg CLK_STATUS_SEL;

reg JESD204B_WB0_SEL;
reg JESD204B_WB1_SEL;
reg JESD204B_WB2_SEL;
reg JESD204B_WB3_SEL;
reg JESD204B_CFIG_SEL;

reg JESD204B_RB0_SEL;
reg JESD204B_RB1_SEL;
reg JESD204B_RB2_SEL;
reg JESD204B_RB3_SEL;
reg JESD204B_CHK_SEL;

reg I2C_ACSS_SEL;

reg JESD204B_W2I_SEL;
reg JESD204B_R2I_SEL;

  // ----------------------------------------------------
  // Address Decoder
  // ----------------------------------------------------
 
always @ (posedge actpls or negedge rst)   
  if (~rst)
  begin
  
      ID_SEL            <= #1 1'b0;
      CAP_CFIG_SEL      <= #1 1'b0;       
      CAP_CTRL_SEL      <= #1 1'b0;
      CAP_RST_SEL       <= #1 1'b0;
      CAP_STATUS_SEL    <= #1 1'b0;
      SPI_CFIG_SEL      <= #1 1'b0;
      CLK_STATUS_SEL    <= #1 1'b0;
      
      JESD204B_WB0_SEL  <= #1 1'b0;
      JESD204B_WB1_SEL  <= #1 1'b0;
      JESD204B_WB2_SEL  <= #1 1'b0;
      JESD204B_WB3_SEL  <= #1 1'b0;
      JESD204B_CFIG_SEL <= #1 1'b0;

      JESD204B_RB0_SEL  <= #1 1'b0;
      JESD204B_RB1_SEL  <= #1 1'b0;
      JESD204B_RB2_SEL  <= #1 1'b0;
      JESD204B_RB3_SEL  <= #1 1'b0;
      JESD204B_CHK_SEL  <= #1 1'b0;
      
      I2C_ACSS_SEL      <= #1 1'b0;
      
      JESD204B_W2I_SEL  <= #1 1'b0;
      JESD204B_R2I_SEL  <= #1 1'b0;

  end
  else if (adflag)
  begin
    casex ({addr})
    8'h00: {  
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 0;
    8'h01: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 1;
    8'h02: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 2;
           
    8'h03: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 3;
           
    8'h04: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 4; 
           
    8'h05: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 5; 
           
    8'h06: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 6;                
              
    8'h07: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 7;
                       
    8'h08: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 8;
                       
    8'h09: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 9;
                       
    8'h0a: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 10;
                       
    8'h0b: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 11;
                       
    8'h0c: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 12;
                       
    8'h0d: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 13;
                       
    8'h0e: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 14;
                       
    8'h0f: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 15;
                       
    8'h10: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 16;

    8'h11: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 17;
   
    8'h12: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 18;
                           
    8'h13: {
             ID_SEL,
             CAP_CFIG_SEL,       
             CAP_CTRL_SEL,
             CAP_RST_SEL,
             CAP_STATUS_SEL,
             SPI_CFIG_SEL,
             CLK_STATUS_SEL,
             
             JESD204B_WB0_SEL,
             JESD204B_WB1_SEL,
             JESD204B_WB2_SEL,
             JESD204B_WB3_SEL,
             JESD204B_CFIG_SEL,

             JESD204B_RB0_SEL,
             JESD204B_RB1_SEL,
             JESD204B_RB2_SEL,
             JESD204B_RB3_SEL,
             JESD204B_CHK_SEL,
             
             I2C_ACSS_SEL,
             
             JESD204B_W2I_SEL,
             JESD204B_R2I_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 19;
     
           
                
    endcase 
  end  

endmodule
