/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2015                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.0     10:08:58 09/09/2015 Gary Yu                                *
* File Created                                                                *
*                                                                             *
* Additional Comments:                                                        *
*                                                                             *
******************************************************************************/
module axi_cfg
(
    m_axi_aclk,
    m_axi_aresetn,
  
    m_axi_awaddr,
    m_axi_awvalid,
    m_axi_awready,
  
    m_axi_wdata,
    m_axi_wvalid,
    m_axi_wready,
  
    m_axi_bresp,
    m_axi_bvalid,
    m_axi_bready,
  
    m_axi_araddr,
    m_axi_arvalid,
    m_axi_arready,
  
    m_axi_rdata,
    m_axi_rresp,
    m_axi_rvalid,
    m_axi_rready,

//jesd204b wb register
    JESD204B_WB_ISET,
//jesd204b configuration register    
    WINDEX_ISET,
    W2INDEX_ISET,
    WRCE_ISET,
    WRACK_ODET,
    
//jesd204b rb register
    JESD204B_RB_ODET,
//jesd204b check register    
    RINDEX_ISET,
    R2INDEX_ISET,
    RDCE_ISET,
    RDACK_ODET
    
);
  input                   m_axi_aclk;
  input                   m_axi_aresetn;
  
  output [11:0]           m_axi_awaddr;
  output                  m_axi_awvalid;
  input                   m_axi_awready;
  
  output [31:0]           m_axi_wdata;
  output                  m_axi_wvalid;
  input                   m_axi_wready;
  
  input  [1:0]            m_axi_bresp;
  input                   m_axi_bvalid;
  output                  m_axi_bready;
  
  output [11:0]           m_axi_araddr;
  output                  m_axi_arvalid;
  input                   m_axi_arready;
  
  input  [31:0]           m_axi_rdata;
  input  [1:0]            m_axi_rresp;
  input                   m_axi_rvalid;
  output                  m_axi_rready;

//jesd204b wb register
input [31:0]              JESD204B_WB_ISET;
//jesd204b configuration register    
input [5:0]               WINDEX_ISET;
input [5:0]               W2INDEX_ISET;
input                     WRCE_ISET;
output                    WRACK_ODET;
    
//jesd204b rb register
output  [31:0]            JESD204B_RB_ODET;
//jesd204b check register    
input [5:0]               RINDEX_ISET;
input [5:0]               R2INDEX_ISET;
input                     RDCE_ISET;
output                    RDACK_ODET;


  reg [11:0]              m_axi_awaddr;
  reg                     m_axi_awvalid;
  
  reg [31:0]              m_axi_wdata;
  reg                     m_axi_wvalid;
  
  reg                     m_axi_bready;
  
  reg [11:0]              m_axi_araddr;
  reg                     m_axi_arvalid;
  
  reg                     m_axi_rready;

//jesd204b configuration register    
  reg                     WRACK_ODET;
    
//jesd204b rb register
  reg  [31:0]             JESD204B_RB_ODET;
//jesd204b check register    
  reg                     RDACK_ODET;

//read jesd204b
reg [3:0] axi_rd_cnt;
wire #1 axi_rd_rst = RDCE_ISET & m_axi_aresetn;
always @(posedge m_axi_aclk or negedge axi_rd_rst)
  if (~axi_rd_rst)
    axi_rd_cnt      <= #1 4'b0;
  else
    axi_rd_cnt      <= #1 axi_rd_cnt + 1;

always @(posedge m_axi_aclk or negedge axi_rd_rst)
  if (~axi_rd_rst)
  begin
    m_axi_araddr    <= #1 12'b0; 
    m_axi_arvalid   <= #1 1'b0;
    m_axi_rready    <= #1 1'b0;
  end
  else if (m_axi_arready)
    casex({axi_rd_cnt})
    4'b0001: begin
               m_axi_araddr    <= #1 {R2INDEX_ISET, RINDEX_ISET}; 
               m_axi_arvalid   <= #1 1'b1;
               m_axi_rready    <= #1 1'b0;
             end   
    4'b0010: begin
               m_axi_araddr    <= #1 12'b0; 
               m_axi_arvalid   <= #1 1'b0;
               m_axi_rready    <= #1 1'b1;
             end   
    endcase

wire #1 axi_rdack_rst = (axi_rd_cnt == 4'b0010) | (~m_axi_aresetn);         
always @(posedge m_axi_aclk or posedge axi_rdack_rst)
  if (axi_rdack_rst)
  begin
    JESD204B_RB_ODET <= #1 32'b0;    
    RDACK_ODET       <= #1 1'b0;
  end
  else if (m_axi_rready & m_axi_rvalid & (m_axi_rresp == 2'b00)) 
  begin
    JESD204B_RB_ODET <= #1 m_axi_rdata;    
    RDACK_ODET       <= #1 1'b1;
  end   
   
//write jesd204b
parameter AXI_WR_ADDR  = 2'b00;
parameter AXI_WR_DATA  = 2'b01;
parameter AXI_WR_RESP  = 2'b11;
parameter AXI_WR_DONE  = 2'b10;
reg [1:0] axi_wr_status;
wire #1 axi_wr_rst = WRCE_ISET & m_axi_aresetn;
always @(posedge m_axi_aclk or negedge axi_wr_rst)
  if (~axi_wr_rst)
  begin
    axi_wr_status   <= #1 AXI_WR_ADDR;
    m_axi_awaddr    <= #1 12'b0; 
    m_axi_awvalid   <= #1 1'b0;
    
    m_axi_wdata     <= #1 32'b0;
    m_axi_wvalid    <= #1 1'b0;
  
    m_axi_bready    <= #1 1'b0;
  end
  else
    case({axi_wr_status})
    AXI_WR_ADDR: if(m_axi_awready)
                 begin
                   axi_wr_status   <= #1 AXI_WR_DATA;
                   m_axi_awaddr    <= #1 12'b0; 
                   m_axi_awvalid   <= #1 1'b0;

                   m_axi_bready    <= #1 1'b0;
                 end
                 else
                 begin  
                   m_axi_awaddr    <= #1 {W2INDEX_ISET, WINDEX_ISET}; 
                   m_axi_awvalid   <= #1 1'b1;
    
                   m_axi_wdata     <= #1 JESD204B_WB_ISET;
                   m_axi_wvalid    <= #1 1'b1;
  
                   m_axi_bready    <= #1 1'b0;
                 end 
    AXI_WR_DATA: if(m_axi_wready)
                 begin 
                   axi_wr_status   <= #1 AXI_WR_RESP;
                   m_axi_awaddr    <= #1 12'b0; 
                   m_axi_awvalid   <= #1 1'b0;
    
                   m_axi_wdata     <= #1 32'b0;
                   m_axi_wvalid    <= #1 1'b0;
                   
                   m_axi_bready    <= #1 1'b0;                              
                 end
                 else
                 begin    
                   m_axi_awaddr    <= #1 12'b0; 
                   m_axi_awvalid   <= #1 1'b0;
  
                   m_axi_bready    <= #1 1'b0;
                 end
    AXI_WR_RESP: if(m_axi_bvalid)
                 begin
                   axi_wr_status   <= #1 AXI_WR_DONE;
                   m_axi_bready    <= #1 1'b1;    
                 end
                 else
                   m_axi_bready    <= #1 1'b0;                    
    AXI_WR_DONE: begin
                   axi_wr_status   <= #1 AXI_WR_DONE;

                   m_axi_awaddr    <= #1 12'b0; 
                   m_axi_awvalid   <= #1 1'b0;
    
                   m_axi_wdata     <= #1 32'b0;
                   m_axi_wvalid    <= #1 1'b0;
                   
                   m_axi_bready    <= #1 1'b0;                   
                 end
    default:     begin
                   axi_wr_status   <= #1 AXI_WR_DONE;
                     
                   m_axi_awaddr    <= #1 12'b0; 
                   m_axi_awvalid   <= #1 1'b0;
    
                   m_axi_wdata     <= #1 32'b0;
                   m_axi_wvalid    <= #1 1'b0;
                   
                   m_axi_bready    <= #1 1'b0;
                 end                                           
    endcase
    
wire #1 axi_wrack_rst = (axi_wr_status == AXI_WR_RESP) | (~m_axi_aresetn);         
always @(posedge m_axi_aclk or posedge axi_wrack_rst)
  if (axi_wrack_rst)    
    WRACK_ODET       <= #1 1'b0;
  else if (m_axi_bready & m_axi_bvalid & (m_axi_bresp == 2'b00))     
    WRACK_ODET       <= #1 1'b1;
     
endmodule
