/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2015                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.0     10:08:58 09/09/2015 Gary Yu                                *
* File Created                                                                *
*                                                                             *
* Additional Comments:                                                        *
*                                                                             *
******************************************************************************/
module ftdi_cfg
(
//id register
//capture configuration register
    MEMSIZE_SET,
    CHSEL_SET,
//capture control register
    START_SET,
    FETONLY_SET,
//capture reset register
    CTRLRST_SET,
//capture status register
    CHVLID_DET,
    SYNCERR_DET,
    FETDONE_DET,
    CAPDONE_DET,
//i2c write register    
    SDA_SET,
    SCL_SET,
//i2c read register            
    SDA_DET,      
//SPI configuration register
    SPI_SET,
    RCLK_SYNC_SET,
    SREF_SYNC_SET,
//clock status register
    SYNC_DET,
    ARST_DET,
    SYSLK_DET,     
    CORELK_DET,
    FIFOLK_DET,
    
//jesd204b wb register
    JESD204B_WB_SET,
//jesd204b configuration register    
    WINDEX_SET,
    W2INDEX_SET,
    WRCE_SET,
    WRACK_DET,
    
//jesd204b rb register
    JESD204B_RB_DET,
//jesd204b check register    
    RINDEX_SET,
    R2INDEX_SET,
    RDCE_SET,
    RDACK_DET,

    cfigclk,     
             
    actpls,
    hostrw,
    adflag,
    
    rst,
    selfclk,    
    
    adport_in, 
    adport_out
       
);

`include "/home/gyu/p2123_080315/src/fpga/cfg_define.h"

//id register
//capture configuration register
output [3:0]    MEMSIZE_SET;
output [2:0]    CHSEL_SET;
//capture control register
output          START_SET;
output          FETONLY_SET;
//capture reset register
output          CTRLRST_SET;
//capture status register
input  [3:0]    CHVLID_DET;
input           SYNCERR_DET;
input           FETDONE_DET;
input           CAPDONE_DET;
//i2c write register    
output          SDA_SET;
output          SCL_SET;
//i2c read register            
input           SDA_DET;  
//SPI configuration register
output [2:0]    SPI_SET;
output          RCLK_SYNC_SET;
output          SREF_SYNC_SET;

//clock status register
input           SYNC_DET;
input           ARST_DET;     
input           SYSLK_DET;     
input           CORELK_DET;
input           FIFOLK_DET;

//jesd204b wb register
output [31:0]   JESD204B_WB_SET;
//jesd204b configuration register    
output [5:0]    WINDEX_SET;
output [5:0]    W2INDEX_SET;
output          WRCE_SET;
input           WRACK_DET;
    
//jesd204b rb register
input  [31:0]   JESD204B_RB_DET;
//jesd204b check register    
output [5:0]    RINDEX_SET;
output [5:0]    R2INDEX_SET;
output          RDCE_SET;
input           RDACK_DET;

input           cfigclk;     

input           actpls;
input           hostrw;
input           adflag;
input           rst;
input           selfclk;
 
input   [7:0]   adport_in; 
output  [7:0]   adport_out;

//id register
reg     [7:0]   ID_REG;
//capture configuration register
reg     [3:0]   MEMSIZE_SET;
reg     [2:0]   CHSEL_SET;
//capture control register
reg             START_SET;
reg             FETONLY_SET; 
//capture reset register
reg             CTRLRST_SET;
//i2c write register    
reg             SDA_SET;
reg             SCL_SET;
//SPI configuration register
reg     [2:0]   SPI_SET;
reg             RCLK_SYNC_SET;
reg             SREF_SYNC_SET;

//jesd204b wb register
reg     [31:0]  JESD204B_WB_SET;
//jesd204b configuration register    
reg     [5:0]   WINDEX_SET;
reg     [5:0]   W2INDEX_SET;
reg             WRCE_SET;
//jesd204b check register    
reg     [5:0]   RINDEX_SET;
reg     [5:0]   R2INDEX_SET;
reg             RDCE_SET;


reg     [7:0]   adport_out;


wire ID_SEL;
wire CAP_CFIG_SEL;       
wire CAP_CTRL_SEL;
wire CAP_RST_SEL;
wire CAP_STATUS_SEL;
wire SPI_CFIG_SEL;
wire CLK_STATUS_SEL;

wire JESD204B_WB0_SEL;
wire JESD204B_WB1_SEL;
wire JESD204B_WB2_SEL;
wire JESD204B_WB3_SEL;
wire JESD204B_CFIG_SEL;

wire JESD204B_RB0_SEL;
wire JESD204B_RB1_SEL;
wire JESD204B_RB2_SEL;
wire JESD204B_RB3_SEL;
wire JESD204B_CHK_SEL;

wire I2C_ACSS_SEL;

wire JESD204B_W2I_SEL;
wire JESD204B_R2I_SEL;

add_dec add_dec
(
.actpls(actpls), 
.adflag(adflag),
.addr(adport_in), 
.rst(rst),

.ID_SEL(ID_SEL),
.CAP_CFIG_SEL(CAP_CFIG_SEL),       
.CAP_CTRL_SEL(CAP_CTRL_SEL),
.CAP_RST_SEL(CAP_RST_SEL),

.CAP_STATUS_SEL(CAP_STATUS_SEL),
.SPI_CFIG_SEL(SPI_CFIG_SEL),
.CLK_STATUS_SEL(CLK_STATUS_SEL),

.JESD204B_WB0_SEL(JESD204B_WB0_SEL),
.JESD204B_WB1_SEL(JESD204B_WB1_SEL),
.JESD204B_WB2_SEL(JESD204B_WB2_SEL),
.JESD204B_WB3_SEL(JESD204B_WB3_SEL),
.JESD204B_CFIG_SEL(JESD204B_CFIG_SEL),

.JESD204B_RB0_SEL(JESD204B_RB0_SEL),
.JESD204B_RB1_SEL(JESD204B_RB1_SEL),
.JESD204B_RB2_SEL(JESD204B_RB2_SEL),
.JESD204B_RB3_SEL(JESD204B_RB3_SEL),
.JESD204B_CHK_SEL(JESD204B_CHK_SEL),

.I2C_ACSS_SEL(I2C_ACSS_SEL),

.JESD204B_W2I_SEL(JESD204B_W2I_SEL),
.JESD204B_R2I_SEL(JESD204B_R2I_SEL)

);


  // ----------------------------------------------------
  // Register Read by Host
  // ----------------------------------------------------

always @ (posedge actpls or negedge rst)
  if (~rst)
  begin
    adport_out  <= #1 8'h0;
  end
  else if (hostrw)
  begin
    if (ID_SEL)
       adport_out  <= #1  ID_REG;
    else if (CAP_STATUS_SEL)
       adport_out  <= #1  {CHVLID_DET,
                           1'b0,
                           SYNCERR_DET,
                           FETDONE_DET,
                           CAPDONE_DET};
    else if (CLK_STATUS_SEL)
       adport_out  <= #1  {3'b0,
                           SYNC_DET,
                           ARST_DET,
                           SYSLK_DET,     
                           CORELK_DET,
                           FIFOLK_DET};
    else if (JESD204B_CFIG_SEL)                       
       adport_out  <= #1  {7'b0, WRACK_DET};
    else if (JESD204B_RB0_SEL)                       
       adport_out  <= #1  JESD204B_RB_DET[7:0];
    else if (JESD204B_RB1_SEL)                       
       adport_out  <= #1  JESD204B_RB_DET[15:8];
    else if (JESD204B_RB2_SEL)                       
       adport_out  <= #1  JESD204B_RB_DET[23:16];
    else if (JESD204B_RB3_SEL)                       
       adport_out  <= #1  JESD204B_RB_DET[31:24];
    else if (JESD204B_CHK_SEL)                       
       adport_out  <= #1  {7'b0, RDACK_DET};
    else if (I2C_ACSS_SEL)            
       adport_out  <= #1  {7'b0, SDA_DET};          
  end
  
  // ----------------------------------------------------
  // Register Write by Host
  // ----------------------------------------------------

always @ (posedge actpls or negedge rst)
  if (~rst)
  begin
    //id register
    ID_REG                   <= #1 id_value;
    //capture configuration register
    MEMSIZE_SET              <= #1 buf_128SPS;
    CHSEL_SET                <= #1 ch01_sel;
    //capture control register
    FETONLY_SET              <= #1 1'b0;
    //i2c write register    
    SDA_SET                  <= #1 1'b1;
    SCL_SET                  <= #1 1'b1;
    //spi configuratin register
    SPI_SET                  <= #1 adca_sel;
    RCLK_SYNC_SET            <= #1 1'b0;
    SREF_SYNC_SET            <= #1 1'b0;
    //jesd204b wb register
    JESD204B_WB_SET          <= #1 32'b0;
    //jesd204b configuration register    
    WINDEX_SET               <= #1 6'b0;
    W2INDEX_SET              <= #1 6'b0;
    //jesd204b check register    
    RINDEX_SET               <= #1 6'b0;
    R2INDEX_SET              <= #1 6'b0;
  end
  else if ((~hostrw) & (~adflag))
  begin
    if (ID_SEL)
      ID_REG                 <= #1 adport_in;
    else if (CAP_CFIG_SEL)
    begin
      MEMSIZE_SET            <= #1 adport_in[7:4];
      CHSEL_SET              <= #1 adport_in[3:1];
    end
    else if (CAP_CTRL_SEL)
      FETONLY_SET            <= #1 adport_in[1];
    else if (SPI_CFIG_SEL)
    begin
      SPI_SET                <= #1 adport_in[2:0];
      RCLK_SYNC_SET          <= #1 adport_in[4]; 
      SREF_SYNC_SET          <= #1 adport_in[3];
    end   
    else if (JESD204B_WB0_SEL)                       
      JESD204B_WB_SET[7:0]   <= #1 adport_in;
    else if (JESD204B_WB1_SEL)                       
      JESD204B_WB_SET[15:8]  <= #1 adport_in;
    else if (JESD204B_WB2_SEL)                       
      JESD204B_WB_SET[23:16] <= #1 adport_in;
    else if (JESD204B_WB3_SEL)                       
      JESD204B_WB_SET[31:24] <= #1 adport_in;      
    else if (JESD204B_CFIG_SEL)
      WINDEX_SET             <= #1 adport_in[7:2];       
    else if (JESD204B_CHK_SEL)
      RINDEX_SET             <= #1 adport_in[7:2];
    else if (I2C_ACSS_SEL)
    begin    
      SCL_SET                <= #1 adport_in[1];
      SDA_SET                <= #1 adport_in[0];
    end
    else if (JESD204B_W2I_SEL)
      W2INDEX_SET             <= #1 adport_in[5:0];
    else if (JESD204B_R2I_SEL)
      R2INDEX_SET             <= #1 adport_in[5:0];
  end

//capture control register self clean bit
reg [3:0] START_SET_selfcnt;    
always @ (posedge selfclk or negedge rst)
  if (~rst)
    START_SET_selfcnt     <= #1 4'b0;
  else
  begin
    START_SET_selfcnt[0]  <= #1 START_SET;
    START_SET_selfcnt[1]  <= #1 START_SET_selfcnt[0];
    START_SET_selfcnt[2]  <= #1 START_SET_selfcnt[1];
    START_SET_selfcnt[3]  <= #1 START_SET_selfcnt[2];
  end  

wire #1 START_SET_rst = rst & (~(START_SET_selfcnt == 4'hf));
always @ (posedge actpls or negedge START_SET_rst)
  if (~START_SET_rst)
    START_SET        <= #1 1'b0;   
  else if ((~hostrw) & (~adflag) & (CAP_CTRL_SEL))
    START_SET        <= #1 adport_in[0];
          
    
//capure reset register self clean bit
reg [3:0] CTRLRST_SET_selfcnt;    
always @ (posedge selfclk or negedge rst)
  if (~rst)
    CTRLRST_SET_selfcnt     <= #1 4'hf;
  else
  begin
    CTRLRST_SET_selfcnt[0]  <= #1 CTRLRST_SET;
    CTRLRST_SET_selfcnt[1]  <= #1 CTRLRST_SET_selfcnt[0];
    CTRLRST_SET_selfcnt[2]  <= #1 CTRLRST_SET_selfcnt[1];
    CTRLRST_SET_selfcnt[3]  <= #1 CTRLRST_SET_selfcnt[2]; 
  end

wire #1 CTRLRST_SET_rst = rst & (~(CTRLRST_SET_selfcnt == 4'b0));   
always @ (posedge actpls or negedge CTRLRST_SET_rst)
  if (~CTRLRST_SET_rst)
    CTRLRST_SET        <= #1 1'b1;   
  else if ((~hostrw) & (~adflag) & (CAP_RST_SEL))
    CTRLRST_SET        <= #1 ~adport_in[0];

    
//jesd204b configuration register self clean bit
reg [11:0] WRCE_SET_selfcnt;
reg        WRCE_SETpre;
always @ (posedge cfigclk or negedge rst)
  if (~rst)
  begin
    WRCE_SET_selfcnt     <= #1 12'b0;
    WRCE_SET             <= #1 1'b0;
  end  
  else
  begin
    WRCE_SET_selfcnt[0]  <= #1 WRCE_SETpre;
    WRCE_SET_selfcnt[1]  <= #1 WRCE_SET_selfcnt[0];
    WRCE_SET_selfcnt[2]  <= #1 WRCE_SET_selfcnt[1];
    WRCE_SET_selfcnt[3]  <= #1 WRCE_SET_selfcnt[2]; 
    WRCE_SET_selfcnt[4]  <= #1 WRCE_SET_selfcnt[3];
    WRCE_SET_selfcnt[5]  <= #1 WRCE_SET_selfcnt[4];
    WRCE_SET_selfcnt[6]  <= #1 WRCE_SET_selfcnt[5]; 
    WRCE_SET_selfcnt[7]  <= #1 WRCE_SET_selfcnt[6];
    WRCE_SET_selfcnt[8]  <= #1 WRCE_SET_selfcnt[7];
    WRCE_SET_selfcnt[9]  <= #1 WRCE_SET_selfcnt[8];
    WRCE_SET_selfcnt[10] <= #1 WRCE_SET_selfcnt[9];
    WRCE_SET_selfcnt[11] <= #1 WRCE_SET_selfcnt[10];
    WRCE_SET             <= #1 WRCE_SET_selfcnt[2];
  end

wire WRCE_SETpre_rst = rst & (~(WRCE_SET_selfcnt == 12'h1ff));   
always @ (posedge actpls or negedge WRCE_SETpre_rst)
  if (~WRCE_SETpre_rst)
    WRCE_SETpre        <= #1 1'b0;   
  else if ((~hostrw) & (~adflag) & (JESD204B_CFIG_SEL))
    WRCE_SETpre        <= #1 adport_in[1];

    
//jesd204b check register self clean bit
reg [11:0] RDCE_SET_selfcnt;
reg        RDCE_SETpre;
always @ (posedge cfigclk or negedge rst)
  if (~rst)
  begin
    RDCE_SET_selfcnt     <= #1 12'b0;
    RDCE_SET             <= #1 1'b0;
  end  
  else
  begin
    RDCE_SET_selfcnt[0]  <= #1 RDCE_SETpre;
    RDCE_SET_selfcnt[1]  <= #1 RDCE_SET_selfcnt[0];
    RDCE_SET_selfcnt[2]  <= #1 RDCE_SET_selfcnt[1];
    RDCE_SET_selfcnt[3]  <= #1 RDCE_SET_selfcnt[2]; 
    RDCE_SET_selfcnt[4]  <= #1 RDCE_SET_selfcnt[3];
    RDCE_SET_selfcnt[5]  <= #1 RDCE_SET_selfcnt[4];
    RDCE_SET_selfcnt[6]  <= #1 RDCE_SET_selfcnt[5]; 
    RDCE_SET_selfcnt[7]  <= #1 RDCE_SET_selfcnt[6];
    RDCE_SET_selfcnt[8]  <= #1 RDCE_SET_selfcnt[7];
    RDCE_SET_selfcnt[9]  <= #1 RDCE_SET_selfcnt[8];
    RDCE_SET_selfcnt[10] <= #1 RDCE_SET_selfcnt[9];
    RDCE_SET_selfcnt[11] <= #1 RDCE_SET_selfcnt[10];
    RDCE_SET             <= #1 RDCE_SET_selfcnt[2];
  end

wire RDCE_SETpre_rst = rst & (~(RDCE_SET_selfcnt == 12'h1ff));   
always @ (posedge actpls or negedge RDCE_SETpre_rst)
  if (~RDCE_SETpre_rst)
    RDCE_SETpre        <= #1 1'b0;   
  else if ((~hostrw) & (~adflag) & (JESD204B_CHK_SEL))
    RDCE_SETpre        <= #1 adport_in[1];
    
endmodule
