/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2015                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.0     10:08:58 09/09/2015 Gary Yu                                *
* File Created                                                                *
*                                                                             *
* Additional Comments:                                                        *
*                                                                             *
******************************************************************************/

`timescale 1ns / 1ps

(* DowngradeIPIdentifiedWarnings = "yes" *)
module jd204bv6p0l4sc0_example_design #(
  parameter               pLanes = 4 ) (
  
//gyu FTDI Sync FIFO interface
  output   [7:0]          fifo_data,
  input                   txen,
  output                  wrn,
  input                   fifoclk,
 
//gyu  General Control and Jumper
  input                   actpls,
  input                   hostrw,
  input                   adflag,
  input                   cfgrst,
  inout    [7:0]          adport,

//gyu i2c master interface
  inout                   ssda,
  output                  sscl, 
  
//gyu spi master input from FDTI
  input                   sck,
  input                   scs,
  input                   mosi,
  output                  miso,

//gyu spi slave output to ADC
  output                  adcck,
  output                  adccsa,
  output                  adccsb,
  output                  s6954cs,
  output                  clkgencs,
  output                  sysrefcs,
  output                  adcdi,
  input                   adcdo,

//gyu rclk sref ctrl   
  output                  rclk_sync,
  output                  sref_sync,
  
  
  input                   refclk0p,
  input                   refclk0n,


  input                   rx_reset,

  input  [pLanes-1:0]  rxp,
  input  [pLanes-1:0]  rxn,


  input                   rx_sysclk_p,//gyu clk tree
  input                   rx_sysclk_n,//gyu clk tree

  // Rx AXI common signals
  output                  rx_aresetn,

  input                   rx_sysref_p,
  input                   rx_sysref_n,

  output                  rx1_sync_p,
  output                  rx1_sync_n,

  output                  rx_sync_p,
  output                  rx_sync_n


);

//`include "/home/gyu/p2123_080315/src/fpga/char_define.h"

  wire                   s_axi_aclk;
  wire                   s_axi_aresetn;
  
  wire  [11:0]           s_axi_awaddr;
  wire                   s_axi_awvalid;
  wire                   s_axi_awready;
  
  wire  [31:0]           s_axi_wdata;
  wire                   s_axi_wvalid;
  wire                   s_axi_wready;
  
  wire [1:0]             s_axi_bresp;
  wire                   s_axi_bvalid;
  wire                   s_axi_bready;
  
  wire  [11:0]           s_axi_araddr;
  wire                   s_axi_arvalid;
  wire                   s_axi_arready;
  
  wire [31:0]            s_axi_rdata;
  wire [1:0]             s_axi_rresp;
  wire                   s_axi_rvalid;
  wire                   s_axi_rready;


  wire                    rx_sysref;
  wire                    rx_sync;
  wire                 #1 rx1_sync = rx_sync;
    


OBUFDS OBUFDS_rx_sync
(
.O(rx_sync_p),
.OB(rx_sync_n),
.I(rx_sync)
);

OBUFDS OBUFDS_rx1_sync
(
.O(rx1_sync_p),
.OB(rx1_sync_n),
.I(rx1_sync)
);

IBUFDS IBUFDS_rx_sysref
(
.I(rx_sysref_p),
.IB(rx_sysref_n),
.O(rx_sysref)
);


//gyu reset tree
wire #1 grstp = rx_reset | (~cfgrst);
wire #1 grstn = (~rx_reset) & cfgrst;

//gyu clk tree
wire rx_sysclk_lk;

clk_wiz_rx_sysclk clk_wiz_rx_sysclk
 (// Clock in ports
  .clk_in1_p(rx_sysclk_p),
  .clk_in1_n(rx_sysclk_n),
  // Clock out ports
  .clk_out1(s_axi_aclk),
  // Status and control signals
  .reset(grstp),//gyu axi reset
  .locked(rx_sysclk_lk)
 );

wire coregen_clk;
wire core_clk, clkdb1, clkdb2;
wire bram_a_clk;
wire clk_wiz_rx_aclk_lk;
clk_wiz_rx_aclk_2 clk_wiz_rx_aclk_2
 (// Clock in ports
  .clk_in1(coregen_clk),
  // Clock out ports
  .clk_out1(core_clk),
  .clk_out2(clkdb2),
  .clk_out3(clkdb1),
  // Status and control signals
  //.reset(grstp | (~rx_aresetn)),
  .reset(1'b0),
  .locked(clk_wiz_rx_aclk_lk)
 );   
//gyu clk tree

assign #1 s_axi_aresetn = rx_sysclk_lk;

  // Rx AXI common signals


  wire                               rx_tvalid;
  wire   [31:0]                      rx0_tdata;
  wire   [31:0]                      rx1_tdata;
  wire   [31:0]                      rx2_tdata;
  wire   [31:0]                      rx3_tdata;


jd204bv6p0l4sc0 i_jd204bv6p0l4sc0_support_block(

  // GT Reference Clock
  .refclk_p               (refclk0p),
  .refclk_n               (refclk0n),

  // GT Common Ports


   // Rx
  .rx_reset             (grstp),

  .rx_core_clk_out      (coregen_clk),

  .rx_sysref            (rx_sysref),
  .rx_sync              (rx_sync),

  .rxp                  (rxp),
  .rxn                  (rxn),

  // Rx AXI-S interface for each lane
  .rx_aclk              (),
  .rx_aresetn           (rx_aresetn),

  .rx_start_of_frame    (),
  .rx_end_of_frame      (),
  .rx_frame_error       (),

  .rx_tdata             ({rx3_tdata,
                          rx2_tdata,
                          rx1_tdata,
                          rx0_tdata}),
  .rx_tvalid            (rx_tvalid),

  // AXI-Lite Control/Status
  .s_axi_aclk           (s_axi_aclk),
  .s_axi_aresetn        (s_axi_aresetn),
  .s_axi_awaddr         (s_axi_awaddr),
  .s_axi_awvalid        (s_axi_awvalid),
  .s_axi_awready        (s_axi_awready),
  .s_axi_wdata          (s_axi_wdata),
  .s_axi_wstrb          (4'b1111),
  .s_axi_wvalid         (s_axi_wvalid),
  .s_axi_wready         (s_axi_wready),
  .s_axi_bresp          (s_axi_bresp),
  .s_axi_bvalid         (s_axi_bvalid),
  .s_axi_bready         (s_axi_bready),
  .s_axi_araddr         (s_axi_araddr),
  .s_axi_arvalid        (s_axi_arvalid),
  .s_axi_arready        (s_axi_arready),
  .s_axi_rdata          (s_axi_rdata),
  .s_axi_rresp          (s_axi_rresp),
  .s_axi_rvalid         (s_axi_rvalid),
  .s_axi_rready         (s_axi_rready)
);

//capture configuration register
wire              capture_done, fetch_done, pre_fetch_done;
wire              clk_wiz_rx_fclk_lk;

wire  [3:0]       MEMSIZE_SET;
wire  [2:0]       CHSEL_SET;
//capture control register
wire              START_SET;
wire              FETONLY_SET;
//capture reset register
wire              CTRLRST_SET;
//capture status register
wire  [3:0] #1    CHVLID_DET   = {rx_tvalid, rx_tvalid, rx_tvalid, rx_tvalid};
wire        #1    FETDONE_DET  = fetch_done;
wire        #1    CAPDONE_DET  = capture_done;
//SPI configuration register
wire  [2:0]       SPI_SET;
//clock status register
wire        #1    SYNC_DET     = rx_sync;
wire        #1    ARST_DET     = rx_aresetn;     
wire        #1    SYSLK_DET    = rx_sysclk_lk;     
wire        #1    CORELK_DET   = clk_wiz_rx_aclk_lk;
wire        #1    FIFOLK_DET   = clk_wiz_rx_fclk_lk;
//jesd204b wb register
wire  [31:0]      JESD204B_WB_SET;
//jesd204b configuration register    
wire  [5:0]       WINDEX_SET;
wire  [5:0]       W2INDEX_SET;
wire              WRCE_SET;
wire              WRACK_DET;
    
//jesd204b rb register
wire  [31:0]      JESD204B_RB_DET;
//jesd204b check register    
wire  [5:0]       RINDEX_SET;
wire  [5:0]       R2INDEX_SET;
wire              RDCE_SET;
wire              RDACK_DET;
                     
wire  [7:0]       adport_in; 
wire  [7:0]       adport_out;                

assign      #4.5  adport       = (hostrw) ? adport_out : 8'hz;
assign      #1    adport_in    = adport;


//i2c master interface
wire        SDA_SET, SDA_DET, SCL_SET;
assign      #4.5  ssda         = (~SDA_SET) ? 1'b0 : 1'bz;
assign      #1    SDA_DET      = ssda;

assign      #4.5  sscl         = (~SCL_SET) ? 1'b0 : 1'bz;  

//spi    
  assign #4 adcck    = sck;
  
  assign #4 adccsa   = (SPI_SET == 3'b000) ? scs  : 1'b1;
  assign #4 adccsb   = (SPI_SET == 3'b001) ? scs  : 1'b1;
  assign #4 clkgencs = (SPI_SET == 3'b010) ? scs  : 1'b1;
  assign #4 sysrefcs = (SPI_SET == 3'b011) ? scs  : 1'b1;
  assign #4 s6954cs  = (SPI_SET == 3'b100) ? scs  : 1'b1;

  assign #4 adcdi    = mosi;
  assign #4 miso     = adcdo;


//sync err detect  
reg  SYNCERR_DET; 
wire #1 SYNCERR_DET_rst = grstp | START_SET;
always @(posedge core_clk or posedge SYNCERR_DET_rst)
   if (SYNCERR_DET_rst)
     SYNCERR_DET        <= #1 1'b0;
   else if ((~CAPDONE_DET) & (~SYNC_DET))
     SYNCERR_DET        <= #1 1'b1;
   
     
             
ftdi_cfg ftdi_cfg
(
//id register
//capture configuration register
    .MEMSIZE_SET(MEMSIZE_SET),
    .CHSEL_SET(CHSEL_SET),
//capture control register
    .START_SET(START_SET),
    .FETONLY_SET(FETONLY_SET),  
//capture reset register
    .CTRLRST_SET(CTRLRST_SET),
//capture status register
    .CHVLID_DET(CHVLID_DET),
    .SYNCERR_DET(SYNCERR_DET),//sync err detect
    .FETDONE_DET(FETDONE_DET),
    .CAPDONE_DET(CAPDONE_DET),
//i2c write register    
    .SDA_SET(SDA_SET),
    .SCL_SET(SCL_SET),
//i2c read register            
    .SDA_DET(SDA_DET),    
//SPI configuration register
    .SPI_SET(SPI_SET),
    .RCLK_SYNC_SET(rclk_sync),//gyu rclk sref ctrl
    .SREF_SYNC_SET(sref_sync),
//clock status register
    .SYNC_DET(SYNC_DET),
    .ARST_DET(ARST_DET),
    .SYSLK_DET(SYSLK_DET),     
    .CORELK_DET(CORELK_DET),
    .FIFOLK_DET(FIFOLK_DET),
//jesd204b wb register
    .JESD204B_WB_SET(JESD204B_WB_SET),
//jesd204b configuration register    
    .WINDEX_SET(WINDEX_SET),
    .W2INDEX_SET(W2INDEX_SET),
    .WRCE_SET(WRCE_SET),
    .WRACK_DET(WRACK_DET),
//jesd204b rb register
    .JESD204B_RB_DET(JESD204B_RB_DET),
//jesd204b check register    
    .RINDEX_SET(RINDEX_SET),
    .R2INDEX_SET(R2INDEX_SET),
    .RDCE_SET(RDCE_SET),
    .RDACK_DET(RDACK_DET),

    .cfigclk(s_axi_aclk),     
    
    .actpls(actpls),
    .hostrw(hostrw),
    .adflag(adflag),
    
    
    .rst(grstn),
    .selfclk(bram_a_clk),
    
    .adport_in(adport_in), 
    .adport_out(adport_out)
       
);

axi_cfg axi_cfg
(
    .m_axi_aclk(s_axi_aclk),
    .m_axi_aresetn(s_axi_aresetn),
  
    .m_axi_awaddr(s_axi_awaddr),
    .m_axi_awvalid(s_axi_awvalid),
    .m_axi_awready(s_axi_awready),
  
    .m_axi_wdata(s_axi_wdata),
    .m_axi_wvalid(s_axi_wvalid),
    .m_axi_wready(s_axi_wready),
  
    .m_axi_bresp(s_axi_bresp),
    .m_axi_bvalid(s_axi_bvalid),
    .m_axi_bready(s_axi_bready),
  
    .m_axi_araddr(s_axi_araddr),
    .m_axi_arvalid(s_axi_arvalid),
    .m_axi_arready(s_axi_arready),
  
    .m_axi_rdata(s_axi_rdata),
    .m_axi_rresp(s_axi_rresp),
    .m_axi_rvalid(s_axi_rvalid),
    .m_axi_rready(s_axi_rready),

//jesd204b wb register
    .JESD204B_WB_ISET(JESD204B_WB_SET),
//jesd204b configuration register    
    .WINDEX_ISET(WINDEX_SET),
    .W2INDEX_ISET(W2INDEX_SET),
    .WRCE_ISET(WRCE_SET),
    .WRACK_ODET(WRACK_DET),
    
//jesd204b rb register
    .JESD204B_RB_ODET(JESD204B_RB_DET),
//jesd204b check register    
    .RINDEX_ISET(RINDEX_SET),
    .R2INDEX_ISET(R2INDEX_SET),
    .RDCE_ISET(RDCE_SET),
    .RDACK_ODET(RDACK_DET)
    
);

reg [9:0] BUFFER_A_ADDR_CNT;
reg [12:0]  BUFFER_B_ADDR_CNT;

//reg [55:0] BUFSIZE_DSPLY;
always @ (MEMSIZE_SET)
  casex(MEMSIZE_SET)
  // MEMSIZE_128KSPS
  4'b1010 :  begin BUFFER_A_ADDR_CNT <=  #1 10'h3ff;
                   BUFFER_B_ADDR_CNT <=  #1 13'h1fff;
 //                  BUFSIZE_DSPLY   <=  #1 {_1, _2, _8, _K, _S, _P, _s};
             end
  // MEMSIZE_64KSPS
  4'b1001 :  begin BUFFER_A_ADDR_CNT <=  #1 10'h1ff;
                   BUFFER_B_ADDR_CNT <=  #1 13'h0fff;
  //                 BUFSIZE_DSPLY   <=  #1 {_sp, _6, _4, _K, _S, _P, _s};
             end  
  // MEMSIZE_32KSPS
  4'b1000 :  begin BUFFER_A_ADDR_CNT <=  #1 10'h0ff;
                   BUFFER_B_ADDR_CNT <=  #1 13'h07ff;
  //                 BUFSIZE_DSPLY   <=  #1 {_sp, _3, _2, _K, _S, _P, _s};
             end  
  // MEMSIZE_16KSPS
  4'b0111 :  begin BUFFER_A_ADDR_CNT <=  #1 10'h07f;
                   BUFFER_B_ADDR_CNT <=  #1 13'h03ff;
  //                 BUFSIZE_DSPLY   <=  #1 {_sp, _1, _6, _K, _S, _P, _s};
             end  
  // MEMSIZE_8KSPS
  4'b0110 :  begin BUFFER_A_ADDR_CNT <=  #1 10'h03f;
                   BUFFER_B_ADDR_CNT <=  #1 13'h01ff;
  //                 BUFSIZE_DSPLY   <=  #1 {_sp, _sp, _8, _K, _S, _P, _s};
             end  
  // MEMSIZE_4KSPS
  4'b0101 :  begin BUFFER_A_ADDR_CNT <=  #1 10'h01f;
                   BUFFER_B_ADDR_CNT <=  #1 13'h00ff;
  //                 BUFSIZE_DSPLY   <=  #1 {_sp, _sp, _4, _K, _S, _P, _s};
             end  
  // MEMSIZE_2KSPS
  4'b0100 :  begin BUFFER_A_ADDR_CNT <=  #1 10'h00f;
                   BUFFER_B_ADDR_CNT <=  #1 13'h007f;
  //                 BUFSIZE_DSPLY   <=  #1 {_sp, _sp, _2, _K, _S, _P, _s};
             end  
  // MEMSIZE_1KSPS
  4'b0011 :  begin BUFFER_A_ADDR_CNT <=  #1 10'h007;
                   BUFFER_B_ADDR_CNT <=  #1 13'h003f;
  //                 BUFSIZE_DSPLY   <=  #1 {_sp, _sp, _1, _K, _S, _P, _s};
             end  
  // MEMSIZE_512SPS
  4'b0010 :  begin BUFFER_A_ADDR_CNT <=  #1 10'h003;
                   BUFFER_B_ADDR_CNT <=  #1 13'h001f;
  //                 BUFSIZE_DSPLY   <=  #1 {_sp, _5, _1, _2, _S, _P, _s};
             end  
  // MEMSIZE_256SPS
  4'b0001 :  begin BUFFER_A_ADDR_CNT <=  #1 10'h001;
                   BUFFER_B_ADDR_CNT <=  #1 13'h000f;
  //                 BUFSIZE_DSPLY   <=  #1 {_sp, _2, _5, _6, _S, _P, _s};
             end  
  // MEMSIZE_128SPS
  4'b0000 :  begin BUFFER_A_ADDR_CNT <=  #1 10'h000;
                   BUFFER_B_ADDR_CNT <=  #1 13'h0007;
  //                 BUFSIZE_DSPLY   <=  #1 {_sp, _1, _2, _8, _S, _P, _s};
             end  
  // MEMSIZE_128SPS
  default :  begin BUFFER_A_ADDR_CNT <=  #1 10'h000;
                   BUFFER_B_ADDR_CNT <=  #1 13'h0007;
  //                 BUFSIZE_DSPLY   <=  #1 {_sp, _1, _2, _8, _S, _P, _s};
             end   
  endcase

wire         bram_b_clk;
wire [14:0]  bram_a_addr;
wire [17:0]  bram_b_addr;  

wire [2:0]   bram_a_addr_dummy;
wire         capture_data, fetch_data;
wire         bram_wren;


clk_wiz_rx_fclk clk_wiz_rx_fclk
 (// Clock in ports
.clk_in1(fifoclk),
  // Clock out ports
.clk_out1(bram_b_clk),
  // Status and control signals
.reset(grstp | START_SET),
.locked(clk_wiz_rx_fclk_lk)

 );

xlbram_a_addr xlbram_a_addr
(
.addr_trigger(bram_a_clk), 
.addr({bram_a_addr_dummy, bram_a_addr}),

.cnt_done(capture_done),
.cnt_data(capture_data),

.BUFFER_ADDR_CNT({3'b0, BUFFER_A_ADDR_CNT}),
.rst(grstn & CTRLRST_SET));

wire shift_addr;
xlbram_b_addr # 
(
.BRAM_B_ADDR_WIDTH(18)
)
xlbram_b_addr
(
.addr_trigger(bram_b_clk), 
.addr(bram_b_addr),

.pre_cnt_done(pre_fetch_done),
.cnt_data(fetch_data),

.BUFFER_ADDR_CNT(BUFFER_B_ADDR_CNT),
.shift_addr(shift_addr),
.rst(grstn & CTRLRST_SET));


xlbram_ctrl xlbram_ctrl(
.rst(grstn & CTRLRST_SET),

.capture_start(START_SET),
.capture_done(capture_done),
.fetch_only(FETONLY_SET),

.a_clk(bram_a_clk),

.bram_a_add_ctrl(capture_data),
.bram_wren(bram_wren),

.txen(txen),
.wrn(wrn),

.fetch_start(clk_wiz_rx_fclk_lk),
.fetch_done(fetch_done),
.pre_fetch_done(pre_fetch_done),

.b_clk(bram_b_clk),
.shift_addr(shift_addr),
.bram_b_add_ctrl(fetch_data)

  );

  
(* DONT_TOUCH *)reg [31:0] rx0_prev_tdata;
(* DONT_TOUCH *)reg [31:0] rx1_prev_tdata;
(* DONT_TOUCH *)reg [31:0] rx2_prev_tdata;
(* DONT_TOUCH *)reg [31:0] rx3_prev_tdata;




always @(posedge core_clk or negedge grstn)
if (~grstn)
begin
  rx0_prev_tdata <= #1 32'b0;
  rx1_prev_tdata <= #1 32'b0;
  rx2_prev_tdata <= #1 32'b0;
  rx3_prev_tdata <= #1 32'b0;
end
else
begin
  rx0_prev_tdata <= #1 rx0_tdata;
  rx1_prev_tdata <= #1 rx1_tdata;
  rx2_prev_tdata <= #1 rx2_tdata;
  rx3_prev_tdata <= #1 rx3_tdata;
end

wire #1 bram_a_clk_mux_sel = (CHSEL_SET == 3'b100);


BUFGMUX    bram_a_clk_mux (
                          .I0(clkdb2),
                          .I1(clkdb1),
                          .O(bram_a_clk),
                          .S(bram_a_clk_mux_sel));


 
(* DONT_TOUCH *)reg [63:0] rx0_ip2m_data;
always @(posedge bram_a_clk or negedge grstn)
if (~grstn)
  rx0_ip2m_data            <= #1 64'b0;
else
  case({CHSEL_SET})
  3'b000  : rx0_ip2m_data  <= #1 {rx0_tdata[23:16],      rx0_tdata[31:24],      rx0_tdata[7:0],      rx0_tdata[15:8],
                                  rx0_prev_tdata[23:16], rx0_prev_tdata[31:24], rx0_prev_tdata[7:0], rx0_prev_tdata[15:8]};
  3'b001  : rx0_ip2m_data  <= #1 {rx1_tdata[23:16],      rx1_tdata[31:24],      rx1_tdata[7:0],      rx1_tdata[15:8],
                                  rx1_prev_tdata[23:16], rx1_prev_tdata[31:24], rx1_prev_tdata[7:0], rx1_prev_tdata[15:8]};
  3'b100  : rx0_ip2m_data  <= #1 {rx1_tdata[23:16],      rx1_tdata[31:24],      rx1_tdata[7:0],      rx1_tdata[15:8],
                                  rx0_tdata[23:16],      rx0_tdata[31:24],      rx0_tdata[7:0],      rx0_tdata[15:8]};
  default : rx0_ip2m_data  <= #1 {rx1_tdata[23:16],      rx1_tdata[31:24],      rx1_tdata[7:0],      rx1_tdata[15:8],
                                  rx0_tdata[23:16],      rx0_tdata[31:24],      rx0_tdata[7:0],      rx0_tdata[15:8]};
  endcase

(* DONT_TOUCH *)reg [63:0] rx1_ip2m_data;  
always @(posedge bram_a_clk or negedge grstn)
if (~grstn)
  rx1_ip2m_data            <= #1 64'b0;
else
  case({CHSEL_SET})
  3'b010  : rx1_ip2m_data  <= #1 {rx2_tdata[23:16],      rx2_tdata[31:24],      rx2_tdata[7:0],      rx2_tdata[15:8],
                                  rx2_prev_tdata[23:16], rx2_prev_tdata[31:24], rx2_prev_tdata[7:0], rx2_prev_tdata[15:8]};
  3'b011  : rx1_ip2m_data  <= #1 {rx3_tdata[23:16],      rx3_tdata[31:24],      rx3_tdata[7:0],      rx3_tdata[15:8],
                                  rx3_prev_tdata[23:16], rx3_prev_tdata[31:24], rx3_prev_tdata[7:0], rx3_prev_tdata[15:8]};
  3'b101  : rx1_ip2m_data  <= #1 {rx3_tdata[23:16],      rx3_tdata[31:24],      rx3_tdata[7:0],      rx3_tdata[15:8],
                                  rx2_tdata[23:16],      rx2_tdata[31:24],      rx2_tdata[7:0],      rx2_tdata[15:8]};
  default : rx1_ip2m_data  <= #1 {rx3_tdata[23:16],      rx3_tdata[31:24],      rx3_tdata[7:0],      rx3_tdata[15:8],
                                  rx2_tdata[23:16],      rx2_tdata[31:24],      rx2_tdata[7:0],      rx2_tdata[15:8]};
  endcase

reg fifo_data_sel;
blk_mem_gen_gcp blk_mem_gen_gcp(
  .clka(bram_a_clk),
  .wea(bram_wren),
  .addra(bram_a_addr),
  .dina({rx1_ip2m_data, rx0_ip2m_data}),
//gyu ch1s1, ch1s0, ch0s1, ch0s0 for all
//gyu ch0s3, ch0s2, ch0s1, ch0s0 for ch0
//gyu ch1s3, ch1s2, ch1s1, ch1s0 for ch1          
  .clkb(bram_b_clk),
  .addrb({bram_b_addr[17:3], fifo_data_sel, bram_b_addr[2:0]}),
  .doutb(fifo_data)
);

always @(fifo_data_sel  or 
         grstn          or 
         CHSEL_SET)
         
if (~grstn)
  fifo_data_sel                <= #1  1'b0;
else
  case({CHSEL_SET})
  3'b000  : fifo_data_sel      <= #1  1'b0;
  3'b001  : fifo_data_sel      <= #1  1'b0;
  3'b010  : fifo_data_sel      <= #1  1'b1;
  3'b011  : fifo_data_sel      <= #1  1'b1;
  3'b100  : fifo_data_sel      <= #1  1'b0;
  3'b101  : fifo_data_sel      <= #1  1'b1; 
  default : fifo_data_sel      <= #1  1'b0;
  endcase



endmodule


