/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2015                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.0     10:08:58 09/09/2015 Gary Yu                                *
* File Created                                                                *
*                                                                             *
* Additional Comments:                                                        *
*                                                                             *
******************************************************************************/

`timescale 1ns / 1ps

module xlbram_ctrl(
rst,

capture_start,
capture_done,
fetch_only,

a_clk,

bram_a_add_ctrl,
bram_wren,

txen,
wrn,

fetch_start,
fetch_done,
pre_fetch_done,

b_clk,
shift_addr,
bram_b_add_ctrl

  );
input   rst;
  
input   capture_start;
input   capture_done;
input   fetch_only;

input   a_clk;

output  bram_a_add_ctrl;
output  bram_wren;

input   txen;
output  wrn;

input   fetch_start;
output  fetch_done;
input   pre_fetch_done;

input   b_clk;
output  shift_addr;
output  bram_b_add_ctrl;


reg     bram_a_add_ctrl;
reg     bram_wren;

reg     wrn;

//reg     bram_b_add_ctrl;//gyu handshake



//sync capture_start
reg [3:0] capture_sp_reg;
reg capture_trig;

always @ (posedge a_clk or negedge rst)
  if (~rst)
  begin
    capture_sp_reg         <= #1 4'h0;
    capture_trig           <= #1 1'b0;
  end  
  else
  begin
    capture_sp_reg[0]      <= #1 capture_start;
    capture_sp_reg[1]      <= #1 capture_sp_reg[0];
    capture_sp_reg[2]      <= #1 capture_sp_reg[1];
    capture_sp_reg[3]      <= #1 capture_sp_reg[2];
    capture_trig           <= #1 (~capture_sp_reg[3]) & capture_sp_reg[2]; 
  end  

  
//process capture_trig & capture_done
reg [6:0] capture_reg;

always @ (posedge a_clk or negedge rst)
  if(~rst)
    capture_reg[6:0] <= #1 7'b0;
  else
  begin
    capture_reg[0] <= #1 capture_trig;
    capture_reg[1] <= #1 capture_reg[0];
    capture_reg[2] <= #1 capture_reg[1];
    capture_reg[3] <= #1 capture_reg[2];
    capture_reg[4] <= #1 capture_reg[3];
    capture_reg[5] <= #1 capture_reg[4];
    capture_reg[6] <= #1 capture_reg[5];
  end

reg [3:0] capture_done_reg;
      
always @ (posedge a_clk or negedge rst)
  if(~rst)
    capture_done_reg[3:0] <= #1 4'b0;
  else
  begin
    capture_done_reg[0] <= #1 capture_done;
    capture_done_reg[1] <= #1 capture_done_reg[0];
    capture_done_reg[2] <= #1 capture_done_reg[1];
    capture_done_reg[3] <= #1 capture_done_reg[2];
  end

   
//generate bram_a_add_ctrl 
always @ (posedge a_clk or negedge rst)
  if(~rst)
    bram_a_add_ctrl      <= #1 1'b0;
  else if (capture_reg[5])
    bram_a_add_ctrl      <= #1 1'b1;
  else if (capture_done_reg[0])
    bram_a_add_ctrl      <= #1 1'b0;

       
//generate bram_wren 
always @ (posedge a_clk or negedge rst)
  if(~rst)
    bram_wren            <= #1 1'b0;
  else if (capture_reg[5] & (~fetch_only))
    bram_wren            <= #1 1'b1;
  else if (capture_done)
    bram_wren            <= #1 1'b0;


//generate fetch_permit   
reg      fetch_permit;
always @ (posedge a_clk or negedge rst)
begin
  if(~rst)
    fetch_permit    <= #1 1'b0;
  else if (capture_done_reg[3])
    fetch_permit    <= #1 1'b1;
end


//sync fetch_start
reg [3:0] fetch_sp_reg;
reg fetch_trig;

wire #1 fetch_rst = rst & fetch_start;
always @ (posedge b_clk or negedge fetch_rst)
  if (~fetch_rst)
  begin
    fetch_sp_reg         <= #1 4'h0;
    fetch_trig           <= #1 1'b0;
  end  
  else
  begin
    fetch_sp_reg[0]      <= #1 fetch_permit;
    fetch_sp_reg[1]      <= #1 fetch_sp_reg[0];
    fetch_sp_reg[2]      <= #1 fetch_sp_reg[1];
    fetch_sp_reg[3]      <= #1 fetch_sp_reg[2];
    fetch_trig           <= #1 fetch_sp_reg[1]; 
  end  

//sync txen
reg [6:0]  txen_reg;
always @ (posedge b_clk or negedge fetch_rst)
  if(~fetch_rst)
    txen_reg             <= #1 7'hff;
  else if (fetch_trig)
  begin
    txen_reg[0]          <= #1 txen;
    txen_reg[1]          <= #1 txen_reg[0];
    txen_reg[2]          <= #1 txen_reg[1];
    txen_reg[3]          <= #1 txen_reg[2];
    txen_reg[4]          <= #1 txen_reg[3];
    txen_reg[5]          <= #1 txen_reg[4];
    txen_reg[6]          <= #1 txen_reg[5];
  end    

//process pre_fetch_done
reg       pre_fetch_done_reg;            
always @ (posedge b_clk or negedge fetch_rst)
  if(~fetch_rst)
    pre_fetch_done_reg <= #1 1'b0;
  else
    pre_fetch_done_reg <= #1 pre_fetch_done;
 
//generate fetch_done
reg       fetch_done;  
always @ (posedge b_clk or negedge fetch_rst)
  if(~fetch_rst)
    fetch_done     <= #1 1'b0;
  else if((~txen) & (pre_fetch_done))
    fetch_done     <= #1 1'b1;       
  
//process fetch_done
reg [3:0] fetch_done_reg;  
always @ (posedge b_clk or negedge fetch_rst)
  if(~fetch_rst)
    fetch_done_reg[3:0] <= #1 4'b0;
  else
  begin
    fetch_done_reg[0] <= #1 fetch_done;
    fetch_done_reg[1] <= #1 fetch_done_reg[0];
    fetch_done_reg[2] <= #1 fetch_done_reg[1];
    fetch_done_reg[3] <= #1 fetch_done_reg[2];
  end

    
//process shift_addr//gyu handshake
reg        shift_addr;
always @ (posedge b_clk or negedge fetch_rst)
  if(~fetch_rst)
    shift_addr           <= #1 1'b0;    
  else if ((fetch_done_reg[1] == 1'b0) && (txen_reg == 7'h01) && (pre_fetch_done_reg ==  1'b0))
    shift_addr           <= #1 1'b1;
  else 
    shift_addr           <= #1 1'b0;    
    

assign #1 bram_b_add_ctrl = ~(txen | txen_reg[1] | fetch_done_reg[1]);//gyu handshake


//generate wrn
always @ (posedge b_clk or negedge fetch_rst)
  if(~fetch_rst)
    wrn    <= #1 1'b1;
  else if (fetch_done_reg[1])
    wrn    <= #1 1'b1;  
  else
    wrn    <= #1 txen_reg[1];
                        

endmodule

