module ltc2123_jesd204b_logic
(
  output wire         PLL_ref,  
  output wire         div2_rstb,
  output wire         two_ln_per_adc,
  output wire         two_adc_per_ln, 
  output wire         ser_A1_en,
  output wire         ser_A0_en,
  output wire         ser_B0_en,
  output wire         ser_B1_en, 
  output wire [19:0]  DAT_A1, 
  output wire [19:0]  DAT_A0,
  output wire [19:0]  DAT_B0,
  output wire [19:0]  DAT_B1,
   
  input  wire         dev_clk,
  input  wire         adc_clk,
  input  wire         nap,
  input  wire         pd_all,
  input  wire         pda,
  input  wire         pdb,
  input  wire         single, 
  input  wire [15:0]  DATA_A,
  input  wire [15:0]  DATA_B,
  input  wire         RST_dis,
  input  wire         alert_en,
  input  wire [2:0]   R_,  
  input  wire         sync,
  input  wire         sysref,      
  input  wire         CLK_frm,
  input  wire         CLK_2char,     
  input  wire [7:0]   DID,           
  input  wire [3:0]   BID, 
  input  wire         SCR,           
  input  wire [2:0]   L_,            
  input  wire [4:0]   K_,            
  input  wire [2:0]   M_,            
  input  wire [1:0]   CS,            
  input  wire [3:0]   N_,            
  input  wire [1:0]   SUBCLASSV, 
  input  wire         LAS_dis, 
  input  wire         LAM_dis,       
  input  wire         FAM_dis,       
  input  wire         TX_SYNC_en,  
  input  wire [2:0]   PAT,   
  input  wire [2:0]   LID_A1, 
  input  wire [2:0]   LID_A0, 
  input  wire [2:0]   LID_B0,
  input  wire [2:0]   LID_B1,         
  input  wire [1:0]   CID_A1,
  input  wire [1:0]   CID_A0,
  input  wire [1:0]   CID_B0,
  input  wire [1:0]   CID_B1,         

  input  wire         static_clk,
  input  wire         scan_mode

  );        

  wire [15:0]  DATA_A0;
  wire [15:0]  DATA_A1;
  wire [15:0]  DATA_B0;
  wire [15:0]  DATA_B1;
  wire         SYNC_det; 
  wire         MF_clk;       
  wire         MF_end;        
  wire         MF_chg;        
  wire         MF_chg_frm;    
     

jesd204b_top_x4 jesd204b_top_x4
(
  .DAT_A0(DAT_A0),         
  .DAT_A1(DAT_A1),         
  .DAT_B0(DAT_B0),         
  .DAT_B1(DAT_B1),         

  .scan_mode(scan_mode),
  .static_clk(static_clk),
      
  .ser_A0_en(ser_A0_en),      
  .ser_A1_en(ser_A1_en),      
  .ser_B0_en(ser_B0_en),      
  .ser_B1_en(ser_B1_en),         
  .CLK_2char(CLK_2char),      
  .CLK_frm(CLK_frm),        
  .MF_clk(MF_clk),         
  .MF_end(MF_end),         
  .MF_chg(MF_chg),         
  .MF_chg_frm(MF_chg_frm),     
  .SYNC_det(SYNC_det),       
  .TX_SYNC_en(TX_SYNC_en),     
  .DATA_A0(DATA_A0),        
  .DATA_A1(DATA_A1),        
  .DATA_B0(DATA_B0),        
  .DATA_B1(DATA_B1),        
  .DID(DID),            
  .BID(BID),            
  .L_(L_),             
  .K_(K_),             
  .M_(M_),             
  .CS(CS),             
  .N_(N_),             
  .SUBCLASSV(SUBCLASSV),      
  .LID_A0(LID_A0),         
  .LID_A1(LID_A1),         
  .LID_B0(LID_B0),         
  .LID_B1(LID_B1),          
  .CID_A0(CID_A0),         
  .CID_A1(CID_A1),         
  .CID_B0(CID_B0),         
  .CID_B1(CID_B1),          
  .SCR(SCR),            
  .LAS_dis(LAS_dis),        
  .FAM_dis(FAM_dis),        
  .LAM_dis(LAM_dis),        
  .two_adc_per_ln(two_adc_per_ln), 
  .two_ln_per_adc(two_ln_per_adc), 
  .PAT(PAT)                                               
);

jesd204b_custom jesd204b_custom
(
  .DATA_A0(DATA_A0),       
  .DATA_A1(DATA_A1),       
  .DATA_B0(DATA_B0),       
  .DATA_B1(DATA_B1),       
  .MF_clk(MF_clk),        
  .MF_chg(MF_chg),        
  .MF_end(MF_end),        
  .PLL_ref(PLL_ref),       
  .SYNC_det(SYNC_det),      
  .div2_rstb(div2_rstb),     
  .ser_A0_en(ser_A0_en),     
  .ser_A1_en(ser_A1_en),     
  .ser_B0_en(ser_B0_en),     
  .ser_B1_en(ser_B1_en),     
  .two_adc_per_ln(two_adc_per_ln),
  .CLK_frm(CLK_frm),       
  .DATA_A(DATA_A),         
  .DATA_B(DATA_B),         
  .K_(K_),            
  .L_(L_),            
  .R_(R_),            
  .RST_dis(RST_dis),       
  .SUBCLASSV(SUBCLASSV),     
  .adc_clk(adc_clk),       
  .alert_en(alert_en),      
  .dev_clk(dev_clk),       
  .nap(nap),           
  .pd_all(pd_all),        
  .pda(pda),           
  .pdb(pdb),           
  .single(single),        
  .sync(sync),          
  .sysref(sysref),        
  .two_ln_per_adc(two_ln_per_adc),
  .MF_chg_frm(MF_chg_frm)     
);
endmodule

module jesd204b_top_x4
(
  output wire [19:0] DAT_A0, 
  output wire [19:0] DAT_A1,
  output wire [19:0] DAT_B0,
  output wire [19:0] DAT_B1,     

  input wire         scan_mode,
  input wire         static_clk,
     
  input wire         ser_A0_en,
  input wire         ser_A1_en,
  input wire         ser_B0_en,
  input wire         ser_B1_en,        
  input wire         CLK_2char,     
  input wire         CLK_frm,       
  input wire         MF_clk,        
  input wire         MF_end,        
  input wire         MF_chg,        
  input wire         MF_chg_frm,    
  input wire         SYNC_det,      
  input wire         TX_SYNC_en,    
  input wire [15:0]  DATA_A0, 
  input wire [15:0]  DATA_A1,
  input wire [15:0]  DATA_B0,
  input wire [15:0]  DATA_B1,     
  input wire [7:0]   DID,           
  input wire [3:0]   BID,           
  input wire [2:0]   L_,            
  input wire [4:0]   K_,            
  input wire [2:0]   M_,            
  input wire [1:0]   CS,            
  input wire [3:0]   N_,            
  input wire [1:0]   SUBCLASSV,     
  input wire [2:0]   LID_A0, 
  input wire [2:0]   LID_A1, 
  input wire [2:0]   LID_B0,
  input wire [2:0]   LID_B1,         
  input wire [1:0]   CID_A0,
  input wire [1:0]   CID_A1,
  input wire [1:0]   CID_B0,
  input wire [1:0]   CID_B1,         
  input wire         SCR,           
  input wire         LAS_dis,       
  input wire         FAM_dis,       
  input wire         LAM_dis,       
  input wire         two_adc_per_ln,
  input wire         two_ln_per_adc,
  input wire [2:0]   PAT                                              
);          

                                

jesd204b_top  jesd204b_top_A0               
( 
  .DATA_out(DAT_A0),
  .static_clk(static_clk),
  .scan_mode(scan_mode),
  .ser_en(ser_A0_en),        
  .CLK_2char(CLK_2char),     
  .CLK_frm(CLK_frm),       
  .MF_clk(MF_clk),        
  .MF_end(MF_end),        
  .MF_chg(MF_chg),        
  .MF_chg_frm(MF_chg_frm),    
  .SYNC_det(SYNC_det),      
  .TX_SYNC_en_in(TX_SYNC_en),    
  .ADC_data(DATA_A0),      
  .DID_in(DID),           
  .BID_in(BID),           
  .L_in(L_),            
  .K_in(K_),            
  .M_in(M_),            
  .CS_in(CS),            
  .N_in(N_),            
  .SUBCLASSV_in(SUBCLASSV),     
  .LID_in(LID_A0),           
  .CID_in(CID_A0),           
  .SCR_in(SCR),           
  .LAS_dis_in(LAS_dis),       
  .FAM_dis_in(FAM_dis),       
  .LAM_dis_in(LAM_dis),       
  .two_adc_per_ln_in(two_adc_per_ln),
  .two_ln_per_adc_in(two_ln_per_adc),
  .PAT_in(PAT)                                             
);   

jesd204b_top  jesd204b_top_A1               
(                                   
  .DATA_out(DAT_A1),
  .static_clk(static_clk),
  .scan_mode(scan_mode),
  .ser_en(ser_A1_en),        
  .CLK_2char(CLK_2char),     
  .CLK_frm(CLK_frm),       
  .MF_clk(MF_clk),        
  .MF_end(MF_end),        
  .MF_chg(MF_chg),        
  .MF_chg_frm(MF_chg_frm),    
  .SYNC_det(SYNC_det),      
  .TX_SYNC_en_in(TX_SYNC_en),    
  .ADC_data(DATA_A1),      
  .DID_in(DID),           
  .BID_in(BID),           
  .L_in(L_),            
  .K_in(K_),            
  .M_in(M_),            
  .CS_in(CS),            
  .N_in(N_),            
  .SUBCLASSV_in(SUBCLASSV),     
  .LID_in(LID_A1),           
  .CID_in(CID_A1),           
  .SCR_in(SCR),           
  .LAS_dis_in(LAS_dis),       
  .FAM_dis_in(FAM_dis),       
  .LAM_dis_in(LAM_dis),       
  .two_adc_per_ln_in(two_adc_per_ln),
  .two_ln_per_adc_in(two_ln_per_adc),
  .PAT_in(PAT)                                             
);                                

jesd204b_top  jesd204b_top_B0
(
  .DATA_out(DAT_B0),
  .static_clk(static_clk),
  .scan_mode(scan_mode),  
  .ser_en(ser_B0_en),
  .CLK_2char(CLK_2char),
  .CLK_frm(CLK_frm),
  .MF_clk(MF_clk),
  .MF_end(MF_end),
  .MF_chg(MF_chg),
  .MF_chg_frm(MF_chg_frm),
  .SYNC_det(SYNC_det),
  .TX_SYNC_en_in(TX_SYNC_en),
  .ADC_data(DATA_B0),
  .DID_in(DID),
  .BID_in(BID),
  .L_in(L_),
  .K_in(K_),
  .M_in(M_),
  .CS_in(CS),
  .N_in(N_),
  .SUBCLASSV_in(SUBCLASSV),
  .LID_in(LID_B0),
  .CID_in(CID_B0),
  .SCR_in(SCR),
  .LAS_dis_in(LAS_dis),
  .FAM_dis_in(FAM_dis),
  .LAM_dis_in(LAM_dis),
  .two_adc_per_ln_in(two_adc_per_ln),
  .two_ln_per_adc_in(two_ln_per_adc),
  .PAT_in(PAT)
);

jesd204b_top  jesd204b_top_B1
(
  .DATA_out(DAT_B1),
  .static_clk(static_clk),
  .scan_mode(scan_mode), 
  .ser_en(ser_B1_en),
  .CLK_2char(CLK_2char),
  .CLK_frm(CLK_frm),
  .MF_clk(MF_clk),
  .MF_end(MF_end),
  .MF_chg(MF_chg),
  .MF_chg_frm(MF_chg_frm),
  .SYNC_det(SYNC_det),
  .TX_SYNC_en_in(TX_SYNC_en),
  .ADC_data(DATA_B1),
  .DID_in(DID),
  .BID_in(BID),
  .L_in(L_),
  .K_in(K_),
  .M_in(M_),
  .CS_in(CS),
  .N_in(N_),
  .SUBCLASSV_in(SUBCLASSV),
  .LID_in(LID_B1),
  .CID_in(CID_B1),
  .SCR_in(SCR),
  .LAS_dis_in(LAS_dis),
  .FAM_dis_in(FAM_dis),
  .LAM_dis_in(LAM_dis),
  .two_adc_per_ln_in(two_adc_per_ln),
  .two_ln_per_adc_in(two_ln_per_adc),
  .PAT_in(PAT)
);
endmodule                                         

`timescale 1ns / 1 ps

module jesd204b_top
(
  output wire [19:0] DATA_out,          // Most Significant and Least Significant 10-bit outputs of 8B10B encoders
  input wire         static_clk,
  input wire         scan_mode,         //
  input wire         ser_en,            // serial enable
  input wire         CLK_2char,         // two character clock
  input wire         CLK_frm,           // frame clock
  input wire         MF_clk,            // Multiframe clock
  input wire         MF_end,            // End of Multiframe marker
  input wire         MF_chg,            // Multiframe change occurred
  input wire         MF_chg_frm,        // Multiframe change occurred, pulse width is 1 frame
  input wire         SYNC_det,          // SYNC has been detected (4 frame clock cycles)
  input wire [15:0]  ADC_data,          // 16-bit sample from ADC
  input wire         TX_SYNC_en_in,     //
  input wire [7:0]   DID_in,            // Device ID
  input wire [3:0]   BID_in,            // Bank ID
  input wire [2:0]   L_in,              // Total Lanes
  input wire [4:0]   K_in,              // Frames per Multi-Frame
  input wire [2:0]   M_in,              //
  input wire [1:0]   CS_in,             // Control Bits per Sample
  input wire [3:0]   N_in,              //
  input wire [1:0]   SUBCLASSV_in,      // Subclass Version
  input wire [2:0]   LID_in,            // Lane ID
  input wire [1:0]   CID_in,            // Converter ID
  input wire         SCR_in,            // Scramble enable
  input wire         LAS_dis_in,        // Lane Alignment Sequence disable
  input wire         FAM_dis_in,        // Frame Alignment Monitor disable
  input wire         LAM_dis_in,        // Lane Alignment Monitor disable
  input wire         two_adc_per_ln_in, //
  input wire         two_ln_per_adc_in, //
  input wire [2:0]   PAT_in             // test patterns (000 = Normal, 001 = K28.5, 002 = K28.7, 003 = D21.5,
                                        // 004 = PRBS15 (1+x^14+x^15), 005 = Lane Alignment Sequence,
                                        // 006 = Test Samples Sequence, 007 = Modified RPAT pattern.
);


  /******************************************************************************/
  /** WIRES *********************************************************************/
  /******************************************************************************/
  wire [9:0]    CODE1;              // Most Significant 10-bit output of 8B10B encoder
  wire [9:0]    CODE2;              // Least Significant 10-bit output of 8B10B encoder
  wire [7:0]    OCTET1;
  wire [7:0]    OCTET2;
  wire [15:0]   SDATA_in;
  wire [15:0]   SDATA_out;
  wire [15:0]   TSS_out;
  wire [15:0]   RPAT_out;
  wire [15:0]   LAS_out;
  wire          K28_en1;
  wire          K28_en2;
  wire [2:0]    K28_index1;
  wire [2:0]    K28_index2;
  wire          K28_5_pat_en;
  wire          K28_7_pat_en;
  wire          SCR_en;
  wire          nextSEND_LAS;
  wire          SEND_LAS;
  wire          SEND_F;
  wire          SEND_A;
  wire          SEND_K;
  wire          LAS_pat_en;
  wire          TSS_pat_en;
  wire          RPAT_pat_en;
  wire          disparity;
  wire          FRM_end;
  wire          MF_strt;
  wire          LAS_done;
  wire          LAS_MF2;
  wire          TX_SYNC_en;             
  wire [7:0]    DID;           
  wire [3:0]    BID;           
  wire [2:0]    L_;            
  wire [4:0]    K_;             
  wire [2:0]    M_;             
  wire [1:0]    CS;            
  wire [3:0]    N_;             
  wire [1:0]    SUBCLASSV;     
  wire [2:0]    LID;           
  wire [1:0]    CID;           
  wire          SCR;           
  wire          LAS_dis;       
  wire          FAM_dis;       
  wire          LAM_dis;       
  wire          two_adc_per_ln;
  wire          two_ln_per_adc;
  wire [2:0]    PAT;          


  assign DATA_out = {CODE1, CODE2};

  /******************************************************************************/
  /** INSTANCES *****************************************************************/
  /******************************************************************************/
  jesd204b_inff jesd204b_inff                  
  (                                  
    .TX_SYNC_en(TX_SYNC_en),        
    .DID(DID),               
    .BID(BID),               
    .L_(L_),                
    .K_(K_),                
    .M_(M_),                
    .CS(CS),                
    .N_(N_),                
    .SUBCLASSV(SUBCLASSV),         
    .LID(LID),               
    .CID(CID),               
    .SCR(SCR),               
    .LAS_dis(LAS_dis),           
    .FAM_dis(FAM_dis),           
    .LAM_dis(LAM_dis),           
    .two_adc_per_ln(two_adc_per_ln),    
    .two_ln_per_adc(two_ln_per_adc),    
    .PAT(PAT),
    .static_clk(static_clk),               
    .scan_mode(scan_mode),        
    .TX_SYNC_en_in(TX_SYNC_en_in),     
    .DID_in(DID_in),            
    .BID_in(BID_in),            
    .L_in(L_in),              
    .K_in(K_in),              
    .M_in(M_in),              
    .CS_in(CS_in),             
    .N_in(N_in),              
    .SUBCLASSV_in(SUBCLASSV_in),      
    .LID_in(LID_in),            
    .CID_in(CID_in),            
    .SCR_in(SCR_in),            
    .LAS_dis_in(LAS_dis_in),        
    .FAM_dis_in(FAM_dis_in),        
    .LAM_dis_in(LAM_dis_in),        
    .two_adc_per_ln_in(two_adc_per_ln_in), 
    .two_ln_per_adc_in(two_ln_per_adc_in), 
    .PAT_in(PAT_in)             
  );                                                        

  
  jesd204b_core  jesd204b_core
  (
    .OCTET1(OCTET1),             //output
    .OCTET2(OCTET2),             //output
    .nextSEND_LAS(nextSEND_LAS), //output
    .SEND_LAS(SEND_LAS),         //output
    .SEND_F(SEND_F),             //output
    .SEND_A(SEND_A),             //output
    .SEND_K(SEND_K),             //output
    .FRM_end(FRM_end),           //output
    .MF_strt(MF_strt),           //output
    .LAS_pat_en(LAS_pat_en),     //output
    .TSS_pat_en(TSS_pat_en),     //output
    .RPAT_pat_en(RPAT_pat_en),   //output
    .K28_5_pat_en(K28_5_pat_en), //output
    .K28_7_pat_en(K28_7_pat_en), //output
    .SCR_en(SCR_en),             //output
    .SDATA_in(SDATA_in),         //output
    .SDATA_out(SDATA_out),
    .ADC_data(ADC_data),
    .TSS_out(TSS_out),
    .RPAT_out(RPAT_out),
    .LAS_out(LAS_out),
    .CLK_2char(CLK_2char),
    .CLK_frm(CLK_frm),
    .MF_clk(MF_clk),
    .MF_end(MF_end),
    .MF_chg(MF_chg),
    .MF_chg_frm(MF_chg_frm),
    .LAS_done(LAS_done),
    .FAM_dis(FAM_dis),
    .LAM_dis(LAM_dis),
    .LAS_dis(LAS_dis),
    .ser_en(ser_en),
    .SYNC_det(SYNC_det),
    .TX_SYNC_en(TX_SYNC_en),
    .PAT(PAT),
    .SCR(SCR),
    .two_adc_per_ln(two_adc_per_ln),
    .SUBCLASSV(SUBCLASSV)
  );

  jesd204b_scrambler  scrambler
  (
    .SDATA_out(SDATA_out),  //output
    .CLK_2char(CLK_2char),
    .SDATA_in(SDATA_in),
    .SCR_en(SCR_en),
    .scan_mode(scan_mode)
  );

  encode_8b10b  encoder
  (
    .CODE1(CODE1),          //output
    .CODE2(CODE2),          //output
    .disparity(disparity),  //output
    .OCTET1(OCTET1),
    .OCTET2(OCTET2),
    .CLK_2char(CLK_2char),
    .K28_en1(K28_en1),
    .K28_en2(K28_en2),
    .K28_index1(K28_index1[2:0]),
    .K28_index2(K28_index2[2:0]),
    .ser_en(ser_en)
  );

  jesd204b_K28_gen  K28_gen
  (
    .K28_en1(K28_en1),            //output
    .K28_en2(K28_en2),            //output
    .K28_index1(K28_index1[2:0]), //output
    .K28_index2(K28_index2[2:0]), //output
    .SEND_K(SEND_K),
    .SEND_F(SEND_F),
    .SEND_A(SEND_A),
    .nextSEND_LAS(nextSEND_LAS),
    .SEND_LAS(SEND_LAS),
    .ser_en(ser_en),
    .CLK_2char(CLK_2char),
    .MF_strt(MF_strt),
    .LAS_MF2(LAS_MF2),
    .LAS_done(LAS_done),
    .K28_5_pat_en(K28_5_pat_en),
    .K28_7_pat_en(K28_7_pat_en)
  );

  jesd204b_LAS MLAS
  (
    .LAS_out(LAS_out),            // output
    .LAS_MF2(LAS_MF2),            // output
    .LAS_done(LAS_done),          // output
    .CLK_2char(CLK_2char),
    .LAS_pat_en(LAS_pat_en),
    .nextSEND_LAS(nextSEND_LAS),
    .SEND_LAS(SEND_LAS),
    .DID(DID),
    .BID(BID),
    .LID(LID),
    .SCR(SCR),
    .L_(L_),
    .K_(K_),
    .M_(M_),
    .CS(CS),
    .N_(N_),
    .SUBCLASSV(SUBCLASSV),
    .FRM_end(FRM_end),
    .MF_end(MF_end),
    .MF_clk(MF_clk),
    .two_adc_per_ln(two_adc_per_ln),
    .two_ln_per_adc(two_ln_per_adc)
  );

  jesd204b_RPAT MRPAT
  (
    .RPAT_out(RPAT_out),    // output
    .CLK_2char(CLK_2char),
    .RPAT_pat_en(RPAT_pat_en),
    .disparity(disparity)
  );

  jesd204b_TSS MTSS
  (
    .TSS_out(TSS_out),      // output
    .CLK_2char(CLK_2char),
    .CLK_frm(CLK_frm),
    .MF_end(MF_end),
    .TSS_pat_en(TSS_pat_en),
    .N_(N_),
    .CID(CID),
    .LID(LID),
    .two_adc_per_ln(two_adc_per_ln),
    .two_ln_per_adc(two_ln_per_adc)
  );
endmodule

`timescale 1ns / 1 ps

module jesd204b_core (
  output reg  [7:0]  OCTET1,          // The Most significant Octet to 8b/10b encoder
  output reg  [7:0]  OCTET2,          // The Least significant Octet to 8b/10b encoder
  output reg         nextSEND_LAS,    //
  output reg         SEND_LAS,        // Send LAS (dynamic)
  output reg         SEND_F,          // Send single K28.7 for Frame Alignment Monitoring (dynamic)
  output reg         SEND_A,          // Send single K28.3 for Lane Alignment Monitoring (dynamic)
  output reg         SEND_K,          // Send K28.5 for Initial SYNC (dynamic)
  output wire        FRM_end,         // Always 1 unless two_adc_per_ln, when it is 1 for second half of frame (dynamic)
  output wire        MF_strt,         // Start of MF
  output reg         LAS_pat_en,      // Send LAS (static)
  output reg         TSS_pat_en,      // Send TSS (static)
  output reg         RPAT_pat_en,     // Send RPAT (static)
  output reg         K28_5_pat_en,    // Send K28.5 pattern (static)
  output reg         K28_7_pat_en,    // Send K28.7 pattern (static)
  output reg         SCR_en,          // Scrambler enable (static)
  output reg  [15:0] SDATA_in,        // Scrambler data input
  input  wire [15:0] SDATA_out,       // Scrambler data output
  input  wire [15:0] ADC_data,        // One 16 bit ADC sample
  input  wire [15:0] TSS_out,         // Test Sample Sequence from jesd204b_TSS
  input  wire [15:0] RPAT_out,        // RPAT pattern from jesd204b_RPAT
  input  wire [15:0] LAS_out,         // Lane Alignment Sequence from jesd204b_LAS
  input  wire        CLK_2char,       // 2 octet clock. Derived from PLL /5 clock.  Duty cycle is 3/5 (periodic)
  input  wire        CLK_frm,         // Frame clock (periodic)
  input  wire        MF_clk,          // Multi-frame clock pulse, Period = CLK_frm/K, PW = 1/CLK_frm (periodic)
  input  wire        MF_end,          // Clock pulse at end of MF, PW = 1/CLK_frm (periodic)
  input  wire        MF_chg,          // A MF change has been made (dynamic)
  input  wire        MF_chg_frm,      // A MF change has been made, pulse width is 1 frame (dynamic)
  input  wire        LAS_done,        // Lane Alignment Sequence is done
  input  wire        FAM_dis,         // Frame Alignment Monitoring enabled (static)
  input  wire        LAM_dis,         // Lane Alignment Monitoring enabled (static)
  input  wire        LAS_dis,         // Lane Alignment Sequence enabled (static)
  input  wire        ser_en,          // Serializer enable (static)
  input  wire        SYNC_det,        // SYNC has been detected with FRM_CLK continuously for >=4 frame clock cycles (dynamic)
  input  wire        TX_SYNC_en,      // transmitter enduced syncronization enabled (static)
  input  wire [2:0]  PAT,             // Test Pattern control (static)
  input  wire        SCR,             // Enable Data scrambling (static)
  input  wire        two_adc_per_ln,  // (static)
  input  wire [1:0]  SUBCLASSV        // (static)
  );

  reg         [15:0] ADC_data_rt;     // re-timed ADC data
  reg         [7:0]  qOCTET1;
  reg         [7:0]  qOCTET2;
  reg         [7:0]  nextOCTET1;
  reg         [7:0]  nextOCTET2;
  reg         [7:0]  OCTET2_tm1;
  reg         [15:0] nextSDATA_in;
  reg         [15:0] LAS_out_tm1;      
  reg                nextSEND_F; 
  reg                nextSEND_A; 
  reg                SEND_F_tm1;
  reg                SEND_F_tm2;
  reg                SEND_A_tm1;
  reg                SEND_A_tm2;
  reg                LAS_pat_en_tm1;
  reg                LAS_pat_en_tm2;
  reg                qK28_5_pat_en;     
  reg                qK28_7_pat_en;     
  reg                nextK28_5_pat_en;
  reg                nextK28_7_pat_en;
  reg                qSEND_K;
  reg                nextSEND_K;
  wire               force_SYNC_1MF;  // Flag to force one MF of SYNC commas for LAS_pat_en
  wire               start_SYNC_1MF;  // starts the force_SYNC_1MF if stuck at "_NORMAL" state
  wire               SUB0;
  wire               SYNC_request;
  wire               skip_LAS;
  wire               SCRAMBLE;
  wire               SEND_F_SCR;
  wire               SEND_A_SCR;
  wire               SEND_F_NSCR_2AL;
  wire               SEND_A_NSCR_2AL;
  wire               SEND_F_NSCR;
  wire               SEND_A_NSCR;
  wire               FRM_strt;

  parameter          _NORMAL         = 2'h0;
  parameter          _FRAME_SYNC     = 2'h1;
  parameter          _LANE_ALIGN_SEQ = 2'h2;

  reg         [1:0]  STATE;
  reg         [1:0]  nextSTATE;

  /* When there are 2 ADCs per lane, there are 4 octets per frame
  so OCTET2 does not always contain the last octet of the frame.
  In this case, CLK_2char is divided by 2 to make CLK_frm 50% duty
  cycle.  A low level on CLK_frm indicates the last half of the
  frame. */


  assign  FRM_strt = !(!CLK_frm && two_adc_per_ln);   // Marks the first two Octets of a frame
  assign  FRM_end = !(CLK_frm && two_adc_per_ln);     // Marks the last two Octets of a frame
  assign  MF_strt = (MF_clk || MF_chg_frm) && FRM_strt;
  assign  force_SYNC_1MF = (LAS_pat_en && !LAS_pat_en_tm1) || (LAS_pat_en_tm1 && !LAS_pat_en_tm2);  // Send K28.5 for  at least 1 MF
  assign  start_SYNC_1MF = (STATE == _NORMAL) && (nextSTATE == _NORMAL) && LAS_pat_en && LAS_pat_en_tm1 && LAS_pat_en_tm2;
  assign  SUB0 = !SUBCLASSV[0] && !SUBCLASSV[1];
  assign  SYNC_request = SYNC_det || (MF_chg && (TX_SYNC_en || SEND_LAS) && !SUB0) || (force_SYNC_1MF && LAS_pat_en);
  assign  skip_LAS = LAS_dis && !LAS_pat_en;
  assign  SCRAMBLE = SCR && ser_en;

  // used to setup force_SYNC_1MF
  always @(posedge MF_clk or posedge SYNC_det)
  begin
    if (SYNC_det)
    begin
      LAS_pat_en_tm1 <= 1'b0;
      LAS_pat_en_tm2 <= 1'b0;
    end
    else
    if (start_SYNC_1MF)
    begin
      LAS_pat_en_tm1 <= 1'b0;
      LAS_pat_en_tm2 <= 1'b0;
    end
    else
    begin
      LAS_pat_en_tm1 <= LAS_pat_en;
      LAS_pat_en_tm2 <= LAS_pat_en_tm1;
    end
  end // always

  always @( posedge CLK_2char or negedge ser_en )
  begin
    if (!ser_en)
    begin 
        ADC_data_rt      <= 16'h0000;
        nextOCTET1       <= 8'h00;
        nextOCTET2       <= 8'h00;
      OCTET1           <= 8'h00;
      OCTET2           <= 8'h00;
      OCTET2_tm1       <= 8'h00;
      SDATA_in         <= 16'h0000;
      LAS_out_tm1      <= 16'h0000;
      SEND_F           <= 1'b0;
      SEND_F_tm1       <= 1'b0;
      SEND_F_tm2       <= 1'b0;
      SEND_A           <= 1'b0;
      SEND_A_tm1       <= 1'b0;
      SEND_A_tm2       <= 1'b0;
      STATE            <= _NORMAL;
      SEND_LAS         <= 1'b0;
      nextSEND_K       <= 1'b0;
      SEND_K           <= 1'b0;
      nextK28_5_pat_en <= 1'b0;
      nextK28_7_pat_en <= 1'b0;
      K28_5_pat_en     <= 1'b0;
      K28_7_pat_en     <= 1'b0;        
    end //if
    else
    begin 
        ADC_data_rt      <= ADC_data;
        nextOCTET1       <= qOCTET1;
        nextOCTET2       <= qOCTET2;
      OCTET1           <= nextOCTET1;
      OCTET2           <= nextOCTET2;
      OCTET2_tm1       <= OCTET2;
      SDATA_in         <= nextSDATA_in;
      LAS_out_tm1      <= LAS_out;
      SEND_F           <= nextSEND_F; 
      SEND_F_tm1       <= SEND_F;
      SEND_F_tm2       <= SEND_F_tm1;
      SEND_A           <= nextSEND_A; 
      SEND_A_tm1       <= SEND_A;
      SEND_A_tm2       <= SEND_A_tm1;
      STATE            <= nextSTATE;
      SEND_LAS         <= nextSEND_LAS;
      nextSEND_K       <= qSEND_K;
      SEND_K           <= nextSEND_K;
      nextK28_5_pat_en <= qK28_5_pat_en;
      nextK28_7_pat_en <= qK28_7_pat_en;      
      K28_5_pat_en     <= nextK28_5_pat_en;
      K28_7_pat_en     <= nextK28_7_pat_en;
    end //else
  end //always


  /* Character replacement for Frame Alignment Monitoring
  and Lane Alignment Monitoring.*/

  assign SEND_F_SCR = !FAM_dis && FRM_strt && (qOCTET2 == 8'hFC);
  assign SEND_A_SCR = !LAM_dis && MF_strt && (qOCTET2 == 8'h7C);
  assign SEND_F_NSCR_2AL = !FAM_dis && FRM_strt &&(qOCTET2 == OCTET2) && !(SEND_F_tm2 || SEND_A_tm2);
  assign SEND_A_NSCR_2AL = !LAM_dis && MF_strt && (qOCTET2 == OCTET2);
  assign SEND_F_NSCR = !FAM_dis && (qOCTET2 == nextOCTET2) && !(SEND_F || SEND_A);
  assign SEND_A_NSCR = !LAM_dis && MF_strt && (qOCTET2 == nextOCTET2);

  always @ (SEND_F_SCR, SEND_A_SCR, SEND_F_NSCR_2AL, SEND_A_NSCR_2AL, SEND_F_NSCR, SEND_A_NSCR, STATE, PAT, two_adc_per_ln, SCR_en)
  begin
    if ((STATE == _NORMAL) && ((PAT == 3'h0) || (PAT == 3'h4)))
    begin
      if (SCR_en)
      begin
        nextSEND_F = SEND_F_SCR;
        nextSEND_A = SEND_A_SCR;
      end
      else
      begin
        if (two_adc_per_ln)
        begin
          nextSEND_F = SEND_F_NSCR_2AL;
          nextSEND_A = SEND_A_NSCR_2AL;
        end
        else
        begin
          nextSEND_F = SEND_F_NSCR;
          nextSEND_A = SEND_A_NSCR;
        end
      end
    end
    else
    begin
      nextSEND_F = 1'b0;
      nextSEND_A = 1'b0;
    end
  end //always

  // First Data MUX selection
  always @(SCRAMBLE, ser_en, RPAT_out, TSS_out, PAT, ADC_data_rt, nextSTATE, STATE)
  begin
    case (PAT[2:0])
      3'h0: //Normal ADC or Scrambled ADC Data
      begin
        nextSDATA_in[15:0] = ADC_data_rt[15:0];
        SCR_en = SCRAMBLE && ser_en;
        qK28_5_pat_en = 1'b0;
        qK28_7_pat_en = 1'b0;
        LAS_pat_en = 1'b0;
        TSS_pat_en = 1'b0;
        RPAT_pat_en =1'b0;
      end
      3'h1: //K28.5 test pattern
      begin
        nextSDATA_in[15:0] = 16'h0000;
        SCR_en = 1'b0;
        qK28_5_pat_en = (nextSTATE == _NORMAL);
        qK28_7_pat_en = 1'b0;
        LAS_pat_en = 1'b0;
        TSS_pat_en = 1'b0;
        RPAT_pat_en =1'b0;
      end
      3'h2:  //K28.7 test pattern
      begin
        nextSDATA_in[15:0] = 16'h0000;
        SCR_en = 1'b0;
        qK28_5_pat_en = 1'b0;
        qK28_7_pat_en = (nextSTATE == _NORMAL);
        LAS_pat_en = 1'b0;
        TSS_pat_en = 1'b0;
        RPAT_pat_en =1'b0;
      end
      3'h3:  //D21.5 test pattern
      begin
        nextSDATA_in[15:0] = 16'hB5B5;
        SCR_en = 1'b0;
        qK28_5_pat_en = 1'b0;
        qK28_7_pat_en = 1'b0;
        LAS_pat_en = 1'b0;
        TSS_pat_en = 1'b0;
        RPAT_pat_en =1'b0;
      end
      3'h4:  //PRBS15 pattern
      begin
        nextSDATA_in[15:0] = 16'h0000;
        SCR_en = (STATE == _NORMAL);
        qK28_5_pat_en = 1'b0;
        qK28_7_pat_en = 1'b0;
        LAS_pat_en = 1'b0;
        TSS_pat_en = 1'b0;
        RPAT_pat_en =1'b0;
      end
      3'h5:  //Lane Alignment Sequence
      begin
        nextSDATA_in[15:0] = 16'h0000;  // Optionally set this to the last two Octets in LAS
        SCR_en = 1'b0;              // Optionally set this to SCR
        qK28_5_pat_en = 1'b0;
        qK28_7_pat_en = 1'b0;
        LAS_pat_en = 1'b1;              // passes control to the Transmit State Machine
        TSS_pat_en = 1'b0;
        RPAT_pat_en =1'b0;
      end
      3'h6:  //Test Samples Sequence
      begin
        nextSDATA_in[15:0] = TSS_out[15:0];
        SCR_en = SCRAMBLE && ser_en;
        qK28_5_pat_en = 1'b0;
        qK28_7_pat_en = 1'b0;
        LAS_pat_en = 1'b0;
        TSS_pat_en = (nextSTATE == _NORMAL); // if TSS enabled, wait until in the "NORMAL" state to send TSS
        RPAT_pat_en =1'b0;
      end
      3'h7:  //Modified RPAT pattern
      begin
        nextSDATA_in[15:0] = RPAT_out[15:0];
        SCR_en = 1'b0;
        qK28_5_pat_en = 1'b0;
        qK28_7_pat_en = 1'b0;
        LAS_pat_en = 1'b0;
        TSS_pat_en = 1'b0;
        RPAT_pat_en = (nextSTATE == _NORMAL); // if RPAT enabled, wait until in the "NORMAL" state to send RPAT
      end
    endcase
  end //always

  // ************************** Transmit State Machine ******************************
  // State machine for frame synchronization and lane alignment sequence transmission
  // Also, Second Data MUX selection
  always @(STATE, SYNC_request, SUB0, FRM_strt, MF_strt, skip_LAS, SDATA_out, LAS_pat_en, LAS_done, SUBCLASSV, SDATA_out, LAS_out_tm1)
  begin
    case (STATE)
      _NORMAL:
      begin
        qOCTET1 = SDATA_out[15:8];
        qOCTET2 = SDATA_out[7:0];
        if (SYNC_request)
        begin
          nextSTATE = _FRAME_SYNC;
          nextSEND_LAS = 1'b0;
          qSEND_K = 1'b1;
        end
        else
        begin
          nextSTATE = _NORMAL;
          nextSEND_LAS = 1'b0;
          qSEND_K = 1'b0;
        end
      end
      _FRAME_SYNC:
      begin
        qOCTET1 = SDATA_out[15:8];
        qOCTET2 = SDATA_out[7:0];
        if (!SYNC_request && (SUB0 && FRM_strt || !SUB0 && MF_strt) && !skip_LAS)
        begin
          nextSTATE = _LANE_ALIGN_SEQ;
          nextSEND_LAS = 1'b1;
          qSEND_K = 1'b0;
        end
        else if (!SYNC_request && (SUB0 && FRM_strt || !SUB0 && MF_strt) && skip_LAS)
        begin
          nextSTATE = _NORMAL;
          nextSEND_LAS = 1'b0;
          qSEND_K = 1'b0;
        end
        else
        begin
          nextSTATE = _FRAME_SYNC;
          nextSEND_LAS = 1'b0;
          qSEND_K = 1'b1;
        end
      end
      _LANE_ALIGN_SEQ:
      begin
        qOCTET1 = LAS_out_tm1[15:8];
        qOCTET2 = LAS_out_tm1[7:0];
        if (LAS_done && !SYNC_request) //if (LAS_done && !LAS_pat_en && !SYNC_request)
        begin
          nextSTATE = _NORMAL;
         nextSEND_LAS = 1'b0;
         qSEND_K = 1'b0;
        end
        else if (SYNC_request)
        begin
          nextSTATE = _FRAME_SYNC;
          nextSEND_LAS = 1'b0;
          qSEND_K = 1'b1;
        end
        else
        begin
          nextSTATE = _LANE_ALIGN_SEQ;
          nextSEND_LAS = 1'b1;
          qSEND_K = 1'b0;
        end
      end
      default:
      begin
        qOCTET1 = SDATA_out[15:8];
        qOCTET2 = SDATA_out[7:0];
        if (SYNC_request)
        begin
          nextSTATE = _FRAME_SYNC;
          nextSEND_LAS = 1'b0;
          qSEND_K = 1'b1;
        end
        else
        begin
          nextSTATE = _NORMAL;
          nextSEND_LAS = 1'b0;
          qSEND_K = 1'b0;
        end
      end
    endcase
  end // always



  // synopsys_translate off
  reg [119:0] ascii_state;
  always @(STATE)
  begin
    case(STATE)
      _NORMAL: ascii_state = "NORM";
      _FRAME_SYNC: ascii_state = "FSYNC";
      _LANE_ALIGN_SEQ: ascii_state = "LAS";
      default: ascii_state = "ERROR";
    endcase
  end //always

  initial
  begin
    ascii_state = "NORM";
  end
  // synopsys_translate on

endmodule

`timescale 1ns / 1 ps

module jesd204b_K28_gen (
  output  reg        K28_en1,      // Substitute Data with a K28 CODE1
  output  reg        K28_en2,      // Substitute Data with a K28 CODE2
  output  reg  [2:0] K28_index1,   // Index to the K28 CODE1
  output  reg  [2:0] K28_index2,   // Index to the K28 CODE2
  input  wire        SEND_K,       // send K28.5
  input  wire        SEND_F ,      //
  input  wire        SEND_A,       //
  input  wire        nextSEND_LAS, //
  input  wire        SEND_LAS,     //
  input  wire        ser_en,       //
  input  wire        CLK_2char,    //
  input  wire        MF_strt,      //
  input  wire        LAS_MF2,      // 1 for second multiframe of LAS
  input  wire        LAS_done,     //
  input  wire        K28_5_pat_en, //
  input  wire        K28_7_pat_en  //
  );

  reg                nextK28_en1;
  reg                nextK28_en2;
  reg         [2:0]  nextK28_index1;
  reg         [2:0]  nextK28_index2;
  reg                MF_strt_tm1;
  reg                MF_strt_tm2;
  reg                SEND_LAS_tm1;

  always @(posedge CLK_2char or negedge ser_en)
  begin
    if (!ser_en)
    begin
      K28_index1   <= 3'b000;
      K28_index2   <= 3'b000;
      K28_en1      <= 1'b0;
      K28_en2      <= 1'b0;
      MF_strt_tm1  <= 1'b0;
      MF_strt_tm2  <= 1'b0;
      SEND_LAS_tm1 <= 1'b0;
    end
    else
    begin
      K28_index1   <= nextK28_index1;
      K28_index2   <= nextK28_index2;
      K28_en1      <= nextK28_en1;
      K28_en2      <= nextK28_en2;
      MF_strt_tm1  <= MF_strt; 
      MF_strt_tm2  <= MF_strt_tm1;
      SEND_LAS_tm1 <= SEND_LAS;
    end
  end

  always @(SEND_K, K28_5_pat_en, SEND_A, SEND_LAS, SEND_LAS_tm1, SEND_F, K28_7_pat_en, MF_strt_tm1, MF_strt_tm2, LAS_done, LAS_MF2)
  begin
    if (SEND_K || K28_5_pat_en)  // K28.5
    begin
      nextK28_index1 = 3'h5;
      nextK28_index2 = 3'h5;
      nextK28_en1    = 1'b1;
      nextK28_en2    = 1'b1;
    end
    else if (SEND_A || (SEND_LAS_tm1 && MF_strt_tm1))   // K28.3, Lane alignment symbol
    begin
      nextK28_index1 = 3'h0;
      nextK28_index2 = 3'h3;
      nextK28_en1    = 1'b0;
      nextK28_en2    = 1'b1;
    end
    else if (SEND_F || K28_7_pat_en)  //K28.7, Frame alignment symbol
    begin
      nextK28_index1 = 3'h7;
      nextK28_index2 = 3'h7;
      nextK28_en1 = K28_7_pat_en ? 1'b1:1'b0;
      nextK28_en2    = 1'b1;
    end
    else if (SEND_LAS && MF_strt_tm2 && !LAS_done)  // K28.0, K28.4
    begin
      nextK28_index1 = 3'h0;
      nextK28_index2 = 3'h4;
      nextK28_en1    = 1'b1;
      nextK28_en2 = LAS_MF2 ? 1'b1:1'b0;
    end
    else
    begin
      nextK28_index1 = 3'h0;
      nextK28_index2 = 3'h0;
      nextK28_en1    = 1'b0;
      nextK28_en2    = 1'b0;
    end
  end //always
endmodule

`timescale 1ns / 1 ps

module jesd204b_scrambler
  (
   output wire [15:0]    SDATA_out,       // scrambler data out
   input  wire           CLK_2char,
   input  wire [15:0]    SDATA_in,        // scrambler data in
   input  wire           SCR_en,
   input  wire           scan_mode
   );

   reg  [15:0]     SDR;                   // Registered Scrambled Data
   reg  [15:0]     SD;                    // Scrambled Data
   //wire [15:0]     SD_in;


   //assign SD_in = SDATA_in[15:0] & {16{SCR_en}};

   always @ (posedge CLK_2char or negedge SCR_en)
   begin
          if (!SCR_en)
          begin
        if (!scan_mode) SDR[15:0] <= 16'h00FF;
      end
      else SDR[15:0] <= SD[15:0];
   end

   always @ (SDATA_in, SDR, SD[15:14])
   begin
      SD[0]  <= SDATA_in[0]  ^ SD[14]  ^ SD[15];
      SD[1]  <= SDATA_in[1]  ^ SD[15]  ^ SDR[0];
      SD[2]  <= SDATA_in[2]  ^ SDR[0]  ^ SDR[1];
      SD[3]  <= SDATA_in[3]  ^ SDR[1]  ^ SDR[2];
      SD[4]  <= SDATA_in[4]  ^ SDR[2]  ^ SDR[3];
      SD[5]  <= SDATA_in[5]  ^ SDR[3]  ^ SDR[4];
      SD[6]  <= SDATA_in[6]  ^ SDR[4]  ^ SDR[5];
      SD[7]  <= SDATA_in[7]  ^ SDR[5]  ^ SDR[6];
      SD[8]  <= SDATA_in[8]  ^ SDR[6]  ^ SDR[7];
      SD[9]  <= SDATA_in[9]  ^ SDR[7]  ^ SDR[8];
      SD[10] <= SDATA_in[10] ^ SDR[8]  ^ SDR[9];
      SD[11] <= SDATA_in[11] ^ SDR[9]  ^ SDR[10];
      SD[12] <= SDATA_in[12] ^ SDR[10] ^ SDR[11];
      SD[13] <= SDATA_in[13] ^ SDR[11] ^ SDR[12];
      SD[14] <= SDATA_in[14] ^ SDR[12] ^ SDR[13];
      SD[15] <= SDATA_in[15] ^ SDR[13] ^ SDR[14];
   end

   assign SDATA_out[15:0] = SCR_en ? SD[15:0] : SDATA_in[15:0];

endmodule

 `timescale 1ns / 1 ps

  //****************************************************************************
  module encode_8b10b
  (
    output reg  [9:0] CODE1,      // most significant (first) 10bit code group output
    output reg  [9:0] CODE2,      // least significant (second) 10bit code group output
    output wire       disparity,
    input  wire [7:0] OCTET1,     // most significant (first) OCTET input from ADC
    input  wire [7:0] OCTET2,     // least significant (second) OCTET input from ADC
    input  wire       CLK_2char,  // 2 character (octet) clock
    input  wire       K28_en1,    // Substitute Data with a K28 CODE1
    input  wire       K28_en2,    // Substitute Data with a K28 CODE2
    input  wire [2:0] K28_index1, // Index to the K28 CODE1
    input  wire [2:0] K28_index2, // Index to the K28 CODE2
    input  wire       ser_en
   );

    wire        [5:0] nextO5b6b1n; //next 5b6b portion of code group 1 for current -disp
    wire        [5:0] nextO5b6b1p; //next 5b6b portion of code group 1 for current +disp
    wire        [5:0] nextO5b6b2n; //next 5b6b portion of code group 2 for current -disp
    wire        [5:0] nextO5b6b2p; //next 5b6b portion of code group 2 for current +disp
    wire        [3:0] nextO3b4b1n; //next 3b4b portion of code group 1 for current -disp
    wire        [3:0] nextO3b4b1p; //next 3b4b portion of code group 1 for current +disp
    wire        [3:0] nextO3b4b2n; //next 3b4b portion of code group 2 for current -disp
    wire        [3:0] nextO3b4b2p; //next 3b4b portion of code group 2 for current +disp
    reg         [5:0] O5b6b1n;     //5b6b portion of code group 1 for current -disp
    reg         [5:0] O5b6b1p;     //5b6b portion of code group 1 for current +disp
    reg         [5:0] O5b6b2n;     //5b6b portion of code group 2 for current -disp
    reg         [5:0] O5b6b2p;     //5b6b portion of code group 2 for current +disp
    reg         [3:0] O3b4b1n;     //3b4b portion of code group 1 for current -disp
    reg         [3:0] O3b4b1p;     //3b4b portion of code group 1 for current +disp
    reg         [3:0] O3b4b2n;     //3b4b portion of code group 2 for current -disp
    reg         [3:0] O3b4b2p;     //3b4b portion of code group 2 for current +disp
    reg         [9:0] nextCODE1;
    reg         [9:0] nextCODE2;
    wire              nextRD5b6b1n;  //next RD5b6b1n
    wire              nextRD5b6b1p;  //next RD5b6b1p
    wire              nextRD5b6b2n;  //next RD5b6b2n
    wire              nextRD5b6b2p;  //next RD5b6b2p
    wire              nextRD3b4b1n;  //next RD3b4b1n
    wire              nextRD3b4b1p;  //next RD3b4b1p
    wire              nextRD3b4b2n;  //next RD3b4b2n
    wire              nextRD3b4b2p;  //next RD3b4b2p
    reg               RD5b6b1n;      //Running Disp for 5b6b portion of of code group 1 for current -disp
    reg               RD5b6b1p;      //Running Disp for 5b6b portion of of code group 1 for current +disp
    reg               RD5b6b2n;      //Running Disp for 5b6b portion of of code group 2 for current -disp
    reg               RD5b6b2p;      //Running Disp for 5b6b portion of of code group 2 for current +disp
    reg               RD3b4b1n;      //Running Disp for 3b4b portion of of code group 1 for current -disp
    reg               RD3b4b1p;      //Running Disp for 3b4b portion of of code group 1 for current +disp
    reg               RD3b4b2n;      //Running Disp for 3b4b portion of of code group 2 for current -disp
    reg               RD3b4b2p;      //Running Disp for 3b4b portion of of code group 2 for current +disp
    reg               nextRD5b6b1;
    reg               RD5b6b1;       //Running Disp for 5b6b portion of of code group 1
    reg               RD3b4b1;       //Running Disp for 3b4b portion of of code group 1
    reg               RD5b6b2;       //Running Disp for 5b6b portion of of code group 2
    reg               RD3b4b2;       //Running Disp for 3b4b portion of of code group 2
    
    assign disparity = RD5b6b1;

    // set defaults to be an input OCTET of 16'h0000
    always @ ( posedge CLK_2char or negedge ser_en )
    begin
      if (!ser_en)
      begin
        RD5b6b1   <= 0;
        O5b6b1n   <= 6'h27;
        O5b6b1p   <= 6'h18;
        O5b6b2n   <= 6'h27;
        O5b6b2p   <= 6'h18;
        O3b4b1n   <= 4'h4;
        O3b4b1p   <= 4'h4;
        O3b4b2n   <= 4'hb;
        O3b4b2p   <= 4'hb;
        CODE1     <= 10'h274;     
        CODE2     <= 10'h274;
        RD5b6b1n  <= 1;
        RD5b6b1p  <= 0;
        RD5b6b2n  <= 1;
        RD5b6b2p  <= 0;
        RD3b4b1n  <= 0;
        RD3b4b1p  <= 1;
        RD3b4b2n  <= 0;
        RD3b4b2p  <= 1;
      end //if
      else
      begin
        RD5b6b1   <= nextRD5b6b1;
        O5b6b1n   <= nextO5b6b1n;
        O5b6b1p   <= nextO5b6b1p;
        O5b6b2n   <= nextO5b6b2n;
        O5b6b2p   <= nextO5b6b2p;
        O3b4b1n   <= nextO3b4b1n;
        O3b4b1p   <= nextO3b4b1p;
        O3b4b2n   <= nextO3b4b2n;
        O3b4b2p   <= nextO3b4b2p;
        CODE1     <= nextCODE1;
        CODE2     <= nextCODE2;
        RD5b6b1n  <= nextRD5b6b1n;
        RD5b6b1p  <= nextRD5b6b1p;
        RD5b6b2n  <= nextRD5b6b2n;
        RD5b6b2p  <= nextRD5b6b2p;
        RD3b4b1n  <= nextRD3b4b1n;
        RD3b4b1p  <= nextRD3b4b1p;
        RD3b4b2n  <= nextRD3b4b2n;
        RD3b4b2p  <= nextRD3b4b2p;
      end //else
    end //always

    always @ ( O5b6b1n, O5b6b1p, O5b6b2n, O5b6b2p, O3b4b1n, O3b4b1p, O3b4b2n, O3b4b2p,
               RD5b6b1n,  RD5b6b1p,  RD5b6b2n,  RD5b6b2p,  RD3b4b1n,  RD3b4b1p,  RD3b4b2n,  RD3b4b2p,
               RD5b6b1, RD3b4b1, RD5b6b2, RD3b4b2, nextRD5b6b1)
    begin
      if (RD5b6b1)  begin nextCODE1[9:4] = O5b6b1p;       RD3b4b1 = RD5b6b1p; end
      else          begin nextCODE1[9:4] = O5b6b1n;       RD3b4b1 = RD5b6b1n; end

      if (RD3b4b1)  begin nextCODE1[3:0] = O3b4b1p;       RD5b6b2 = RD3b4b1p; end
      else          begin nextCODE1[3:0] = O3b4b1n;       RD5b6b2 = RD3b4b1n; end

      if (RD5b6b2)  begin nextCODE2[9:4] = O5b6b2p;       RD3b4b2 = RD5b6b2p; end
      else          begin nextCODE2[9:4] = O5b6b2n;       RD3b4b2 = RD5b6b2n; end

      if (RD3b4b2)  begin nextCODE2[3:0] = O3b4b2p;   nextRD5b6b1 = RD3b4b2p; end
      else          begin nextCODE2[3:0] = O3b4b2n;   nextRD5b6b1 = RD3b4b2n; end
    end //always

    enc_5b6b_3b4b enc_5b6b_3b4b_1st
    (
      .nextO5b6bn(nextO5b6b1n),
      .nextO5b6bp(nextO5b6b1p),
      .nextRD5b6bn(nextRD5b6b1n),
      .nextRD5b6bp(nextRD5b6b1p),
      .nextO3b4bn(nextO3b4b1n),
      .nextO3b4bp(nextO3b4b1p),
      .nextRD3b4bn(nextRD3b4b1n),
      .nextRD3b4bp(nextRD3b4b1p),
      .OCTET(OCTET1),
      .K28_en(K28_en1),
      .K28_index(K28_index1)
    );

    enc_5b6b_3b4b enc_5b6b_3b4b_2nd
    (
      .nextO5b6bn(nextO5b6b2n),
      .nextO5b6bp(nextO5b6b2p),
      .nextRD5b6bn(nextRD5b6b2n),
      .nextRD5b6bp(nextRD5b6b2p),
      .nextO3b4bn(nextO3b4b2n),
      .nextO3b4bp(nextO3b4b2p),
      .nextRD3b4bn(nextRD3b4b2n),
      .nextRD3b4bp(nextRD3b4b2p),
      .OCTET(OCTET2),
      .K28_en(K28_en2),
      .K28_index(K28_index2)
    );
  endmodule

`timescale 1ns / 1 ps

module enc_5b6b_3b4b (
    output reg  [5:0] nextO5b6bn,  // next 5b6b output for current RD-
    output reg  [5:0] nextO5b6bp,  // next 5b6b output for current RD+
    output reg        nextRD5b6bn, // next 5b6b RD for current RD-
    output reg        nextRD5b6bp, // next 5b6b RD for current RD+
    output reg  [3:0] nextO3b4bn,  // next 3b4b output for current RD-
    output reg  [3:0] nextO3b4bp,  // next 3b4b output for current RD+
    output reg        nextRD3b4bn, // next 3b4b RD for current RD-
    output reg        nextRD3b4bp, // next 3b4b RD for current RD+
    input  wire [7:0] OCTET,       // OCTET input from ADC, LAS, or PAT
    input  wire       K28_en,      // Substitute Data with a K28 code
    input  wire [2:0] K28_index    // Index to the K28 code
    );

 always @ ( OCTET, K28_en, K28_index[2:0], nextO5b6bn[1], nextO5b6bn[0], nextRD5b6bn, nextO5b6bp[1], nextO5b6bp[0], nextRD5b6bp)
 begin
   if (K28_en) begin nextO5b6bn = 6'b001111; nextRD5b6bn = 1; nextO5b6bp = 6'b110000; nextRD5b6bp = 0; end
   else
   case ( OCTET[4:0] )
     5'h00: begin nextO5b6bn = 6'b100111; nextRD5b6bn = 1; nextO5b6bp = 6'b011000; nextRD5b6bp = 0; end
     5'h01: begin nextO5b6bn = 6'b011101; nextRD5b6bn = 1; nextO5b6bp = 6'b100010; nextRD5b6bp = 0; end
     5'h02: begin nextO5b6bn = 6'b101101; nextRD5b6bn = 1; nextO5b6bp = 6'b010010; nextRD5b6bp = 0; end
     5'h03: begin nextO5b6bn = 6'b110001; nextRD5b6bn = 0; nextO5b6bp = 6'b110001; nextRD5b6bp = 1; end
     5'h04: begin nextO5b6bn = 6'b110101; nextRD5b6bn = 1; nextO5b6bp = 6'b001010; nextRD5b6bp = 0; end
     5'h05: begin nextO5b6bn = 6'b101001; nextRD5b6bn = 0; nextO5b6bp = 6'b101001; nextRD5b6bp = 1; end
     5'h06: begin nextO5b6bn = 6'b011001; nextRD5b6bn = 0; nextO5b6bp = 6'b011001; nextRD5b6bp = 1; end
     5'h07: begin nextO5b6bn = 6'b111000; nextRD5b6bn = 0; nextO5b6bp = 6'b000111; nextRD5b6bp = 1; end
     5'h08: begin nextO5b6bn = 6'b111001; nextRD5b6bn = 1; nextO5b6bp = 6'b000110; nextRD5b6bp = 0; end
     5'h09: begin nextO5b6bn = 6'b100101; nextRD5b6bn = 0; nextO5b6bp = 6'b100101; nextRD5b6bp = 1; end
     5'h0A: begin nextO5b6bn = 6'b010101; nextRD5b6bn = 0; nextO5b6bp = 6'b010101; nextRD5b6bp = 1; end
     5'h0B: begin nextO5b6bn = 6'b110100; nextRD5b6bn = 0; nextO5b6bp = 6'b110100; nextRD5b6bp = 1; end
     5'h0C: begin nextO5b6bn = 6'b001101; nextRD5b6bn = 0; nextO5b6bp = 6'b001101; nextRD5b6bp = 1; end
     5'h0D: begin nextO5b6bn = 6'b101100; nextRD5b6bn = 0; nextO5b6bp = 6'b101100; nextRD5b6bp = 1; end
     5'h0E: begin nextO5b6bn = 6'b011100; nextRD5b6bn = 0; nextO5b6bp = 6'b011100; nextRD5b6bp = 1; end
     5'h0F: begin nextO5b6bn = 6'b010111; nextRD5b6bn = 1; nextO5b6bp = 6'b101000; nextRD5b6bp = 0; end
     5'h10: begin nextO5b6bn = 6'b011011; nextRD5b6bn = 1; nextO5b6bp = 6'b100100; nextRD5b6bp = 0; end
     5'h11: begin nextO5b6bn = 6'b100011; nextRD5b6bn = 0; nextO5b6bp = 6'b100011; nextRD5b6bp = 1; end
     5'h12: begin nextO5b6bn = 6'b010011; nextRD5b6bn = 0; nextO5b6bp = 6'b010011; nextRD5b6bp = 1; end
     5'h13: begin nextO5b6bn = 6'b110010; nextRD5b6bn = 0; nextO5b6bp = 6'b110010; nextRD5b6bp = 1; end
     5'h14: begin nextO5b6bn = 6'b001011; nextRD5b6bn = 0; nextO5b6bp = 6'b001011; nextRD5b6bp = 1; end
     5'h15: begin nextO5b6bn = 6'b101010; nextRD5b6bn = 0; nextO5b6bp = 6'b101010; nextRD5b6bp = 1; end
     5'h16: begin nextO5b6bn = 6'b011010; nextRD5b6bn = 0; nextO5b6bp = 6'b011010; nextRD5b6bp = 1; end
     5'h17: begin nextO5b6bn = 6'b111010; nextRD5b6bn = 1; nextO5b6bp = 6'b000101; nextRD5b6bp = 0; end
     5'h18: begin nextO5b6bn = 6'b110011; nextRD5b6bn = 1; nextO5b6bp = 6'b001100; nextRD5b6bp = 0; end
     5'h19: begin nextO5b6bn = 6'b100110; nextRD5b6bn = 0; nextO5b6bp = 6'b100110; nextRD5b6bp = 1; end
     5'h1A: begin nextO5b6bn = 6'b010110; nextRD5b6bn = 0; nextO5b6bp = 6'b010110; nextRD5b6bp = 1; end
     5'h1B: begin nextO5b6bn = 6'b110110; nextRD5b6bn = 1; nextO5b6bp = 6'b001001; nextRD5b6bp = 0; end
     5'h1C: begin nextO5b6bn = 6'b001110; nextRD5b6bn = 0; nextO5b6bp = 6'b001110; nextRD5b6bp = 1; end
     5'h1D: begin nextO5b6bn = 6'b101110; nextRD5b6bn = 1; nextO5b6bp = 6'b010001; nextRD5b6bp = 0; end
     5'h1E: begin nextO5b6bn = 6'b011110; nextRD5b6bn = 1; nextO5b6bp = 6'b100001; nextRD5b6bp = 0; end
     5'h1F: begin nextO5b6bn = 6'b101011; nextRD5b6bn = 1; nextO5b6bp = 6'b010100; nextRD5b6bp = 0; end
   endcase
   if (K28_en)
   case ( K28_index[2:0] )
     3'h0:  begin nextO3b4bn = 4'b1011;   nextRD3b4bn = 1; nextO3b4bp = 4'b0100;   nextRD3b4bp = 0; end
     3'h1:  begin nextO3b4bn = 4'b0110;   nextRD3b4bn = 0; nextO3b4bp = 4'b1001;   nextRD3b4bp = 1; end
     3'h2:  begin nextO3b4bn = 4'b1010;   nextRD3b4bn = 0; nextO3b4bp = 4'b0101;   nextRD3b4bp = 1; end
     3'h3:  begin nextO3b4bn = 4'b1100;   nextRD3b4bn = 0; nextO3b4bp = 4'b0011;   nextRD3b4bp = 1; end
     3'h4:  begin nextO3b4bn = 4'b1101;   nextRD3b4bn = 1; nextO3b4bp = 4'b0010;   nextRD3b4bp = 0; end
     3'h5:  begin nextO3b4bn = 4'b0101;   nextRD3b4bn = 0; nextO3b4bp = 4'b1010;   nextRD3b4bp = 1; end
     3'h6:  begin nextO3b4bn = 4'b1001;   nextRD3b4bn = 0; nextO3b4bp = 4'b0110;   nextRD3b4bp = 1; end
     3'h7:  begin nextO3b4bn = 4'b0111;   nextRD3b4bn = 1; nextO3b4bp = 4'b1000;   nextRD3b4bp = 0; end
   endcase
   else
   case ( OCTET[7:5] )
     3'h0:  begin nextO3b4bn = 4'b1011;   nextRD3b4bn = 1; nextO3b4bp = 4'b0100;   nextRD3b4bp = 0; end
     3'h1:  begin nextO3b4bn = 4'b1001;   nextRD3b4bn = 0; nextO3b4bp = 4'b1001;   nextRD3b4bp = 1; end
     3'h2:  begin nextO3b4bn = 4'b0101;   nextRD3b4bn = 0; nextO3b4bp = 4'b0101;   nextRD3b4bp = 1; end
     3'h3:  begin nextO3b4bn = 4'b1100;   nextRD3b4bn = 0; nextO3b4bp = 4'b0011;   nextRD3b4bp = 1; end
     3'h4:  begin nextO3b4bn = 4'b1101;   nextRD3b4bn = 1; nextO3b4bp = 4'b0010;   nextRD3b4bp = 0; end
     3'h5:  begin nextO3b4bn = 4'b1010;   nextRD3b4bn = 0; nextO3b4bp = 4'b1010;   nextRD3b4bp = 1; end
     3'h6:  begin nextO3b4bn = 4'b0110;   nextRD3b4bn = 0; nextO3b4bp = 4'b0110;   nextRD3b4bp = 1; end
     3'h7:  begin nextO3b4bn = 4'b1110;   nextRD3b4bn = 1; nextO3b4bp = 4'b0001;   nextRD3b4bp = 0;
             if (  nextO5b6bn[0] &&  nextO5b6bn[1] && ~nextRD5b6bn ) nextO3b4bn = 4'b0111;
             if ( ~nextO5b6bp[0] && ~nextO5b6bp[1] &&  nextRD5b6bp ) nextO3b4bp = 4'b1000;
            end
   endcase
 end //always
endmodule

`timescale 1ns / 1 ps

module jesd204b_RPAT                   
(                                   
  output reg  [15:0]    RPAT_out,      // RPAT data out
  input  wire           CLK_2char,     // Twice the period of the Character CLK (for 2 Octets)
  input  wire           RPAT_pat_en,
  input  wire           disparity      // Disparity polarity of next word
);

  reg   [3:0]  RPAT_ptr;

  always @(posedge CLK_2char) // or negedge RPAT_pat_en)
  begin
    if (!RPAT_pat_en)
    begin
      RPAT_ptr <= 4'h0;
    end
    else
    begin
      RPAT_ptr <= RPAT_ptr+1;
      if (RPAT_ptr >= 4'hA) RPAT_ptr <= 4'h5;
    end
  end //always

  always @(RPAT_ptr, disparity)
  begin
    case (RPAT_ptr)
        4'h0: RPAT_out = 16'hC5C5;   //keep disparity the same
      4'h1: RPAT_out = 16'hC5C5;   //keep disparity the same
      4'h2: RPAT_out = 16'hC5C5;   //keep disparity the same
      4'h3: RPAT_out = 16'hC5C5;   //keep disparity the same
      4'h4: if (!disparity)
            RPAT_out = 16'hC550;   //reverse disparity
            else
            RPAT_out = 16'hC5C5;   //keep disparity the same
      4'h5: RPAT_out = 16'hBED7;   //begin repeated RPAT sequence with positive disparity
      4'h6: RPAT_out = 16'h2347;
      4'h7: RPAT_out = 16'h6B8F;
      4'h8: RPAT_out = 16'hB314;
      4'h9: RPAT_out = 16'h5EFB;
      4'hA: RPAT_out = 16'h3559;   //end repeated RPAT sequence
      default: RPAT_out = 16'hC5C5;
    endcase
  end //always
endmodule



`timescale 1ns / 1 ps

module jesd204b_LAS                     // intended for a single lane
(                                       // for two lanes, two instances are required
  output reg [15:0]     LAS_out,        // LAS data out
  output reg            LAS_MF2,        // Second Multiframe in the Lane Alignment Sequence
  output reg            LAS_done,       // Lane alignment sequence is done
  input  wire           CLK_2char,      // Twice the period of the Character CLK (2 Octets)
  input  wire           LAS_pat_en,     // LAS pattern enable
  input  wire           nextSEND_LAS,   //
  input  wire           SEND_LAS,       // Send the Lane Alignment Sequence
  input  wire [7:0]     DID,            // Device ID
  input  wire [3:0]     BID,            // Bank ID
  input  wire [2:0]     LID,            // Lane ID
  input  wire           SCR,            // Scrambling Enabled
  input  wire [2:0]     L_,             // Lanes per Device
  input  wire [4:0]     K_,             // Frames per Multiframe
  input  wire [2:0]     M_,             // ADCs per Device
  input  wire [1:0]     CS,             // Control bits per Sample
  input  wire [3:0]     N_,             // Converter Resolution
  input  wire [1:0]     SUBCLASSV,      // JESD204B Subclass
  input  wire           FRM_end,       //
  input  wire           MF_end,         // Multi-Frame End
  input  wire           MF_clk,         // Multi-Frame Clock
  input  wire           two_adc_per_ln, //
  input  wire           two_ln_per_adc  //
);

  reg   [6:0] LAS_2char_cnt;
  reg   [6:0] MF2_2char_cnt;
  reg   [1:0] LAS_MF_cnt;       // Lane Alignment Sequence Multi-Frame Count
  wire  [1:0] F_;               //
  reg         LAS_MF4;          //
  wire        RST_LAS_cnt;
  wire  [7:0] FCHK;             // Sum of fields mod 256
  wire        S_;               // Samples per Converter per Frame cycle
  wire  [4:0] N_tot;            // N + CS
  wire  [2:0] JESDV;

  assign S_ = two_ln_per_adc;   // one sample/converter/ln unless in two ln/ADC mode
  assign JESDV = 3'b1;
  assign FCHK = DID + BID + LID + SCR + L_ + F_ + K_ + M_ + CS + N_ + SUBCLASSV + N_tot + JESDV + S_; //Sum of fields mod 256
  assign N_tot = 4'hF;
  assign RST_LAS_cnt = !nextSEND_LAS || (LAS_MF4 && MF_end && FRM_end);
  assign F_ = two_adc_per_ln ? 2'h3:2'h1;


  // LAS duration counter
  always @(posedge MF_clk)// or negedge SEND_LAS)  // The lane alignment sequence is 4 Multiframes unless the LAS pattern
  begin                                         // is enabled in the SPI.  If the LAS pattern is enabled, the LAS_done
    if (!nextSEND_LAS)                              // signal should not be asserted until the 4th MF after LAS_done de-assertion.
    begin
      LAS_MF_cnt <= 2'h0;
      LAS_done <= 1'b0;
    end
    else
    begin
      LAS_MF_cnt <= LAS_MF_cnt + 1'b1;
      LAS_done <= (LAS_MF4 && !LAS_pat_en);
    end
  end //always

 // 2nd and 4th MF detection
  always @ (LAS_MF_cnt)                         // The 2nd Multiframe is used to transmit lane configuration information.
  begin                                         // The 4th Multiframe marks the last Multiframe in the LAS.
    if (LAS_MF_cnt == 2'h1)
      LAS_MF2 = 1'b1;
    else
      LAS_MF2 = 1'b0;
    if (LAS_MF_cnt == 2'h3)
      LAS_MF4 = 1'b1;
    else
      LAS_MF4 = 1'b0;
  end

  // LAS CLK_2char counter
  always @(posedge CLK_2char)
  begin
    if (RST_LAS_cnt)                            // Reset the counter at the end of each LAS.
    begin                                       // This is needed for LAS_pat_en mode, where the
      LAS_2char_cnt <= 7'h00;                   // LAS is repeated in a loop.
    end
    else
    begin
      LAS_2char_cnt <= LAS_2char_cnt + 1'b1;
    end
  end //always

  // MF2 CLK_2char counter
  always @(posedge CLK_2char) // or negedge LAS_MF2)
  begin
    if (!LAS_MF2)
    begin
      MF2_2char_cnt <= 7'h00;
    end
    else
    begin
      MF2_2char_cnt <= MF2_2char_cnt + 1'b1;
    end
  end //always

  always @(nextSEND_LAS, LAS_2char_cnt, LAS_MF2, DID, BID, LID, L_, F_, K_, M_, CS, N_, SUBCLASSV, N_tot, JESDV, S_, FCHK, SCR, MF2_2char_cnt)
  begin
    if (nextSEND_LAS)
    begin
      LAS_out[15:8] = LAS_2char_cnt*2;
      LAS_out[7:0] = LAS_2char_cnt*2+1;
      if (LAS_MF2)
      begin
        case (MF2_2char_cnt)
          4'h1:                          // Transmit DID and BID
          begin
            LAS_out[15:8] = DID;
            LAS_out[7:0] = {4'h0, BID};
          end
          4'h2:                          // Transmit LID, SCR, and L_
          begin
            LAS_out[15:8] = {3'h0, 2'h0, LID};
            LAS_out[7:0] = {SCR, 2'h0, 2'h0, L_};
          end
          4'h3:                          // Transmit F_ and K_
          begin
            LAS_out[15:8] = {6'h00, F_};
            LAS_out[7:0] = {3'h0, K_};
          end
          4'h4:                          // Transmit M_, CS, and N_
          begin
            LAS_out[15:8] = {5'h00, M_};
            LAS_out[7:0] = {CS, 1'b0, 1'b0, N_};
          end
          4'h5:                          // Transmit SUBCLASSV, N', JESDV, and S_
          begin
            LAS_out[15:8] = {1'b0, SUBCLASSV, N_tot};
            LAS_out[7:0] = {JESDV, 4'h0, S_};
          end
          4'h6:                          // Transmit HD, CF, and RES1
          begin
            LAS_out[15:8] = {1'b0, 2'h0, 5'h00};
            LAS_out[7:0] = 8'h0;
          end
          4'h7:                          // Transmit RES2 and FCHK
          begin
            LAS_out[15:8] = 8'h0;
            LAS_out[7:0] = FCHK;
          end
          default:
          begin
            LAS_out[15:8] = LAS_2char_cnt*2;
            LAS_out[7:0] = LAS_2char_cnt*2+1;
          end
        endcase
      end  // if MF2
    end  // if nextSEND_LAS
    else
    begin
      LAS_out[15:8] = LAS_2char_cnt*2;
      LAS_out[7:0] = LAS_2char_cnt*2+1;
    end
  end  // always
endmodule

`timescale 1ns / 1 ps

module jesd204b_TSS
(
  output reg  [15:0]    TSS_out,        // Test samples sequence data out
  input  wire           CLK_2char,      //
  input  wire           CLK_frm,        // frame clock
  input  wire           MF_end,         // Multi-frame end
  input  wire           TSS_pat_en,     // Test Sample Sequence Pattern Enable (from JESD204b_core)
  input  wire [3:0]     N_,             // ADC resolution (base 0, hard coded)
  input  wire [1:0]     CID,            // converter ID (base 0, hardware configurable, hard coded)
  input  wire [2:0]     LID,            // Lane ID (inner lanes = 0, outer lanes =1, hard coded)
  input  wire           two_adc_per_ln, //
  input  wire           two_ln_per_adc
);

  reg  [4:0]     TSS_frm_cnt;           // TSS frame count
  reg  [1:0]     S_;                    // Samples per ADC per Frame cycle
  wire [2:0]     SID;                   // Sample ID (base 0)
  reg  [2:0]     nextCID_gen;
  reg  [2:0]     CID_gen;               // Generated Converter ID (different from CID if 2 ADCs per lane)
  wire [2:0]     CID1;
  wire [2:0]     SID1;                  // SID base1
  reg  [2:0]     nextCID_gen1;          //
  reg  [2:0]     CID_gen1;              // CID_gen base1
  wire           cntl_bit;

  assign SID = two_ln_per_adc && LID[0];
  assign SID1 = SID + 1'b1;
  assign CID1 = CID + 1'b1;
  assign cntl_bit = ((S_ * CID_gen + LID) == TSS_frm_cnt);

  always @(posedge CLK_frm) // or negedge TSS_pat_en)
  begin
    if (!TSS_pat_en)
    begin
      TSS_frm_cnt <= 5'h00;
    end
    else if (MF_end)
    begin
      TSS_frm_cnt <= 5'h00;
    end
    else
    begin
      TSS_frm_cnt <= TSS_frm_cnt+1;
    end
  end //always

  always @(CID, CID1, CLK_frm, two_adc_per_ln)
  begin
    if (two_adc_per_ln)                           // When two ADCs per lane, CLK_frm is CLK_2char/2, so there is a 50% Duty Cycle
    begin
      //#0.1                                      // and CLK_frm can be used to determine the CID.
      nextCID_gen1[2:0] = CID1[2:0] + CLK_frm;    // base 1,  CID may be 0 and 1 within a single frame if 2 ADC/LN
      nextCID_gen[2:0]  = CID[1:0] + CLK_frm;     //
      S_ = 2'h1;                                  // base 0
    end
    else
    begin
      nextCID_gen1[2:0] = CID1[2:0];              // base 1
      nextCID_gen[2:0]  = {1'b0, CID[1:0]};       // base 0
      S_ = 2'h0;                                  // base 0
    end
  end

  always @(posedge CLK_2char)
  begin
    if (!TSS_pat_en)
    begin
      CID_gen1[2:0] <= CID1[2:0];
      CID_gen[2:0]  <= {1'b0, CID[1:0]};
    end
    else
    begin
      CID_gen1[2:0] <= nextCID_gen1[2:0];
      CID_gen[2:0]  <= nextCID_gen[2:0];
    end
  end

   /* When 2 lanes per ADC are used, there are 2 samples/ADC/Frame cycle.
  Lane 0 corresponds to sample 1 and Lane 1 corresponds to sample 2.
  Lanes A0 and B0 are adjacent to each other. Lanes A1 and B1 are the
  outside lanes.

  Sample ID (SID) is a function of lanes per ADC since one sample
  is sent to each lane.  The SID+1 should always be "1" for inner
  lanes and "2" for outer lanes */

  always @(TSS_frm_cnt, SID1, CID_gen1, cntl_bit, N_)
  begin
    case (TSS_frm_cnt)
      5'h00:
      begin
        if      (N_ == 4'hf) TSS_out[15:0] = {1'b0, 8'h00, 4'b0000, CID_gen1};               // 16 bit
        else if (N_ == 4'he) TSS_out[15:0] = {1'b0, 8'h00, 3'b000, CID_gen1, cntl_bit};      // 15 bit
        else if (N_ == 4'hd) TSS_out[15:0] = {1'b0, 8'h00, 2'b00, CID_gen1, cntl_bit, 1'b0}; // 14 bit
        else if (N_ == 4'hc) TSS_out[15:0] = {1'b0, 8'h00, 1'b0, CID_gen1, cntl_bit, 2'b00}; // 13 bit
        else                 TSS_out[15:0] = {1'b0, 8'h00,      CID_gen1, cntl_bit, 3'b000}; // 12 bit
      end

      5'h01:
      begin
        if      (N_ == 4'hf) TSS_out[15:0] = {1'b0, 8'h00, 4'b0000, SID1};               // 16 bit
        else if (N_ == 4'he) TSS_out[15:0] = {1'b0, 8'h00, 3'b000, SID1, cntl_bit};      // 15 bit
        else if (N_ == 4'hd) TSS_out[15:0] = {1'b0, 8'h00, 2'b00, SID1, cntl_bit, 1'b0}; // 14 bit
        else if (N_ == 4'hc) TSS_out[15:0] = {1'b0, 8'h00, 1'b0, SID1, cntl_bit, 2'b00}; // 13 bit
        else                 TSS_out[15:0] = {1'b0, 8'h00,      SID1, cntl_bit, 3'b000}; // 12 bit
      end
      default:
      begin
        if      (N_ == 4'hf) TSS_out[15:0] = {1'b1, 8'h00, 7'b0000000};                // 16 bit
        else if (N_ == 4'he) TSS_out[15:0] = {1'b1, 8'h00, 6'b000000, cntl_bit};      // 15 bit
        else if (N_ == 4'hd) TSS_out[15:0] = {1'b1, 8'h00, 5'b00000, cntl_bit, 1'b0}; // 14 bit
        else if (N_ == 4'hc) TSS_out[15:0] = {1'b1, 8'h00, 4'b0000, cntl_bit, 2'b00}; // 13 bit
        else                 TSS_out[15:0] = {1'b1, 8'h00, 3'b000, cntl_bit, 3'b000}; // 12 bit
      end
    endcase
  end //always
endmodule

`timescale 1ns / 1 ps

module jesd204b_inff
(
output reg          TX_SYNC_en,
output reg  [7:0]   DID,
output reg  [3:0]   BID,
output reg  [2:0]   L_,
output reg  [4:0]   K_,
output reg  [2:0]   M_,
output reg  [1:0]   CS,
output reg  [3:0]   N_,
output reg  [1:0]   SUBCLASSV,
output reg  [2:0]   LID,
output reg  [1:0]   CID,
output reg          SCR,
output reg          LAS_dis,
output reg          FAM_dis,
output reg          LAM_dis,
output reg          two_adc_per_ln,
output reg          two_ln_per_adc,
output reg  [2:0]   PAT,
input  wire         static_clk,
input  wire         scan_mode,
input  wire         TX_SYNC_en_in,
input  wire [7:0]   DID_in,
input  wire [3:0]   BID_in,
input  wire [2:0]   L_in,
input  wire [4:0]   K_in,
input  wire [2:0]   M_in,
input  wire [1:0]   CS_in,
input  wire [3:0]   N_in,
input  wire [1:0]   SUBCLASSV_in,
input  wire [2:0]   LID_in,
input  wire [1:0]   CID_in,
input  wire         SCR_in,
input  wire         LAS_dis_in,
input  wire         FAM_dis_in,
input  wire         LAM_dis_in,
input  wire         two_adc_per_ln_in,
input  wire         two_ln_per_adc_in,
input  wire [2:0]   PAT_in
);

wire         TX_SYNC_en_set;    
wire [7:0]   DID_set;           
wire [3:0]   BID_set;           
wire [2:0]   L_set;            
wire [4:0]   K_set;            
wire [2:0]   M_set;            
wire [1:0]   CS_set;            
wire [3:0]   N_set;            
wire [1:0]   SUBCLASSV_set;     
wire [2:0]   LID_set;           
wire [1:0]   CID_set;           
wire         SCR_set;           
wire         LAS_dis_set;       
wire         FAM_dis_set;       
wire         LAM_dis_set;       
wire         two_adc_per_ln_set;
wire         two_ln_per_adc_set;
wire [2:0]   PAT_set;           
wire         TX_SYNC_en_rst;    
wire [7:0]   DID_rst;           
wire [3:0]   BID_rst;           
wire [2:0]   L_rst;            
wire [4:0]   K_rst;            
wire [2:0]   M_rst;            
wire [1:0]   CS_rst;            
wire [3:0]   N_rst;            
wire [1:0]   SUBCLASSV_rst;     
wire [2:0]   LID_rst;           
wire [1:0]   CID_rst;           
wire         SCR_rst;           
wire         LAS_dis_rst;       
wire         FAM_dis_rst;       
wire         LAM_dis_rst;       
wire         two_adc_per_ln_rst;
wire         two_ln_per_adc_rst;
wire [2:0]   PAT_rst;            

assign TX_SYNC_en_set = TX_SYNC_en_in && !scan_mode; 
assign TX_SYNC_en_rst = !TX_SYNC_en_in && !scan_mode;
always @(posedge static_clk or posedge TX_SYNC_en_set or posedge TX_SYNC_en_rst)
begin
        if      (TX_SYNC_en_set) TX_SYNC_en <= 1'b1;
        else if (TX_SYNC_en_rst) TX_SYNC_en <= 1'b0;
        else                     TX_SYNC_en <= TX_SYNC_en;
end

assign DID_set[0] =  DID_in[0] && !scan_mode; 
assign DID_rst[0] = !DID_in[0] && !scan_mode;
always @(posedge static_clk or posedge DID_set[0] or posedge DID_rst[0])
begin
        if      (DID_set[0]) DID[0] <= 1'b1;
        else if (DID_rst[0]) DID[0] <= 1'b0;
        else                 DID[0] <= DID[0];
end

assign DID_set[1] =  DID_in[1] && !scan_mode; 
assign DID_rst[1] = !DID_in[1] && !scan_mode;
always @(posedge static_clk or posedge DID_set[1] or posedge DID_rst[1])
begin
        if      (DID_set[1]) DID[1] <= 1'b1;
        else if (DID_rst[1]) DID[1] <= 1'b0;
        else                 DID[1] <= DID[1];
end

assign DID_set[2] =  DID_in[2] && !scan_mode; 
assign DID_rst[2] = !DID_in[2] && !scan_mode;
always @(posedge static_clk or posedge DID_set[2] or posedge DID_rst[2])
begin
        if      (DID_set[2]) DID[2] <= 1'b1;
        else if (DID_rst[2]) DID[2] <= 1'b0;
        else                 DID[2] <= DID[2];
end

assign DID_set[3] =  DID_in[3] && !scan_mode; 
assign DID_rst[3] = !DID_in[3] && !scan_mode;
always @(posedge static_clk or posedge DID_set[3] or posedge DID_rst[3])
begin
        if      (DID_set[3]) DID[3] <= 1'b1;
        else if (DID_rst[3]) DID[3] <= 1'b0;
        else                 DID[3] <= DID[3];
end

assign DID_set[4] =  DID_in[4] && !scan_mode; 
assign DID_rst[4] = !DID_in[4] && !scan_mode;
always @(posedge static_clk or posedge DID_set[4] or posedge DID_rst[4])
begin
        if      (DID_set[4]) DID[4] <= 1'b1;
        else if (DID_rst[4]) DID[4] <= 1'b0;
        else                 DID[4] <= DID[4];
end

assign DID_set[5] =  DID_in[5] && !scan_mode; 
assign DID_rst[5] = !DID_in[5] && !scan_mode;
always @(posedge static_clk or posedge DID_set[5] or posedge DID_rst[5])
begin
        if      (DID_set[5]) DID[5] <= 1'b1;
        else if (DID_rst[5]) DID[5] <= 1'b0;
        else                 DID[5] <= DID[5];
end

assign DID_set[6] =  DID_in[6] && !scan_mode; 
assign DID_rst[6] = !DID_in[6] && !scan_mode;
always @(posedge static_clk or posedge DID_set[6] or posedge DID_rst[6])
begin
        if      (DID_set[6]) DID[6] <= 1'b1;
        else if (DID_rst[6]) DID[6] <= 1'b0;
        else                 DID[6] <= DID[6];
end

assign DID_set[7] =  DID_in[7] && !scan_mode; 
assign DID_rst[7] = !DID_in[7] && !scan_mode;
always @(posedge static_clk or posedge DID_set[7] or posedge DID_rst[7])
begin
        if      (DID_set[7]) DID[7] <= 1'b1;
        else if (DID_rst[7]) DID[7] <= 1'b0;
        else                 DID[7] <= DID[7];
end

assign BID_set[0] =  BID_in[0] && !scan_mode; 
assign BID_rst[0] = !BID_in[0] && !scan_mode;
always @(posedge static_clk or posedge BID_set[0] or posedge BID_rst[0])
begin
        if      (BID_set[0]) BID[0] <= 1'b1;
        else if (BID_rst[0]) BID[0] <= 1'b0;
        else                 BID[0] <= BID[0];
end

assign BID_set[1] =  BID_in[1] && !scan_mode; 
assign BID_rst[1] = !BID_in[1] && !scan_mode;
always @(posedge static_clk or posedge BID_set[1] or posedge BID_rst[1])
begin
        if      (BID_set[1]) BID[1] <= 1'b1;
        else if (BID_rst[1]) BID[1] <= 1'b0;
        else                 BID[1] <= BID[1];
end

assign BID_set[2] =  BID_in[2] && !scan_mode; 
assign BID_rst[2] = !BID_in[2] && !scan_mode;
always @(posedge static_clk or posedge BID_set[2] or posedge BID_rst[2])
begin
        if      (BID_set[2]) BID[2] <= 1'b1;
        else if (BID_rst[2]) BID[2] <= 1'b0;
        else                 BID[2] <= BID[2];
end

assign BID_set[3] =  BID_in[3] && !scan_mode; 
assign BID_rst[3] = !BID_in[3] && !scan_mode;
always @(posedge static_clk or posedge BID_set[3] or posedge BID_rst[3])
begin
        if      (BID_set[3]) BID[3] <= 1'b1;
        else if (BID_rst[3]) BID[3] <= 1'b0;
        else                 BID[3] <= BID[3];
end

assign L_set[0] =  L_in[0] && !scan_mode; 
assign L_rst[0] = !L_in[0] && !scan_mode;
always @(posedge static_clk or posedge L_set[0] or posedge L_rst[0])
begin
        if      (L_set[0]) L_[0] <= 1'b1;
        else if (L_rst[0]) L_[0] <= 1'b0;
        else               L_[0] <= L_[0];
end

assign L_set[1] =  L_in[1] && !scan_mode; 
assign L_rst[1] = !L_in[1] && !scan_mode;
always @(posedge static_clk or posedge L_set[1] or posedge L_rst[1])
begin
        if      (L_set[1]) L_[1] <= 1'b1;
        else if (L_rst[1]) L_[1] <= 1'b0;
        else               L_[1] <= L_[1];
end

assign L_set[2] =  L_in[2] && !scan_mode; 
assign L_rst[2] = !L_in[2] && !scan_mode;
always @(posedge static_clk or posedge L_set[2] or posedge L_rst[2])
begin
        if      (L_set[2]) L_[2] <= 1'b1;
        else if (L_rst[2]) L_[2] <= 1'b0;
        else               L_[2] <= L_[2];
end

assign K_set[0] =  K_in[0] && !scan_mode; 
assign K_rst[0] = !K_in[0] && !scan_mode;
always @(posedge static_clk or posedge K_set[0] or posedge K_rst[0])
begin
        if      (K_set[0]) K_[0] <= 1'b1;
        else if (K_rst[0]) K_[0] <= 1'b0;
        else               K_[0] <= K_[0];
end

assign K_set[1] =  K_in[1] && !scan_mode; 
assign K_rst[1] = !K_in[1] && !scan_mode;
always @(posedge static_clk or posedge K_set[1] or posedge K_rst[1])
begin
        if      (K_set[1]) K_[1] <= 1'b1;
        else if (K_rst[1]) K_[1] <= 1'b0;
        else               K_[1] <= K_[1];
end

assign K_set[2] =  K_in[2] && !scan_mode; 
assign K_rst[2] = !K_in[2] && !scan_mode;
always @(posedge static_clk or posedge K_set[2] or posedge K_rst[2])
begin
        if      (K_set[2]) K_[2] <= 1'b1;
        else if (K_rst[2]) K_[2] <= 1'b0;
        else               K_[2] <= K_[2];
end

assign K_set[3] =  K_in[3] && !scan_mode; 
assign K_rst[3] = !K_in[3] && !scan_mode;
always @(posedge static_clk or posedge K_set[3] or posedge K_rst[3])
begin
        if      (K_set[3]) K_[3] <= 1'b1;
        else if (K_rst[3]) K_[3] <= 1'b0;
        else               K_[3] <= K_[3];
end

assign K_set[4] =  K_in[4] && !scan_mode; 
assign K_rst[4] = !K_in[4] && !scan_mode;
always @(posedge static_clk or posedge K_set[4] or posedge K_rst[4])
begin
        if      (K_set[4]) K_[4] <= 1'b1;
        else if (K_rst[4]) K_[4] <= 1'b0;
        else               K_[4] <= K_[4];
end

assign M_set[0] =  M_in[0] && !scan_mode; 
assign M_rst[0] = !M_in[0] && !scan_mode;
always @(posedge static_clk or posedge M_set[0] or posedge M_rst[0])
begin
        if      (M_set[0]) M_[0] <= 1'b1;
        else if (M_rst[0]) M_[0] <= 1'b0;
        else               M_[0] <= M_[0];
end

assign M_set[1] =  M_in[1] && !scan_mode; 
assign M_rst[1] = !M_in[1] && !scan_mode;
always @(posedge static_clk or posedge M_set[1] or posedge M_rst[1])
begin
        if      (M_set[1]) M_[1] <= 1'b1;
        else if (M_rst[1]) M_[1] <= 1'b0;
        else               M_[1] <= M_[1];
end

assign M_set[2] =  M_in[2] && !scan_mode; 
assign M_rst[2] = !M_in[2] && !scan_mode;
always @(posedge static_clk or posedge M_set[2] or posedge M_rst[2])
begin
        if      (M_set[2]) M_[2] <= 1'b1;
        else if (M_rst[2]) M_[2] <= 1'b0;
        else               M_[2] <= M_[2];
end

assign CS_set[0] =  CS_in[0] && !scan_mode; 
assign CS_rst[0] = !CS_in[0] && !scan_mode;
always @(posedge static_clk or posedge CS_set[0] or posedge CS_rst[0])
begin
        if      (CS_set[0]) CS[0] <= 1'b1;
        else if (CS_rst[0]) CS[0] <= 1'b0;
        else                CS[0] <= CS[0];
end                               

assign CS_set[1] =  CS_in[1] && !scan_mode; 
assign CS_rst[1] = !CS_in[1] && !scan_mode;
always @(posedge static_clk or posedge CS_set[1] or posedge CS_rst[1])
begin
        if      (CS_set[1]) CS[1] <= 1'b1;
        else if (CS_rst[1]) CS[1] <= 1'b0;
        else                CS[1] <= CS[1];
end

assign N_set[0] =  N_in[0] && !scan_mode; 
assign N_rst[0] = !N_in[0] && !scan_mode;
always @(posedge static_clk or posedge N_set[0] or posedge N_rst[0])
begin
        if      (N_set[0]) N_[0] <= 1'b1;
        else if (N_rst[0]) N_[0] <= 1'b0;
        else               N_[0] <= N_[0];
end

assign N_set[1] =  N_in[1] && !scan_mode; 
assign N_rst[1] = !N_in[1] && !scan_mode;
always @(posedge static_clk or posedge N_set[1] or posedge N_rst[1])
begin
        if      (N_set[1]) N_[1] <= 1'b1;
        else if (N_rst[1]) N_[1] <= 1'b0;
        else               N_[1] <= N_[1];
end

assign N_set[2] =  N_in[2] && !scan_mode; 
assign N_rst[2] = !N_in[2] && !scan_mode;
always @(posedge static_clk or posedge N_set[2] or posedge N_rst[2])
begin
        if      (N_set[2]) N_[2] <= 1'b1;
        else if (N_rst[2]) N_[2] <= 1'b0;
        else               N_[2] <= N_[2];
end

assign N_set[3] =  N_in[3] && !scan_mode; 
assign N_rst[3] = !N_in[3] && !scan_mode;
always @(posedge static_clk or posedge N_set[3] or posedge N_rst[3])
begin
        if      (N_set[3]) N_[3] <= 1'b1;
        else if (N_rst[3]) N_[3] <= 1'b0;
        else               N_[3] <= N_[3];
end

assign SUBCLASSV_set[0] =  SUBCLASSV_in[0] && !scan_mode; 
assign SUBCLASSV_rst[0] = !SUBCLASSV_in[0] && !scan_mode;
always @(posedge static_clk or posedge SUBCLASSV_set[0] or posedge SUBCLASSV_rst[0])
begin
        if      (SUBCLASSV_set[0]) SUBCLASSV[0] <= 1'b1;
        else if (SUBCLASSV_rst[0]) SUBCLASSV[0] <= 1'b0;
        else                       SUBCLASSV[0] <= SUBCLASSV[0];
end

assign SUBCLASSV_set[1] =  SUBCLASSV_in[1] && !scan_mode; 
assign SUBCLASSV_rst[1] = !SUBCLASSV_in[1] && !scan_mode;
always @(posedge static_clk or posedge SUBCLASSV_set[1] or posedge SUBCLASSV_rst[1])
begin
        if      (SUBCLASSV_set[1]) SUBCLASSV[1] <= 1'b1;
        else if (SUBCLASSV_rst[1]) SUBCLASSV[1] <= 1'b0;
        else                       SUBCLASSV[1] <= SUBCLASSV[1];
end

assign LID_set[0] =  LID_in[0] && !scan_mode; 
assign LID_rst[0] = !LID_in[0] && !scan_mode;
always @(posedge static_clk or posedge LID_set[0] or posedge LID_rst[0])
begin
        if      (LID_set[0]) LID[0] <= 1'b1;
        else if (LID_rst[0]) LID[0] <= 1'b0;
        else                 LID[0] <= LID[0];
end

assign LID_set[1] =  LID_in[1] && !scan_mode; 
assign LID_rst[1] = !LID_in[1] && !scan_mode;
always @(posedge static_clk or posedge LID_set[1] or posedge LID_rst[1])
begin
        if      (LID_set[1]) LID[1] <= 1'b1;
        else if (LID_rst[1]) LID[1] <= 1'b0;
        else                 LID[1] <= LID[1];
end

assign LID_set[2] =  LID_in[2] && !scan_mode; 
assign LID_rst[2] = !LID_in[2] && !scan_mode;
always @(posedge static_clk or posedge LID_set[2] or posedge LID_rst[2])
begin
        if      (LID_set[2]) LID[2] <= 1'b1;
        else if (LID_rst[2]) LID[2] <= 1'b0;
        else                 LID[2] <= LID[2];
end

assign CID_set[0] =  CID_in[0] && !scan_mode; 
assign CID_rst[0] = !CID_in[0] && !scan_mode;
always @(posedge static_clk or posedge CID_set[0] or posedge CID_rst[0])
begin
        if      (CID_set[0]) CID[0] <= 1'b1;
        else if (CID_rst[0]) CID[0] <= 1'b0;
        else                 CID[0] <= CID[0];
end

assign CID_set[1] =  CID_in[1] && !scan_mode; 
assign CID_rst[1] = !CID_in[1] && !scan_mode;
always @(posedge static_clk or posedge CID_set[1] or posedge CID_rst[1])
begin
        if      (CID_set[1]) CID[1] <= 1'b1;
        else if (CID_rst[1]) CID[1] <= 1'b0;
        else                 CID[1] <= CID[1];
end

assign SCR_set = SCR_in && !scan_mode; 
assign SCR_rst = !SCR_in && !scan_mode;
always @(posedge static_clk or posedge SCR_set or posedge SCR_rst)
begin
        if      (SCR_set) SCR <= 1'b1;
        else if (SCR_rst) SCR <= 1'b0;
        else              SCR <= SCR;
end

assign LAS_dis_set = LAS_dis_in && !scan_mode; 
assign LAS_dis_rst = !LAS_dis_in && !scan_mode;
always @(posedge static_clk or posedge LAS_dis_set or posedge LAS_dis_rst)
begin
        if      (LAS_dis_set) LAS_dis <= 1'b1;
        else if (LAS_dis_rst) LAS_dis <= 1'b0;
        else                  LAS_dis <= LAS_dis;
end

assign FAM_dis_set = FAM_dis_in && !scan_mode; 
assign FAM_dis_rst = !FAM_dis_in && !scan_mode;
always @(posedge static_clk or posedge FAM_dis_set or posedge FAM_dis_rst)
begin
        if      (FAM_dis_set) FAM_dis <= 1'b1;
        else if (FAM_dis_rst) FAM_dis <= 1'b0;
        else                  FAM_dis <= FAM_dis;
end

assign LAM_dis_set = LAM_dis_in && !scan_mode; 
assign LAM_dis_rst = !LAM_dis_in && !scan_mode;
always @(posedge static_clk or posedge LAM_dis_set or posedge LAM_dis_rst)
begin
        if      (LAM_dis_set) LAM_dis <= 1'b1;
        else if (LAM_dis_rst) LAM_dis <= 1'b0;
        else                  LAM_dis <= LAM_dis;
end

assign two_adc_per_ln_set = two_adc_per_ln_in && !scan_mode; 
assign two_adc_per_ln_rst = !two_adc_per_ln_in && !scan_mode;
always @(posedge static_clk or posedge two_adc_per_ln_set or posedge two_adc_per_ln_rst)
begin
        if      (two_adc_per_ln_set) two_adc_per_ln <= 1'b1;
        else if (two_adc_per_ln_rst) two_adc_per_ln <= 1'b0;
        else                         two_adc_per_ln <= two_adc_per_ln;
end

assign two_ln_per_adc_set = two_ln_per_adc_in && !scan_mode; 
assign two_ln_per_adc_rst = !two_ln_per_adc_in && !scan_mode;
always @(posedge static_clk or posedge two_ln_per_adc_set or posedge two_ln_per_adc_rst)
begin
        if      (two_ln_per_adc_set) two_ln_per_adc <= 1'b1;
        else if (two_ln_per_adc_rst) two_ln_per_adc <= 1'b0;
        else                         two_ln_per_adc <= two_ln_per_adc;
end

assign PAT_set[0] = PAT_in[0] && !scan_mode; 
assign PAT_rst[0] = !PAT_in[0] && !scan_mode;
always @(posedge static_clk or posedge PAT_set[0] or posedge PAT_rst[0])
begin
        if      (PAT_set[0]) PAT[0] <= 1'b1;
        else if (PAT_rst[0]) PAT[0] <= 1'b0;
        else                 PAT[0] <= PAT[0];
end

assign PAT_set[1] = PAT_in[1] && !scan_mode; 
assign PAT_rst[1] = !PAT_in[1] && !scan_mode;
always @(posedge static_clk or posedge PAT_set[1] or posedge PAT_rst[1])
begin
        if      (PAT_set[1]) PAT[1] <= 1'b1;
        else if (PAT_rst[1]) PAT[1] <= 1'b0;
        else                 PAT[1] <= PAT[1];
end

assign PAT_set[2] = PAT_in[2] && !scan_mode; 
assign PAT_rst[2] = !PAT_in[2] && !scan_mode;
always @(posedge static_clk or posedge PAT_set[2] or posedge PAT_rst[2])
begin
        if      (PAT_set[2]) PAT[2] <= 1'b1;
        else if (PAT_rst[2]) PAT[2] <= 1'b0;
        else                 PAT[2] <= PAT[2];
end

endmodule


module jesd204b_custom ( DATA_A0,DATA_A1,
DATA_B0,DATA_B1,MF_clk,MF_chg,MF_end,PLL_ref,SYNC_det,div2_rstb,
ser_A0_en,ser_A1_en,ser_B0_en,ser_B1_en,two_adc_per_ln,dvp,gnd,CLK_frm,
DATA_A,DATA_B,K_,L_,R_,RST_dis,SUBCLASSV,adc_clk,alert_en,dev_clk,nap,
pd_all,pda,pdb,single,sync,sysref,two_ln_per_adc,MF_chg_frm ); 

output [15:0]  DATA_A0;
output [15:0]  DATA_A1;
output [15:0] 
 DATA_B0;
output [15:0]  DATA_B1;
output   MF_clk;
output   MF_chg;

output   MF_end;
output   PLL_ref;
output   SYNC_det;
output  
 div2_rstb;
output   ser_A0_en;
output   ser_A1_en;
output  
 ser_B0_en;
output   ser_B1_en;
output   two_adc_per_ln;
inout   dvp;

inout   gnd;
input   CLK_frm;
input [15:0]  DATA_A;
input [15:0] 
 DATA_B;
input [4:0]  K_;
input [2:0]  L_;
input [2:0]  R_;
input  
 RST_dis;
input [1:0]  SUBCLASSV;
input   adc_clk;
input   alert_en;

input   dev_clk;
input   nap;
input   pd_all;
input   pda;
input  
 pdb;
input   single;
input   sync;
input   sysref;
output  
 two_ln_per_adc;
output   MF_chg_frm;

 
 
 
pd_modes pd_modes ( .two_ln_per_adc( 
two_ln_per_adc ), .ser_B1_en( ser_B1_en ), .nap( nap ), .disable_B0( 
disable_B0 ), .disable_B1( disable_B1 ), .pd_all( pd_all ), .single( 
single ), .ser_A0_en( ser_A0_en ), .two_adc_per_ln( two_adc_per_ln ), 
.ser_B0_en( ser_B0_en ), .ser_A1_en( ser_A1_en ), .L_( L_[2:0] ), .gnd( 
gnd ), .disable_A0( disable_A0 ), .disable_A1( disable_A1 ), .pdb( pdb ), 
.vdd( dvp ), .pda( pda ) );
 
align_cntl align_cntl ( .SUBCLASSV( 
SUBCLASSV[1:0] ), .two_ln_per_adc( two_ln_per_adc ), .ADC_clk( adc_clk ), 
.FRM_clk( FRM_clk ), .R_( R_[2:0] ), .CLK_frm( CLK_frm ), .MF_chg( 
MF_chg ), .RST_dis( RST_dis ), .K_( K_[4:0] ), .MF_end( MF_end ), .DEV_clk( 
dev_clk ), .MF_clk( MF_clk ), .alert_en( alert_en ), .MF_chg_frm( 
MF_chg_frm ), .CLKDIV_RSTb( div2_rstb ), .\SYNC~ ( sync ), .gnd( gnd ), 
.SYNC_det( SYNC_det ), .SYSREF( sysref ), .vdd( dvp ) );
 
retime_ff retime_ff ( .DATA_B1( 
DATA_B1[15:0] ), .DATA_B0( DATA_B0[15:0] ), .FRM_clk( FRM_clk ), .disable_B0( 
disable_B0 ), .disable_B1( disable_B1 ), .DATA_A1( DATA_A1[15:0] ), .DATA_A0( 
DATA_A0[15:0] ), .two_adc_per_ln( two_adc_per_ln ), .DATA_B( 
DATA_B[15:0] ), .DATA_A( DATA_A[15:0] ), .PLL_ref( PLL_ref ), .gnd( 
gnd ), .disable_A0( disable_A0 ), .disable_A1( disable_A1 ), .vdd( dvp ) );

endmodule

// 
// Design File for: (ltc2123 pd_modes schematic)
//
// Verilog-AMS netlist generated by the AMS netlister, version 5.10.41.500.6.143.
// Cadence Design Systems, Inc.

 
module pd_modes ( disable_A0,disable_A1,
disable_B0,disable_B1,ser_A0_en,ser_A1_en,ser_B0_en,ser_B1_en,
two_adc_per_ln,two_ln_per_adc,gnd,vdd,L_,nap,pda,pdb,single,pd_all ); 

output   disable_A0;
output   disable_A1;
output   disable_B0;

output   disable_B1;
output   ser_A0_en;
output   ser_A1_en;
output  
 ser_B0_en;
output   ser_B1_en;
output   two_adc_per_ln;
output  
 two_ln_per_adc;
inout   gnd;
inout   vdd;
input [2:0]  L_;
input  
 nap;
input   pda;
input   pdb;
input   single;
input   pd_all;

 
 
 
TIEHI I111
 ( .Y( net089 ) );
 
TIELO I109
 ( .Y( net90 ) );
TIELO I98
 ( .Y( net91 ) );
 
AND2X1 I61
 ( .Y( net103 ), .A( net262 ), .B( PD_ALLb ) );
 
AND2X2 I67
 ( .Y( ser_B0_en ), .A( net173 ), .B( PD_ALLb ) );
AND2X2 I64
 ( .Y( ser_A0_en ), .A( net271 ), .B( PD_ALLb ) );
AND2X2 I70
 ( .Y( ser_B1_en ), .A( net274 ), .B( PD_ALLb ) );
AND2X2 I63
 ( .Y( ser_A1_en ), .A( two_ln_per_adc ), .B( net103 ) );
AND2X2 I72
 ( .Y( two_adc_per_ln ), .A( one_lane ), .B( SINGLEb ) );
 
NAND3X2 I53 ( .Y( disable_B1 ), .A( one_laneb ), .B( two_laneb ), .C( 
net244 ) );
NAND3X2 I44 ( .Y( disable_A1 ), .A( one_laneb ), .B( net265 ), .C( net250 ) );
NAND3X2 I46 ( .Y( disable_A0 ), .A( NAPb ), .B( net169 ), .C( net253 ) );
 
NAND3X1 I100 ( .Y( net133 ), .A( net90 ), .B( net90 ), .C( net90 ) );
NAND3X1 I103 ( .Y( net137 ), .A( net90 ), .B( net90 ), .C( net90 ) );
NAND3X1 I110 ( .Y( net141 ), .A( net133 ), .B( net137 ), .C( net90 ) );
NAND3X1 I90 ( .Y( net145 ), .A( net91 ), .B( net91 ), .C( net91 ) );
NAND3X1 I76 ( .Y( net149 ), .A( one_laneb ), .B( SINGLEb ), .C( PDBb ) );
NAND3X1 I89 ( .Y( net153 ), .A( net91 ), .B( net91 ), .C( net91 ) );
NAND3X1 I39 ( .Y( two_laneb ), .A( L0 ), .B( L1b ), .C( L2b ) );
NAND3X1 I42 ( .Y( comb_PDB ), .A( NAPb ), .B( PDBb ), .C( SINGLEb ) );
NAND3X1 I77 ( .Y( net165 ), .A( two_lane ), .B( NAP ), .C( SINGLEb ) );
NAND3X1 I48 ( .Y( net169 ), .A( one_lane ), .B( PDA ), .C( net256 ) );
NAND3X1 I79 ( .Y( net173 ), .A( net149 ), .B( net165 ), .C( net259 ) );
NAND3X1 I93 ( .Y( net177 ), .A( net91 ), .B( net91 ), .C( net91 ) );
NAND3X1 I1 ( .Y( one_laneb ), .A( L0b ), .B( L1b ), .C( L2b ) );
 
NAND2X2 I52 ( .Y( disable_B0 ), .A( net089 ), .B( net268 ) );
NAND2X2 I58 ( .Y( two_ln_per_adc ), .A( net247 ), .B( four_laneb ) );
 
INVX1 I101
 ( .Y( net92 ), .A( net141 ) );
INVX1 I91
 ( .Y( net192 ), .A( net153 ) );
INVX1 I104
 ( .Y( net194 ), .A( net90 ) );
INVX1 I105
 ( .Y( net196 ), .A( net90 ) );
INVX1 I106
 ( .Y( net198 ), .A( net90 ) );
INVX1 I92
 ( .Y( net200 ), .A( net145 ) );
INVX1 I75
 ( .Y( NAP ), .A( NAPb ) );
INVX1 I94
 ( .Y( net204 ), .A( net177 ) );
INVX1 I55
 ( .Y( two_lane ), .A( two_laneb ) );
INVX1 I37
 ( .Y( PD_ALLb ), .A( pd_all ) );
INVX1 I81
 ( .Y( net210 ), .A( L1b ) );
INVX1 I80
 ( .Y( net212 ), .A( PDBb ) );
INVX1 I59
 ( .Y( four_laneb ), .A( four_lane ) );
INVX1 I18
 ( .Y( SINGLEb ), .A( single ) );
INVX1 I49
 ( .Y( PDA ), .A( PDAb ) );
INVX1 I17
 ( .Y( NAPb ), .A( nap ) );
INVX1 I82
 ( .Y( net222 ), .A( L2b ) );
INVX1 I57
 ( .Y( SINGLE ), .A( SINGLEb ) );
INVX1 I40
 ( .Y( one_lane ), .A( one_laneb ) );
INVX1 I9
 ( .Y( L0b ), .A( L_[0] ) );
INVX1 I8
 ( .Y( L1b ), .A( L_[1] ) );
INVX1 I7
 ( .Y( L2b ), .A( L_[2] ) );
INVX1 I6
 ( .Y( PDBb ), .A( pdb ) );
INVX1 I38
 ( .Y( L0 ), .A( L0b ) );
INVX1 I5
 ( .Y( PDAb ), .A( pda ) );
 
NAND2X1 I74 ( .Y( net241 ), .A( NAP ), .B( four_lane ) );
NAND2X1 I54 ( .Y( net244 ), .A( four_lane ), .B( comb_PDB ) );
NAND2X1 I56 ( .Y( net247 ), .A( two_lane ), .B( SINGLE ) );
NAND2X1 I45 ( .Y( net250 ), .A( two_lane ), .B( SINGLEb ) );
NAND2X1 I73 ( .Y( net253 ), .A( one_laneb ), .B( PDA ) );
NAND2X1 I47 ( .Y( net256 ), .A( PDBb ), .B( SINGLEb ) );
NAND2X1 I78 ( .Y( net259 ), .A( four_lane ), .B( NAP ) );
NAND2X1 I60 ( .Y( net262 ), .A( PDA ), .B( NAPb ) );
NAND2X1 I43 ( .Y( net265 ), .A( one_laneb ), .B( comb_PDA ) );
NAND2X1 I50 ( .Y( net268 ), .A( net089 ), .B( comb_PDB ) );
NAND2X1 I66 ( .Y( net271 ), .A( disable_A0 ), .B( NAPb ) );
NAND2X1 I71 ( .Y( net274 ), .A( disable_B1 ), .B( net241 ) );
NAND2X1 I41 ( .Y( comb_PDA ), .A( NAPb ), .B( PDAb ) );
NAND2X1 I0 ( .Y( four_lane ), .A( L1b ), .B( L2b ) );

endmodule

// 
// Design File for: (tsmc18 NAND3X1 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module NAND3X1 (Y, A, B, C);
output Y;
input A, B, C;

  nand (Y, A, B, C);

  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05,
      tplh$B$Y = 0.05,
      tphl$B$Y = 0.05,
      tplh$C$Y = 0.05,
      tphl$C$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
    (B *> Y) = (tplh$B$Y, tphl$B$Y);
    (C *> Y) = (tplh$C$Y, tphl$C$Y);
  endspecify

endmodule // NAND3X1

`endcelldefine

// 
// Design File for: (ltc2123 align_cntl schematic)
//
// Verilog-AMS netlist generated by the AMS netlister, version 5.10.41.500.6.143.
// Cadence Design Systems, Inc.

 
module align_cntl ( CLKDIV_RSTb,MF_clk,FRM_clk,
SYNC_det,gnd,vdd,ADC_clk,DEV_clk,K_,R_,SUBCLASSV,\SYNC~ ,SYSREF,
RST_dis,two_ln_per_adc,alert_en,MF_chg,MF_end,CLK_frm,MF_chg_frm ); 

output   CLKDIV_RSTb;
output   MF_clk;
output   FRM_clk;
output  
 SYNC_det;
inout   gnd;
inout   vdd;
input   ADC_clk;
input  
 DEV_clk;
input [4:0]  K_;
input [2:0]  R_;
input [1:0]  SUBCLASSV;

input   \SYNC~ ;
input   SYSREF;
input   RST_dis;
input  
 two_ln_per_adc;
input   alert_en;
output   MF_chg;
output   MF_end;

input   CLK_frm;
output   MF_chg_frm;

 
 
 
ser_div2_div1_std I203 ( .q( 
FRM_clk ), .clkin( ADC_clk ), .setn( CLK_RSTb ), .div2( two_ln_per_adc ), 
.gnd( gnd ), .vdd( vdd ) );
 
NAND3X2 I48 ( .Y( net094 ), .A( net0186 ), .B( MF_CLKb ), .C( SYNC_detb ) );
 
DLY4X1 I131
 ( .Y( net0115 ), .A( MF_RSTb ) );
DLY4X1 I132
 ( .Y( net0118 ), .A( net0115 ) );
 
DFFNX2 I79
 ( .QN( net126 ), .CKN( FRM_clk ), .D( MF_clk_rt ), .Q( net127 ) );
 
DFFNRX2 I80 ( .QN( net122 ), .CKN( FRM_clk ), .D( MF_end_rt ), .RN( 
MF_RSTb ), .Q( net123 ) );
DFFNRX2 I88 ( .QN( net0153 ), .CKN( FRM_clk ), .D( SYNC_tm4 ), .RN( 
SYNC_tm4 ), .Q( net0123 ) );
 
DFFRX2 I134
 ( .QN( net0181 ), .CK( CLK_frm ), .D( net0186 ), .RN( net0186 ), .Q( 
net0107 ) );
DFFRX2 I89
 ( .QN( SYNC_detb ), .CK( CLK_frm ), .D( net0123 ), .RN( SYNC_tm4 ), .Q( 
net0129 ) );
DFFRX2 I90
 ( .QN( net0155 ), .CK( CLK_frm ), .D( net123 ), .RN( MF_RSTb ), .Q( 
net0178 ) );
 
INVX4 I140
 ( .Y( MF_chg_frm ), .A( net0196 ) );
INVX4 I97
 ( .Y( net0143 ), .A( net0129 ) );
INVX4 I81
 ( .Y( net135 ), .A( net0182 ) );
INVX4 I14
 ( .Y( CLK_RSTb ), .A( net153 ) );
INVX4 I82
 ( .Y( net133 ), .A( net0178 ) );
INVX4 I125
 ( .Y( MF_RSTb ), .A( net0119 ) );
INVX4 I57
 ( .Y( net0149 ), .A( net0186 ) );
 
INVX2 I118
 ( .Y( net0194 ), .A( net0118 ) );
 
INVX8 I99
 ( .Y( MF_chg ), .A( net0149 ) );
INVX8 I98
 ( .Y( SYNC_det ), .A( net0143 ) );
INVX8 I84
 ( .Y( MF_clk ), .A( net135 ) );
INVX8 I83
 ( .Y( MF_end ), .A( net133 ) );
INVX8 I4
 ( .Y( CLKDIV_RSTb ), .A( net0188 ) );
 
DFFX2 I91
 ( .QN( MF_CLKb ), .CK( CLK_frm ), .D( net127 ), .Q( net0182 ) );
 
NAND2X2 I119 ( .Y( net0235 ), .A( net0194 ), .B( MF_clk_tm1b ) );
NAND2X2 I138 ( .Y( net0196 ), .A( net0181 ), .B( net0186 ) );
NAND2X2 I130 ( .Y( net153 ), .A( AS_CLK_RSTb_sub1 ), .B( AS_CLK_RSTb_sub2 ) );
NAND2X2 I47 ( .Y( net0186 ), .A( net0235 ), .B( net094 ) );
NAND2X2 I124 ( .Y( net0119 ), .A( AS_CLK_RSTb_sub1 ), .B( 
MF_RSTb_sub0_sub2 ) );
NAND2X2 I3 ( .Y( net0188 ), .A( CLK_RSTb_sub1 ), .B( CLK_RSTb_sub2 ) );
 
SYNC_detect I1 ( .SUBCLASSV( 
SUBCLASSV[1:0] ), .MF_RSTb( MF_RSTb_sub0_sub2 ), .FRM_clk( FRM_clk ), 
.SYNC_tm4( SYNC_tm4 ), .RST_dis( RST_dis ), .DEV_clk( DEV_clk ), .CLK_RSTb_sub2( 
CLK_RSTb_sub2 ), .AS_CLK_RSTb_sub2( AS_CLK_RSTb_sub2 ), .\SYNC~ ( 
\SYNC~  ), .gnd( gnd ), .vdd( vdd ) );
 
SYSREF_detect I2 ( .SUBCLASSV( 
SUBCLASSV[1:0] ), .R_( R_[2:0] ), .DEV_clk( DEV_clk ), .MF_clk( 
MF_clk_rt ), .AS_CLK_RSTb_sub1( AS_CLK_RSTb_sub1 ), .CLK_RSTb_sub1( 
CLK_RSTb_sub1 ), .alert_en( alert_en ), .SYSREF_dis( RST_dis ), .gnd( 
gnd ), .SYSREF( SYSREF ), .vdd( vdd ) );
 
MF_cntr I0 ( .MF_RSTb( 
MF_RSTb ), .FRM_clk( FRM_clk ), .K_( K_[4:0] ), .MF_end( MF_end_rt ), 
.MF_clk_tm1b( MF_clk_tm1b ), .MF_clk( MF_clk_rt ), .gnd( gnd ), .vdd( 
vdd ) );

endmodule

// 
// Design File for: (ltc2123 ser_div2_div1_std schematic)
//
// Verilog-AMS netlist generated by the AMS netlister, version 5.10.41.500.6.143.
// Cadence Design Systems, Inc.

 
module ser_div2_div1_std ( q,vdd,gnd,.clkin(
ckp),div2,setn ); 

output   q;
inout   vdd;
inout   gnd;
input   ckp;
input   div2;

input   setn;

wire q;
wire setn;
wire div2n;
wire ckp;
wire net0261;
wire net0337;
wire div2;
wire gnd;
wire net0157;
wire vdd;
 
 
 
INVX4 I53
 ( .Y( div2n ), .A( div2 ) );
 
INVX8 I56
 ( .Y( q ), .A( net0261 ) );
 
MXI2X4 I54
 ( .Y( net0261 ), .S0( div2n ), .A( net0337 ), .B( ckp ) );
 
DFFSX4 I52
 ( .QN( net0157 ), .SN( setn ), .CK( ckp ), .D( net0157 ), .Q( net0337 ) );

endmodule

// 
// Design File for: (tsmc18 MXI2X4 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module MXI2X4 (Y, A, B, S0);
output Y;
input A, B, S0;

  udp_mux2 u0(YN, A, B, S0);
  not      u1(Y, YN);

  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05,
      tplh$B$Y = 0.05,
      tphl$B$Y = 0.05,
      tplh$S0$Y = 0.05,
      tphl$S0$Y = 0.05;
    // path delays
     if ((A == 1'b1) && (B == 1'b0))
        (S0 *> Y) = (tplh$S0$Y, tphl$S0$Y);
     if ((A == 1'b0) && (B == 1'b1))
        (S0 *> Y) = (tplh$S0$Y, tphl$S0$Y);
     (B *> Y) = (tplh$B$Y, tphl$B$Y);
     (A *> Y) = (tplh$A$Y, tphl$A$Y);
  endspecify

endmodule // MXI2X4

`endcelldefine

// 
// Design File for: (tsmc18 DFFSX4 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module DFFSX4 (Q, QN, D, CK, SN);
output Q, QN;
input  D, CK, SN;
reg NOTIFIER;
supply1 xRN;

  buf   XX0 (xSN, SN);
  buf     IC (clk, CK);
  udp_dff I0 (n0, D, clk, xRN, xSN, NOTIFIER);
  and     I4 (flag, xRN, xSN);
  buf     I1 (Q, n0);
  not     I2 (QN, n0);

  specify
    specparam
    tplh$SN$Q  = 0.05,
    tphl$SN$Q  = 0.05,
    tplh$SN$QN  = 0.05,
    tphl$SN$QN  = 0.05,
    tplh$CK$Q   = 0.05,
    tphl$CK$Q   = 0.05,
    tplh$CK$QN  = 0.05,
    tphl$CK$QN  = 0.05,
    tsetup$D$CK = 0.05,
    thold$D$CK  = 0.02,
    tsetup$SN$CK    = 0.05,
    thold$SN$CK    = 0.02,
    tminpwl$SN     = 0.05,
    tminpwl$CK    = 0.05,
    tminpwh$CK    = 0.05;


    if (flag)
      (posedge CK *> (Q +: D)) = (tplh$CK$Q,    tphl$CK$Q);
    if (flag)
      (posedge CK *> (QN -: D)) = (tplh$CK$QN,   tphl$CK$QN);
    $setuphold(posedge CK &&& (flag == 1), posedge D, tsetup$D$CK, thold$D$CK, NOTIFIER);
    $setuphold(posedge CK &&& (flag == 1), negedge D, tsetup$D$CK, thold$D$CK, NOTIFIER);
      (negedge SN *> (Q +: 1'b1)) = (tphl$SN$Q);
      (posedge SN *> (Q -: 1'b1)) = (tphl$SN$Q);
      (negedge SN *> (QN -: 1'b1)) = (tplh$SN$QN);
      (posedge SN *> (QN +: 1'b1)) = (tplh$SN$QN);
    $setuphold(posedge CK, posedge SN, tsetup$SN$CK, thold$SN$CK, NOTIFIER);
    $width(negedge SN, tminpwl$SN, 0, NOTIFIER);
    $width(negedge CK &&& (flag == 1), tminpwl$CK, 0, NOTIFIER);
    $width(posedge CK &&& (flag == 1), tminpwh$CK, 0, NOTIFIER); 

   endspecify
endmodule // DFFSX4

`endcelldefine

// 
// Design File for: (tsmc18 DFFNX2 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module DFFNX2 (Q, QN, D, CKN);
output Q, QN;
input  D, CKN;
reg NOTIFIER;
supply1 xSN,xRN;
  not     IC (clk, CKN);
  udp_dff I0 (n0, D, clk, xRN, xSN, NOTIFIER);
  and     I4 (flag, xRN, xSN);
  buf     I1 (Q, n0);
  not     I2 (QN, n0);

  specify
    specparam
    tplh$CKN$Q  = 0.05,
    tphl$CKN$Q  = 0.05,
    tplh$CKN$QN = 0.05,
    tphl$CKN$QN = 0.05,
    tsetup$D$CKN        = 0.05,
    thold$D$CKN = 0.02,
    tminpwl$CKN    = 0.05,
    tminpwh$CKN    = 0.05;

    if (flag)
      (negedge CKN *> (Q    +: D)) = (tplh$CKN$Q,    tphl$CKN$Q);
    if (flag)
      (negedge CKN *> (QN -: D)) = (tplh$CKN$QN, tphl$CKN$QN);
    $setuphold(negedge CKN &&& (flag == 1), posedge D, tsetup$D$CKN, thold$D$CKN, NOTIFIER);
    $setuphold(negedge CKN &&& (flag == 1), negedge D, tsetup$D$CKN, thold$D$CKN, NOTIFIER);
    $width(negedge CKN &&& (flag == 1), tminpwl$CKN, 0, NOTIFIER);
    $width(posedge CKN &&& (flag == 1), tminpwh$CKN, 0, NOTIFIER); 

   endspecify
endmodule // DFFNX2

`endcelldefine

// 
// Design File for: (tsmc18 DFFNRX2 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module DFFNRX2 (Q, QN, D, CKN, RN);
output Q, QN;
input  D, CKN, RN;
reg NOTIFIER;
supply1 xSN;

  buf   XX0 (xRN, RN);
  not     IC (clk, CKN);
  udp_dff I0 (n0, D, clk, xRN, xSN, NOTIFIER);
  and     I4 (flag, xRN, xSN);
  buf     I1 (Q, n0);
  not     I2 (QN, n0);

  specify
    specparam
    tplh$RN$Q  = 0.05,
    tphl$RN$Q  = 0.05,
    tplh$RN$QN  = 0.05,
    tphl$RN$QN  = 0.05,
    tplh$CKN$Q  = 0.05,
    tphl$CKN$Q  = 0.05,
    tplh$CKN$QN = 0.05,
    tphl$CKN$QN = 0.05,
    tsetup$D$CKN        = 0.05,
    thold$D$CKN = 0.02,
    tsetup$RN$CKN    = 0.05,
    thold$RN$CKN    = 0.02,
    tminpwl$RN     = 0.05,
    tminpwl$CKN    = 0.05,
    tminpwh$CKN    = 0.05;

    if (flag)
      (negedge CKN *> (Q    +: D)) = (tplh$CKN$Q,    tphl$CKN$Q);
    if (flag)
      (negedge CKN *> (QN -: D)) = (tplh$CKN$QN, tphl$CKN$QN);
    $setuphold(negedge CKN &&& (flag == 1), posedge D, tsetup$D$CKN, thold$D$CKN, NOTIFIER);
    $setuphold(negedge CKN &&& (flag == 1), negedge D, tsetup$D$CKN, thold$D$CKN, NOTIFIER);
      (negedge RN *> (Q +: 1'b0)) = (tphl$RN$Q);
      (negedge RN *> (QN -: 1'b0)) = (tplh$RN$QN);
    $setuphold(negedge CKN, posedge RN , tsetup$RN$CKN, thold$RN$CKN, NOTIFIER);
    $width(negedge RN , tminpwl$RN, 0, NOTIFIER);
    $width(negedge CKN &&& (flag == 1), tminpwl$CKN, 0, NOTIFIER);
    $width(posedge CKN &&& (flag == 1), tminpwh$CKN, 0, NOTIFIER); 

   endspecify
endmodule // DFFNRX2

`endcelldefine

// 
// Design File for: (ltc2123 SYNC_detect schematic)
//
// Verilog-AMS netlist generated by the AMS netlister, version 5.10.41.500.6.143.
// Cadence Design Systems, Inc.

 
module SYNC_detect ( CLK_RSTb_sub2,SYNC_tm4,
gnd,vdd,DEV_clk,FRM_clk,\SYNC~ ,SUBCLASSV,RST_dis,MF_RSTb,
AS_CLK_RSTb_sub2 ); 

output   CLK_RSTb_sub2;
output   SYNC_tm4;
inout   gnd;
inout   vdd;

input   DEV_clk;
input   FRM_clk;
input   \SYNC~ ;
input [1:0] 
 SUBCLASSV;
input   RST_dis;
output   MF_RSTb;
output  
 AS_CLK_RSTb_sub2;

 
 
 
DLY4X1 I54
 ( .Y( net066 ), .A( net073 ) );
DLY4X1 I62
 ( .Y( net074 ), .A( sync_tm1 ) );
DLY4X1 I53
 ( .Y( net058 ), .A( net066 ) );
DLY4X1 I61
 ( .Y( net076 ), .A( net074 ) );
 
NOR2X1 I45
 ( .Y( sub0 ), .A( SUBCLASSV[0] ), .B( SUBCLASSV[1] ) );
 
INVX2 I55
 ( .Y( net072 ), .A( net0148 ) );
INVX2 I42
 ( .Y( net040 ), .A( net0107 ) );
 
NAND3X2 I19 ( .Y( net054 ), .A( sync_tm2 ), .B( net086 ), .C( net071 ) );
NAND3X2 I65 ( .Y( net059 ), .A( net078 ), .B( net076 ), .C( sub0 ) );
 
NOR3X1 I29
 ( .Y( sub2 ), .A( SUBCLASSV[0] ), .B( net030 ), .C( net036 ) );
 
INVX1 I37
 ( .Y( net049 ), .A( RST_dis ) );
INVX1 I38
 ( .Y( net036 ), .A( net049 ) );
INVX1 I30
 ( .Y( net030 ), .A( SUBCLASSV[1] ) );
 
NOR2X4 I21
 ( .Y( SYNC_tm4 ), .A( net089 ), .B( net054 ) );
 
INVX4 I41
 ( .Y( CLK_RSTb_sub2 ), .A( net040 ) );
INVX4 I26
 ( .Y( net047 ), .A( \SYNC~  ) );
INVX4 I63
 ( .Y( MF_RSTb ), .A( net0154 ) );
INVX4 I56
 ( .Y( AS_CLK_RSTb_sub2 ), .A( net072 ) );
 
DFFRX2 I3
 ( .QN( net073 ), .CK( DEV_clk ), .D( \SYNC~  ), .RN( sub2 ), .Q( 
net0104 ) );
DFFRX2 I39
 ( .QN( net038 ), .CK( FRM_clk ), .D( net086 ), .RN( net047 ), .Q( 
net071 ) );
DFFRX2 I15
 ( .QN( net078 ), .CK( FRM_clk ), .D( net047 ), .RN( net047 ), .Q( 
sync_tm1 ) );
DFFRX2 I17
 ( .QN( net068 ), .CK( FRM_clk ), .D( sync_tm2 ), .RN( net047 ), .Q( 
net086 ) );
DFFRX2 I16
 ( .QN( net063 ), .CK( FRM_clk ), .D( sync_tm1 ), .RN( net047 ), .Q( 
sync_tm2 ) );
DFFRX2 I34
 ( .QN( net065 ), .CK( DEV_clk ), .D( net0104 ), .RN( sub2 ), .Q( 
net029 ) );
 
NAND2X2 I64 ( .Y( net0154 ), .A( net0148 ), .B( net059 ) );
NAND2X2 I52 ( .Y( net0148 ), .A( net0104 ), .B( net058 ) );
NAND2X2 I20 ( .Y( net089 ), .A( net047 ), .B( sync_tm1 ) );
NAND2X2 I31 ( .Y( net0107 ), .A( net0104 ), .B( net065 ) );

endmodule

// 
// Design File for: (tsmc18 NOR2X1 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module NOR2X1 (Y, A, B);
output Y;
input A, B;

  nor (Y, A, B);

  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05,
      tplh$B$Y = 0.05,
      tphl$B$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
    (B *> Y) = (tplh$B$Y, tphl$B$Y);
  endspecify

endmodule // NOR2X1

`endcelldefine

// 
// Design File for: (tsmc18 NAND3X2 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module NAND3X2 (Y, A, B, C);
output Y;
input A, B, C;

  nand (Y, A, B, C);

  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05,
      tplh$B$Y = 0.05,
      tphl$B$Y = 0.05,
      tplh$C$Y = 0.05,
      tphl$C$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
    (B *> Y) = (tplh$B$Y, tphl$B$Y);
    (C *> Y) = (tplh$C$Y, tphl$C$Y);
  endspecify

endmodule // NAND3X2

`endcelldefine

// 
// Design File for: (tsmc18 NOR3X1 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module NOR3X1 (Y, A, B, C);
output Y;
input A, B, C;

  nor (Y, A, B, C);

  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05,
      tplh$B$Y = 0.05,
      tphl$B$Y = 0.05,
      tplh$C$Y = 0.05,
      tphl$C$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
    (B *> Y) = (tplh$B$Y, tphl$B$Y);
    (C *> Y) = (tplh$C$Y, tphl$C$Y);
  endspecify

endmodule // NOR3X1

`endcelldefine

// 
// Design File for: (tsmc18 NOR2X4 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module NOR2X4 (Y, A, B);
output Y;
input A, B;

  nor (Y, A, B);

  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05,
      tplh$B$Y = 0.05,
      tphl$B$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
    (B *> Y) = (tplh$B$Y, tphl$B$Y);
  endspecify

endmodule // NOR2X4

`endcelldefine

// 
// Design File for: (ltc2123 SYSREF_detect schematic)
//
// Verilog-AMS netlist generated by the AMS netlister, version 5.10.41.500.6.143.
// Cadence Design Systems, Inc.

 
module SYSREF_detect ( gnd,vdd,DEV_clk,MF_clk,
SYSREF,alert_en,SUBCLASSV,CLK_RSTb_sub1,R_,SYSREF_dis,AS_CLK_RSTb_sub1 ); 

inout   gnd;
inout   vdd;
input   DEV_clk;
input   MF_clk;
input  
 SYSREF;
input   alert_en;
input [1:0]  SUBCLASSV;
output  
 CLK_RSTb_sub1;
input [2:0]  R_;
input   SYSREF_dis;
output  
 AS_CLK_RSTb_sub1; 
 
DFFRX4 I3
 ( .QN( net085 ), .CK( DEV_clk ), .D( SYSREF ), .RN( sub1_en ), .Q( 
sys_rt1 ) );
 
DLY4X1 I160
 ( .Y( net098 ), .A( net085 ) );
DLY4X1 I157
 ( .Y( net087 ), .A( net098 ) );
 
NOR3X2 I16
 ( .Y( sub1_en ), .A( net063 ), .B( SUBCLASSV[1] ), .C( net0279 ) );
 
TIELO I149
 ( .Y( net0118 ) );
 
AND3X1 I139
 ( .Y( net0151 ), .A( R0 ), .B( R1b ), .C( R2 ) );
AND3X1 I137
 ( .Y( net0157 ), .A( R0 ), .B( R1 ), .C( R2b ) );
AND3X1 I140
 ( .Y( net073 ), .A( R0b ), .B( R1 ), .C( R2 ) );
AND3X1 I135
 ( .Y( net0166 ), .A( R0b ), .B( R1b ), .C( R2b ) );
AND3X1 I136
 ( .Y( net071 ), .A( R0 ), .B( R1b ), .C( R2b ) );
AND3X1 I141
 ( .Y( net075 ), .A( R0b ), .B( R1b ), .C( R2 ) );
AND3X1 I142
 ( .Y( net0172 ), .A( R0 ), .B( R1 ), .C( R2 ) );
AND3X1 I138
 ( .Y( net0160 ), .A( R0b ), .B( R1 ), .C( R2b ) );
 
NAND2X1 I127 ( .Y( ld3b ), .A( ld_cnt ), .B( net0157 ) );
NAND2X1 I128 ( .Y( ld2b ), .A( ld_cnt ), .B( net0160 ) );
NAND2X1 I129 ( .Y( ld6b ), .A( ld_cnt ), .B( net073 ) );
NAND2X1 I131 ( .Y( ld5b ), .A( ld_cnt ), .B( net0151 ) );
NAND2X1 I130 ( .Y( ld7b ), .A( ld_cnt ), .B( net0172 ) );
NAND2X1 I132 ( .Y( ld4b ), .A( ld_cnt ), .B( net075 ) );
NAND2X1 I125 ( .Y( ld0b ), .A( ld_cnt ), .B( net0166 ) );
NAND2X1 I126 ( .Y( ld1b ), .A( ld_cnt ), .B( net071 ) );
 
DFFSRX1 I109 ( .QN( net0199 ), .SN( ld4b ), .CK( MF_clock ), .D( Q5 ), .RN( 
load_cntb ), .Q( Q4 ) );
DFFSRX1 I110 ( .QN( net0244 ), .SN( ld5b ), .CK( MF_clock ), .D( Q6 ), .RN( 
load_cntb ), .Q( Q5 ) );
DFFSRX1 I107 ( .QN( net0251 ), .SN( ld7b ), .CK( MF_clock ), .D( net0118 ), 
.RN( load_cntb ), .Q( Q7 ) );
DFFSRX1 I111 ( .QN( net0181 ), .SN( ld1b ), .CK( MF_clock ), .D( Q2 ), .RN( 
load_cntb ), .Q( Q1 ) );
DFFSRX1 I114 ( .QN( net0250 ), .SN( ld3b ), .CK( MF_clock ), .D( Q4 ), .RN( 
load_cntb ), .Q( Q3 ) );
DFFSRX1 I112 ( .QN( net0175 ), .SN( ld0b ), .CK( MF_clock ), .D( Q1 ), .RN( 
load_cntb ), .Q( Q0 ) );
DFFSRX1 I113 ( .QN( net0245 ), .SN( ld2b ), .CK( MF_clock ), .D( Q3 ), .RN( 
load_cntb ), .Q( Q2 ) );
DFFSRX1 I108 ( .QN( net0246 ), .SN( ld6b ), .CK( MF_clock ), .D( Q7 ), .RN( 
load_cntb ), .Q( Q6 ) );
 
INVX4 I91
 ( .Y( MF_clock ), .A( net0230 ) );
INVX4 I133
 ( .Y( load_cntb ), .A( load_cnt ) );
INVX4 I134
 ( .Y( ld_cnt ), .A( load_cntb ) );
 
NAND2X2 I155 ( .Y( net0204 ), .A( sys_rt1 ), .B( net087 ) );
NAND2X2 I90 ( .Y( net0230 ), .A( MF_clk ), .B( alert_mode ) );
NAND2X2 I85 ( .Y( blockb ), .A( block_rt1 ), .B( block_rt0 ) );
NAND2X2 I63 ( .Y( load_cnt ), .A( sysref_detb ), .B( alert_mode ) );
NAND2X2 I33 ( .Y( net090 ), .A( alert_en ), .B( sub1_en ) );
 
AND2X2 I97
 ( .Y( block1sysb ), .A( blockb ), .B( alert_mode ) );
 
TIEHI I76
 ( .Y( net0114 ) );
 
NAND2X4 I158 ( .Y( AS_CLK_RSTb_sub1 ), .A( net0296 ), .B( MF_RST_en ) );
NAND2X4 I78 ( .Y( CLK_RSTb_sub1 ), .A( sysref_det ), .B( MF_RST_en ) );
NAND2X4 I96 ( .Y( MF_RST_en ), .A( alert_mode ), .B( blocked ) );
NAND2X4 I74 ( .Y( sysref_detb ), .A( sys_rt1 ), .B( sys_rt0 ) );
 
DFFRX2 I151
 ( .QN( net0234 ), .CK( MF_clock ), .D( block1 ), .RN( load_cntb ), .Q( 
block0 ) );
DFFRX2 I94
 ( .QN( block_rt0 ), .CK( DEV_clk ), .D( block_rt1 ), .RN( alert_mode ), 
.Q( net0195 ) );
DFFRX2 I86
 ( .QN( net0167 ), .CK( DEV_clk ), .D( block0 ), .RN( alert_mode ), .Q( 
block_rt1 ) );
DFFRX2 I73
 ( .QN( sys_rt0 ), .CK( DEV_clk ), .D( sys_rt1 ), .RN( sub1_en ), .Q( 
net0163 ) );
DFFRX2 I123
 ( .QN( net0274 ), .CK( MF_clock ), .D( Q0 ), .RN( load_cntb ), .Q( 
block1 ) );
DFFRX2 I75
 ( .QN( blocked ), .CK( sysref_detb ), .D( net0114 ), .RN( block1sysb ), 
.Q( net0191 ) );
 
INVX1 I153
 ( .Y( net0281 ), .A( SYSREF_dis ) );
INVX1 I146
 ( .Y( R2 ), .A( R2b ) );
INVX1 I148
 ( .Y( R0 ), .A( R0b ) );
INVX1 I144
 ( .Y( R1b ), .A( R_[1] ) );
INVX1 I145
 ( .Y( R2b ), .A( R_[2] ) );
INVX1 I154
 ( .Y( net0279 ), .A( net0281 ) );
INVX1 I143
 ( .Y( R0b ), .A( R_[0] ) );
INVX1 I147
 ( .Y( R1 ), .A( R1b ) );
INVX1 I17
 ( .Y( net063 ), .A( SUBCLASSV[0] ) );
 
INVX2 I159
 ( .Y( net0296 ), .A( net0204 ) );
INVX2 I34
 ( .Y( alert_mode ), .A( net090 ) );
INVX2 I77
 ( .Y( sysref_det ), .A( sysref_detb ) );

endmodule

// 
// Design File for: (tsmc18 DFFRX4 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module DFFRX4 (Q, QN, D, CK, RN);
output Q, QN;
input  D, CK, RN;
reg NOTIFIER;
supply1 xSN;

  buf   XX0 (xRN, RN);
  buf     IC (clk, CK);
  udp_dff I0 (n0, D, clk, xRN, xSN, NOTIFIER);
  and     I4 (flag, xRN, xSN);
  buf     I1 (Q, n0);
  not     I2 (QN, n0);

  specify
    specparam
    tplh$RN$Q  = 0.05,
    tphl$RN$Q  = 0.05,
    tplh$RN$QN  = 0.05,
    tphl$RN$QN  = 0.05,
    tplh$CK$Q   = 0.05,
    tphl$CK$Q   = 0.05,
    tplh$CK$QN  = 0.05,
    tphl$CK$QN  = 0.05,
    tsetup$D$CK = 0.05,
    thold$D$CK  = 0.02,
    tsetup$RN$CK    = 0.05,
    thold$RN$CK    = 0.02,
    tminpwl$RN     = 0.05,
    tminpwl$CK    = 0.05,
    tminpwh$CK    = 0.05;


    if (flag)
      (posedge CK *> (Q +: D)) = (tplh$CK$Q,    tphl$CK$Q);
    if (flag)
      (posedge CK *> (QN -: D)) = (tplh$CK$QN,   tphl$CK$QN);
    $setuphold(posedge CK &&& (flag == 1), posedge D, tsetup$D$CK, thold$D$CK, NOTIFIER);
    $setuphold(posedge CK &&& (flag == 1), negedge D, tsetup$D$CK, thold$D$CK, NOTIFIER);
      (negedge RN *> (Q +: 1'b0)) = (tphl$RN$Q);
      (negedge RN *> (QN -: 1'b0)) = (tplh$RN$QN);
    $setuphold(posedge CK, posedge RN, tsetup$RN$CK, thold$RN$CK, NOTIFIER);
    $width(negedge RN, tminpwl$RN, 0, NOTIFIER);
    $width(negedge CK &&& (flag == 1), tminpwl$CK, 0, NOTIFIER);
    $width(posedge CK &&& (flag == 1), tminpwh$CK, 0, NOTIFIER); 

   endspecify
endmodule // DFFRX4

`endcelldefine

// 
// Design File for: (tsmc18 NOR3X2 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module NOR3X2 (Y, A, B, C);
output Y;
input A, B, C;

  nor (Y, A, B, C);

  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05,
      tplh$B$Y = 0.05,
      tphl$B$Y = 0.05,
      tplh$C$Y = 0.05,
      tphl$C$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
    (B *> Y) = (tplh$B$Y, tphl$B$Y);
    (C *> Y) = (tplh$C$Y, tphl$C$Y);
  endspecify

endmodule // NOR3X2

`endcelldefine

// 
// Design File for: (tsmc18 AND3X1 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module AND3X1 (Y, A, B, C);
output Y;
input A, B, C;

  and (Y, A, B, C);

  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05,
      tplh$B$Y = 0.05,
      tphl$B$Y = 0.05,
      tplh$C$Y = 0.05,
      tphl$C$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
    (B *> Y) = (tplh$B$Y, tphl$B$Y);
    (C *> Y) = (tplh$C$Y, tphl$C$Y);
  endspecify

endmodule // AND3X1

`endcelldefine

// 
// Design File for: (tsmc18 NAND2X1 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module NAND2X1 (Y, A, B);
output Y;
input A, B;

  nand (Y, A, B);

  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05,
      tplh$B$Y = 0.05,
      tphl$B$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
    (B *> Y) = (tplh$B$Y, tphl$B$Y);
  endspecify

endmodule // NAND2X1

`endcelldefine

// 
// Design File for: (tsmc18 DFFSRX1 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module DFFSRX1 (Q, QN, D, CK, SN, RN);
output Q, QN;
input  D, CK, SN, RN;
reg NOTIFIER;
  buf   XX0 (xSN, SN);
  buf   XX1 (xRN, RN);
  buf     IC (clk, CK);
  udp_dff I0 (n0, D, clk, xRN, xSN, NOTIFIER);
  and     I4 (flag, xRN, xSN);
  buf     I1 (Q, n0);
  not     I2 (QN, n0);

  specify
    specparam
    tplh$SN$Q  = 0.05,
    tphl$SN$Q  = 0.05,
    tplh$SN$QN  = 0.05,
    tphl$SN$QN  = 0.05,
    tplh$RN$Q  = 0.05,
    tphl$RN$Q  = 0.05,
    tplh$RN$QN  = 0.05,
    tphl$RN$QN  = 0.05,
    tplh$CK$Q   = 0.05,
    tphl$CK$Q   = 0.05,
    tplh$CK$QN  = 0.05,
    tphl$CK$QN  = 0.05,
    tsetup$D$CK = 0.05,
    thold$D$CK  = 0.02,
    tsetup$SN$CK    = 0.05,
    thold$SN$CK    = 0.02,
    tminpwl$SN     = 0.05,
    tsetup$RN$CK    = 0.05,
    thold$RN$CK    = 0.02,
    tminpwl$RN     = 0.05,
    thold$RN$SN = 0.05,
    thold$SN$RN = 0.05,
    tminpwl$CK    = 0.05,
    tminpwh$CK    = 0.05;


    if (flag)
      (posedge CK *> (Q +: D)) = (tplh$CK$Q,    tphl$CK$Q);
    if (flag)
      (posedge CK *> (QN -: D)) = (tplh$CK$QN,   tphl$CK$QN);
    $setuphold(posedge CK &&& (flag == 1), posedge D, tsetup$D$CK, thold$D$CK, NOTIFIER);
    $setuphold(posedge CK &&& (flag == 1), negedge D, tsetup$D$CK, thold$D$CK, NOTIFIER);
      (negedge RN *> (Q +: 1'b0)) = (tphl$RN$Q);
      (negedge RN *> (QN -: 1'b0)) = (tplh$RN$QN);
    $setuphold(posedge CK, posedge RN &&& (SN == 1'b1), tsetup$RN$CK, thold$RN$CK, NOTIFIER);
    $width(negedge RN &&& (SN == 1'b1), tminpwl$RN, 0, NOTIFIER);
      (negedge SN *> (Q +: 1'b1)) = (tphl$SN$Q);
      (posedge SN *> (Q -: 1'b1)) = (tphl$SN$Q);
      (negedge SN *> (QN -: 1'b1)) = (tplh$SN$QN);
      (posedge SN *> (QN +: 1'b1)) = (tplh$SN$QN);
    $setuphold(posedge CK, posedge SN, tsetup$SN$CK, thold$SN$CK, NOTIFIER);
    $width(negedge SN, tminpwl$SN, 0, NOTIFIER);
    $width(negedge CK &&& (flag == 1), tminpwl$CK, 0, NOTIFIER);
    $width(posedge CK &&& (flag == 1), tminpwh$CK, 0, NOTIFIER); 
    $hold(posedge RN, posedge SN, thold$SN$RN, NOTIFIER);    
    $hold(posedge SN, posedge RN, thold$RN$SN, NOTIFIER);    

   endspecify
endmodule // DFFSRX1

`endcelldefine

// 
// Design File for: (tsmc18 AND2X2 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module AND2X2 (Y, A, B);
output Y;
input A, B;

  and (Y, A, B);

  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05,
      tplh$B$Y = 0.05,
      tphl$B$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
    (B *> Y) = (tplh$B$Y, tphl$B$Y);
  endspecify

endmodule // AND2X2

`endcelldefine

// 
// Design File for: (tsmc18 TIEHI verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module TIEHI (Y);
output Y;

  buf I0(Y, 1'b1);

endmodule //TIEHI 

`endcelldefine

// 
// Design File for: (ltc2123 MF_cntr schematic)
//
// Verilog-AMS netlist generated by the AMS netlister, version 5.10.41.500.6.143.
// Cadence Design Systems, Inc.

 
module MF_cntr ( MF_clk,gnd,vdd,FRM_clk,K_,
MF_RSTb,MF_end,MF_clk_tm1b ); 

output   MF_clk;
inout   gnd;
inout   vdd;
input   FRM_clk;

input [4:0]  K_;
input   MF_RSTb;
output   MF_end;
output  
 MF_clk_tm1b; 
 
DFFX2 I81
 ( .QN( net230 ), .CK( FRM_clk ), .D( net123 ), .Q( net121 ) );
 
DFFSX2 I9
 ( .QN( net233 ), .SN( MF_RSTb ), .CK( FRM_clk ), .D( net179 ), .Q( 
net194 ) );
 
INVX2 I62
 ( .Y( net129 ), .A( net178 ) );
 
DFFRX2 I46
 ( .QN( net132 ), .CK( FRM_clk ), .D( net129 ), .RN( MF_RSTb ), .Q( 
net123 ) );
DFFRX2 I8
 ( .QN( net234 ), .CK( FRM_clk ), .D( net175 ), .RN( MF_RSTb ), .Q( 
net190 ) );
DFFRX2 I7
 ( .QN( net231 ), .CK( FRM_clk ), .D( net172 ), .RN( MF_RSTb ), .Q( 
net202 ) );
DFFRX2 I6
 ( .QN( net236 ), .CK( FRM_clk ), .D( net169 ), .RN( MF_RSTb ), .Q( 
net186 ) );
DFFRX2 I5
 ( .QN( net152 ), .CK( FRM_clk ), .D( net181 ), .RN( MF_RSTb ), .Q( 
net198 ) );
 
INVX4 I82
 ( .Y( MF_clk_tm1b ), .A( net121 ) );
INVX4 I51
 ( .Y( net178 ), .A( net213 ) );
INVX4 I47
 ( .Y( net160 ), .A( net123 ) );
INVX4 I63
 ( .Y( net162 ), .A( net129 ) );
 
INVX8 I64
 ( .Y( MF_end ), .A( net162 ) );
INVX8 I38
 ( .Y( MF_clk ), .A( net160 ) );
 
AND2X1 I41
 ( .Y( net169 ), .A( cnt3 ), .B( net178 ) );
AND2X1 I42
 ( .Y( net172 ), .A( cnt2 ), .B( net178 ) );
AND2X1 I43
 ( .Y( net175 ), .A( cnt1 ), .B( net178 ) );
AND2X1 I44
 ( .Y( net179 ), .A( cnt0 ), .B( net178 ) );
AND2X1 I40
 ( .Y( net181 ), .A( cnt4 ), .B( net178 ) );
 
AHHCONX2 I2 ( .CI( net184 ), .A( net186 ), .CON( net196 ), .S( cnt3 ) );
AHHCONX2 I1 ( .CI( net188 ), .A( net190 ), .CON( net200 ), .S( cnt1 ) );
 
AHHCINX2 I4 ( .CIN( net228 ), .A( net194 ), .CO( net188 ), .S( cnt0 ) );
AHHCINX2 I3 ( .CIN( net196 ), .A( net198 ), .CO( net195 ), .S( cnt4 ) );
AHHCINX2 I0 ( .CIN( net200 ), .A( net202 ), .CO( net184 ), .S( cnt2 ) );
 
AFHCONX2 I13 ( .CI( net204 ), .A( net186 ), .B( net245 ), .CON( net214 ), .S( 
net240 ) );
AFHCONX2 I11 ( .CI( net209 ), .A( net190 ), .B( net247 ), .CON( net219 ), .S( 
net238 ) );
 
AFHCINX2 I14 ( .CIN( net214 ), .CO( net213 ), .A( net198 ), .B( net251 ), .S( 
net242 ) );
AFHCINX2 I12 ( .CIN( net219 ), .CO( net204 ), .A( net202 ), .B( net249 ), .S( 
net239 ) );
AFHCINX2 I10 ( .CIN( net229 ), .CO( net209 ), .A( net194 ), .B( net243 ), .S( 
net241 ) );
 
TIELO I26
 ( .Y( net228 ) );
TIELO I27
 ( .Y( net229 ) );
 
INVX1 I19
 ( .Y( net243 ), .A( K_[0] ) );
INVX1 I16
 ( .Y( net245 ), .A( K_[3] ) );
INVX1 I18
 ( .Y( net247 ), .A( K_[1] ) );
INVX1 I17
 ( .Y( net249 ), .A( K_[2] ) );
INVX1 I15
 ( .Y( net251 ), .A( K_[4] ) );

endmodule

// 
// Design File for: (tsmc18 DFFX2 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module DFFX2 (Q, QN, D, CK);
output Q, QN;
input  D, CK;
reg NOTIFIER;
supply1 xSN,xRN;
  buf     IC (clk, CK);
  udp_dff I0 (n0, D, clk, xRN, xSN, NOTIFIER);
  and     I4 (flag, xRN, xSN);
  buf     I1 (Q, n0);
  not     I2 (QN, n0);

  specify
    specparam
    tplh$CK$Q   = 0.05,
    tphl$CK$Q   = 0.05,
    tplh$CK$QN  = 0.05,
    tphl$CK$QN  = 0.05,
    tsetup$D$CK = 0.05,
    thold$D$CK  = 0.02,
    tminpwl$CK    = 0.05,
    tminpwh$CK    = 0.05;


    if (flag)
      (posedge CK *> (Q +: D)) = (tplh$CK$Q,    tphl$CK$Q);
    if (flag)
      (posedge CK *> (QN -: D)) = (tplh$CK$QN,   tphl$CK$QN);
    $setuphold(posedge CK &&& (flag == 1), posedge D, tsetup$D$CK, thold$D$CK, NOTIFIER);
    $setuphold(posedge CK &&& (flag == 1), negedge D, tsetup$D$CK, thold$D$CK, NOTIFIER);
    $width(negedge CK &&& (flag == 1), tminpwl$CK, 0, NOTIFIER);
    $width(posedge CK &&& (flag == 1), tminpwh$CK, 0, NOTIFIER); 

   endspecify
endmodule // DFFX2

`endcelldefine

// 
// Design File for: (tsmc18 DFFSX2 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module DFFSX2 (Q, QN, D, CK, SN);
output Q, QN;
input  D, CK, SN;
reg NOTIFIER;
supply1 xRN;

  buf   XX0 (xSN, SN);
  buf     IC (clk, CK);
  udp_dff I0 (n0, D, clk, xRN, xSN, NOTIFIER);
  and     I4 (flag, xRN, xSN);
  buf     I1 (Q, n0);
  not     I2 (QN, n0);

  specify
    specparam
    tplh$SN$Q  = 0.05,
    tphl$SN$Q  = 0.05,
    tplh$SN$QN  = 0.05,
    tphl$SN$QN  = 0.05,
    tplh$CK$Q   = 0.05,
    tphl$CK$Q   = 0.05,
    tplh$CK$QN  = 0.05,
    tphl$CK$QN  = 0.05,
    tsetup$D$CK = 0.05,
    thold$D$CK  = 0.02,
    tsetup$SN$CK    = 0.05,
    thold$SN$CK    = 0.02,
    tminpwl$SN     = 0.05,
    tminpwl$CK    = 0.05,
    tminpwh$CK    = 0.05;


    if (flag)
      (posedge CK *> (Q +: D)) = (tplh$CK$Q,    tphl$CK$Q);
    if (flag)
      (posedge CK *> (QN -: D)) = (tplh$CK$QN,   tphl$CK$QN);
    $setuphold(posedge CK &&& (flag == 1), posedge D, tsetup$D$CK, thold$D$CK, NOTIFIER);
    $setuphold(posedge CK &&& (flag == 1), negedge D, tsetup$D$CK, thold$D$CK, NOTIFIER);
      (negedge SN *> (Q +: 1'b1)) = (tphl$SN$Q);
      (posedge SN *> (Q -: 1'b1)) = (tphl$SN$Q);
      (negedge SN *> (QN -: 1'b1)) = (tplh$SN$QN);
      (posedge SN *> (QN +: 1'b1)) = (tplh$SN$QN);
    $setuphold(posedge CK, posedge SN, tsetup$SN$CK, thold$SN$CK, NOTIFIER);
    $width(negedge SN, tminpwl$SN, 0, NOTIFIER);
    $width(negedge CK &&& (flag == 1), tminpwl$CK, 0, NOTIFIER);
    $width(posedge CK &&& (flag == 1), tminpwh$CK, 0, NOTIFIER); 

   endspecify
endmodule // DFFSX2

`endcelldefine

// 
// Design File for: (tsmc18 DFFRX2 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module DFFRX2 (Q, QN, D, CK, RN);
output Q, QN;
input  D, CK, RN;
reg NOTIFIER;
supply1 xSN;

  buf   XX0 (xRN, RN);
  buf     IC (clk, CK);
  udp_dff I0 (n0, D, clk, xRN, xSN, NOTIFIER);
  and     I4 (flag, xRN, xSN);
  buf     I1 (Q, n0);
  not     I2 (QN, n0);

  specify
    specparam
    tplh$RN$Q  = 0.05,
    tphl$RN$Q  = 0.05,
    tplh$RN$QN  = 0.05,
    tphl$RN$QN  = 0.05,
    tplh$CK$Q   = 0.05,
    tphl$CK$Q   = 0.05,
    tplh$CK$QN  = 0.05,
    tphl$CK$QN  = 0.05,
    tsetup$D$CK = 0.05,
    thold$D$CK  = 0.02,
    tsetup$RN$CK    = 0.05,
    thold$RN$CK    = 0.02,
    tminpwl$RN     = 0.05,
    tminpwl$CK    = 0.05,
    tminpwh$CK    = 0.05;


    if (flag)
      (posedge CK *> (Q +: D)) = (tplh$CK$Q,    tphl$CK$Q);
    if (flag)
      (posedge CK *> (QN -: D)) = (tplh$CK$QN,   tphl$CK$QN);
    $setuphold(posedge CK &&& (flag == 1), posedge D, tsetup$D$CK, thold$D$CK, NOTIFIER);
    $setuphold(posedge CK &&& (flag == 1), negedge D, tsetup$D$CK, thold$D$CK, NOTIFIER);
      (negedge RN *> (Q +: 1'b0)) = (tphl$RN$Q);
      (negedge RN *> (QN -: 1'b0)) = (tplh$RN$QN);
    $setuphold(posedge CK, posedge RN, tsetup$RN$CK, thold$RN$CK, NOTIFIER);
    $width(negedge RN, tminpwl$RN, 0, NOTIFIER);
    $width(negedge CK &&& (flag == 1), tminpwl$CK, 0, NOTIFIER);
    $width(posedge CK &&& (flag == 1), tminpwh$CK, 0, NOTIFIER); 

   endspecify
endmodule // DFFRX2

`endcelldefine

// 
// Design File for: (tsmc18 INVX8 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module INVX8 (Y, A);
output Y;
input A;

  not I0(Y, A);
  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
  endspecify

endmodule // INVX8

`endcelldefine

// 
// Design File for: (tsmc18 AND2X1 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module AND2X1 (Y, A, B);
output Y;
input A, B;

  and (Y, A, B);

  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05,
      tplh$B$Y = 0.05,
      tphl$B$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
    (B *> Y) = (tplh$B$Y, tphl$B$Y);
  endspecify

endmodule // AND2X1

`endcelldefine

// 
// Design File for: (tsmc18 AHHCONX2 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module AHHCONX2 ( S, CON, A, CI);
output S, CON;
input A, CI;
  xor I0 (S, A, CI);
  and  I1 (cout, A, CI);   
  not I2 (CON, cout);

  specify
    specparam

      tplh$A$S  = 0.05,
      tphl$A$S  = 0.05,
      tplh$A$CON  = 0.05,
      tphl$A$CON  = 0.05,
      tplh$CI$S  = 0.05,
      tphl$CI$S  = 0.05,
      tplh$CI$CON  = 0.05,
      tphl$CI$CON  = 0.05;


    if (CI == 1'b0)
        (A  *> S)  = (tplh$A$S,   tphl$A$S);
    if (CI == 1'b1)
        (A  *> S)  = (tplh$A$S,   tphl$A$S);
    if (A == 1'b1)
        (CI  *> S)  = (tplh$CI$S,   tphl$CI$S);
    if (A == 1'b0)
        (CI  *> S)  = (tplh$CI$S,   tphl$CI$S);
    (A  *> CON)  = (tplh$A$CON,   tphl$A$CON);
    (CI *> CON) = (tplh$CI$CON, tphl$CI$CON);

  endspecify
endmodule // AHHCONX2

`endcelldefine

// 
// Design File for: (tsmc18 AHHCINX2 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module AHHCINX2 ( S, CO, A, CIN);
output S, CO;
input A, CIN;
  not I0 (ci, CIN);
  xor I1 (S, A, ci);
  and  I2 (CO, A, ci);   

  specify
    specparam

      tplh$A$S  = 0.05,
      tphl$A$S  = 0.05,
      tplh$A$CO  = 0.05,
      tphl$A$CO  = 0.05,
      tplh$CIN$S  = 0.05,
      tphl$CIN$S  = 0.05,
      tplh$CIN$CO  = 0.05,
      tphl$CIN$CO  = 0.05;


    if (CIN == 1'b0)
        (A  *> S)  = (tplh$A$S,   tphl$A$S);
    if (CIN == 1'b1)
        (A  *> S)  = (tplh$A$S,   tphl$A$S);
    if (A == 1'b1)
        (CIN  *> S)  = (tplh$CIN$S,   tphl$CIN$S);
    if (A == 1'b0)
        (CIN  *> S)  = (tplh$CIN$S,   tphl$CIN$S);
    (A  *> CO)  = (tplh$A$CO,   tphl$A$CO);
    (CIN *> CO) = (tplh$CIN$CO, tphl$CIN$CO);

  endspecify
endmodule // AHHCINX2

`endcelldefine

// 
// Design File for: (tsmc18 AFHCONX2 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module AFHCONX2 ( S, CON, A, B, CI);
output S, CON;
input A, B, CI;
  xor I0 (S, A, B, CI);
  and I1 (a_and_b, A, B);
  and I2 (a_and_ci, A, CI);
  and I3 (b_and_ci, B, CI);
  or  I4 (cout, a_and_b, a_and_ci, b_and_ci);   
  not I5 (CON, cout);

  specify
    specparam

      tplh$A$S  = 0.05,
      tphl$A$S  = 0.05,
      tplh$A$CON  = 0.05,
      tphl$A$CON  = 0.05,
      tplh$B$S  = 0.05,
      tphl$B$S  = 0.05,
      tplh$B$CON  = 0.05,
      tphl$B$CON  = 0.05,
      tplh$CI$S  = 0.05,
      tphl$CI$S  = 0.05,
      tplh$CI$CON  = 0.05,
      tphl$CI$CON  = 0.05;


     if ((A == 1'b0 && B == 1'b0) || (A == 1'b1 && B == 1'b1))
        (CI *> S)  = (tplh$CI$S,  tphl$CI$S);
     if ((A == 1'b0 && B == 1'b1) || (A == 1'b1 && B == 1'b0))
        (CI *> S)  = (tplh$CI$S,  tphl$CI$S);
     if (B == 1'b0 && CI == 1'b0)
        (A  *> S)  = (tplh$A$S,   tphl$A$S);
     if (B == 1'b1 && CI == 1'b1)
        (A  *> S)  = (tplh$A$S,   tphl$A$S);
     if (B == 1'b0 && CI == 1'b1)
        (A  *> S)  = (tplh$A$S,   tphl$A$S);
     if (B == 1'b1 && CI == 1'b0)
        (A  *> S)  = (tplh$A$S,   tphl$A$S);
     if (A == 1'b0 && CI == 1'b0)
        (B  *> S)  = (tplh$B$S,   tphl$B$S);
     if (A == 1'b1 && CI == 1'b1)
        (B  *> S)  = (tplh$B$S,   tphl$B$S);
     if (A == 1'b0 && CI == 1'b1)
        (B  *> S)  = (tplh$B$S,   tphl$B$S);
     if (A == 1'b1 && CI == 1'b0)
        (B  *> S)  = (tplh$B$S,   tphl$B$S);
     (CI *> CON) = (tplh$CI$CON, tphl$CI$CON);
     if (B == 1'b1)
        (A  *> CON) = (tplh$A$CON,  tphl$A$CON);
     if (B == 1'b0)
        (A  *> CON) = (tplh$A$CON,  tphl$A$CON);
     if (A == 1'b1)
        (B  *> CON) = (tplh$B$CON,  tphl$B$CON);
     if (A == 1'b0)
        (B  *> CON) = (tplh$B$CON,  tphl$B$CON);

  endspecify
endmodule // AFHCONX2

`endcelldefine

// 
// Design File for: (tsmc18 AFHCINX2 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module AFHCINX2 ( S, CO, A, B, CIN);
output S, CO;
input A, B, CIN;
  not I0 (ci, CIN);
  xor I1 (S, A, B, ci);
  and I2 (a_and_b, A, B);
  and I3 (a_and_ci, A, ci);
  and I4 (b_and_ci, B, ci);
  or  I5 (CO, a_and_b, a_and_ci, b_and_ci);   

  specify
    specparam

      tplh$A$S  = 0.05,
      tphl$A$S  = 0.05,
      tplh$A$CO  = 0.05,
      tphl$A$CO  = 0.05,
      tplh$B$S  = 0.05,
      tphl$B$S  = 0.05,
      tplh$B$CO  = 0.05,
      tphl$B$CO  = 0.05,
      tplh$CIN$S  = 0.05,
      tphl$CIN$S  = 0.05,
      tplh$CIN$CO  = 0.05,
      tphl$CIN$CO  = 0.05;


     if ((A == 1'b0 && B == 1'b0) || (A == 1'b1 && B == 1'b1))
        (CIN *> S)  = (tplh$CIN$S,  tphl$CIN$S);
     if ((A == 1'b0 && B == 1'b1) || (A == 1'b1 && B == 1'b0))
        (CIN *> S)  = (tplh$CIN$S,  tphl$CIN$S);
     if (B == 1'b0 && CIN == 1'b0)
        (A  *> S)  = (tplh$A$S,   tphl$A$S);
     if (B == 1'b1 && CIN == 1'b1)
        (A  *> S)  = (tplh$A$S,   tphl$A$S);
     if (B == 1'b0 && CIN == 1'b1)
        (A  *> S)  = (tplh$A$S,   tphl$A$S);
     if (B == 1'b1 && CIN == 1'b0)
        (A  *> S)  = (tplh$A$S,   tphl$A$S);
     if (A == 1'b0 && CIN == 1'b0)
        (B  *> S)  = (tplh$B$S,   tphl$B$S);
     if (A == 1'b1 && CIN == 1'b1)
        (B  *> S)  = (tplh$B$S,   tphl$B$S);
     if (A == 1'b0 && CIN == 1'b1)
        (B  *> S)  = (tplh$B$S,   tphl$B$S);
     if (A == 1'b1 && CIN == 1'b0)
        (B  *> S)  = (tplh$B$S,   tphl$B$S);
     (CIN *> CO) = (tplh$CIN$CO, tphl$CIN$CO);
     if (B == 1'b1)
        (A  *> CO) = (tplh$A$CO,  tphl$A$CO);
     if (B == 1'b0)
        (A  *> CO) = (tplh$A$CO,  tphl$A$CO);
     if (A == 1'b1)
        (B  *> CO) = (tplh$B$CO,  tphl$B$CO);
     if (A == 1'b0)
        (B  *> CO) = (tplh$B$CO,  tphl$B$CO);

  endspecify
endmodule // AFHCINX2

`endcelldefine

// 
// Design File for: (tsmc18 TIELO verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module TIELO (Y);
output Y;

  buf I0(Y, 1'b0);

endmodule //TIELO 

`endcelldefine

// 
// Design File for: (ltc2124_rj retime_ff schematic)
//
// Verilog-AMS netlist generated by the AMS netlister, version 5.10.41.500.6.143.
// Cadence Design Systems, Inc.

 
module retime_ff ( DATA_A0,DATA_A1,DATA_B0,
DATA_B1,gnd,vdd,DATA_A,DATA_B,PLL_ref,FRM_clk,disable_A0,disable_A1,
disable_B0,disable_B1,two_adc_per_ln ); 

output [15:0]  DATA_A0;
output [15:0]  DATA_A1;
output [15:0] 
 DATA_B0;
output [15:0]  DATA_B1;
inout   gnd;
inout   vdd;

input [15:0]  DATA_A;
input [15:0]  DATA_B;
output   PLL_ref;
input  
 FRM_clk;
input   disable_A0;
input   disable_A1;
input   disable_B0;

input   disable_B1;
input   two_adc_per_ln;

wire [15:0] DAT_A;
wire [0:15] net0177;
wire [15:0] DAT_B;
wire [15:0] DB0N;
wire [15:0] DB0b;
wire [0:15] net61;
wire [15:0] DA0b;
wire [15:0] DB1;
wire [15:0] DAT_A0;
wire [15:0] DB0;
wire [15:0] DAB0R;
wire [15:0] DB1b;
wire [15:0] DAB0;
wire [0:15] net55;
wire [15:0] DA1b;
wire [15:0] DAT_A0N;
wire [15:0] DA0;
wire [15:0] DA1;
wire [0:15] net77;
wire [15:0] DAT_B0;
 
 
DFFNRX1 DFFNB1_15 ( .QN( 
net61[0] ), .CKN( net065 ), .D( DAT_B[15] ), .RN( net088 ), .Q( 
DB1[15] ) ),
  DFFNB1_14 ( .QN( net61[1] ), .CKN( net065 ), .D( DAT_B[14] ), .RN( 
net088 ), .Q( DB1[14] ) ),
  DFFNB1_13 ( .QN( net61[2] ), .CKN( net065 ), .D( DAT_B[13] ), .RN( 
net088 ), .Q( DB1[13] ) ),
  DFFNB1_12 ( .QN( net61[3] ), .CKN( net065 ), .D( DAT_B[12] ), .RN( 
net088 ), .Q( DB1[12] ) ),
  DFFNB1_11 ( .QN( net61[4] ), .CKN( net065 ), .D( DAT_B[11] ), .RN( 
net088 ), .Q( DB1[11] ) ),
  DFFNB1_10 ( .QN( net61[5] ), .CKN( net065 ), .D( DAT_B[10] ), .RN( 
net088 ), .Q( DB1[10] ) ),
  DFFNB1_9 ( .QN( net61[6] ), .CKN( net065 ), .D( DAT_B[9] ), .RN( 
net088 ), .Q( DB1[9] ) ),
  DFFNB1_8 ( .QN( net61[7] ), .CKN( net065 ), .D( DAT_B[8] ), .RN( 
net088 ), .Q( DB1[8] ) ),
  DFFNB1_7 ( .QN( net61[8] ), .CKN( net065 ), .D( DAT_B[7] ), .RN( 
net088 ), .Q( DB1[7] ) ),
  DFFNB1_6 ( .QN( net61[9] ), .CKN( net065 ), .D( DAT_B[6] ), .RN( 
net088 ), .Q( DB1[6] ) ),
  DFFNB1_5 ( .QN( net61[10] ), .CKN( net065 ), .D( DAT_B[5] ), .RN( 
net088 ), .Q( DB1[5] ) ),
  DFFNB1_4 ( .QN( net61[11] ), .CKN( net065 ), .D( DAT_B[4] ), .RN( 
net088 ), .Q( DB1[4] ) ),
  DFFNB1_3 ( .QN( net61[12] ), .CKN( net065 ), .D( DAT_B[3] ), .RN( 
net088 ), .Q( DB1[3] ) ),
  DFFNB1_2 ( .QN( net61[13] ), .CKN( net065 ), .D( DAT_B[2] ), .RN( 
net088 ), .Q( DB1[2] ) ),
  DFFNB1_1 ( .QN( net61[14] ), .CKN( net065 ), .D( DAT_B[1] ), .RN( 
net088 ), .Q( DB1[1] ) ),
  DFFNB1_0 ( .QN( net61[15] ), .CKN( net065 ), .D( DAT_B[0] ), .RN( 
net088 ), .Q( DB1[0] ) );
DFFNRX1 DFFNA1_15 ( .QN( 
net55[0] ), .CKN( net059 ), .D( DAT_A[15] ), .RN( net045 ), .Q( 
DA1[15] ) ),
  DFFNA1_14 ( .QN( net55[1] ), .CKN( net059 ), .D( DAT_A[14] ), .RN( 
net045 ), .Q( DA1[14] ) ),
  DFFNA1_13 ( .QN( net55[2] ), .CKN( net059 ), .D( DAT_A[13] ), .RN( 
net045 ), .Q( DA1[13] ) ),
  DFFNA1_12 ( .QN( net55[3] ), .CKN( net059 ), .D( DAT_A[12] ), .RN( 
net045 ), .Q( DA1[12] ) ),
  DFFNA1_11 ( .QN( net55[4] ), .CKN( net059 ), .D( DAT_A[11] ), .RN( 
net045 ), .Q( DA1[11] ) ),
  DFFNA1_10 ( .QN( net55[5] ), .CKN( net059 ), .D( DAT_A[10] ), .RN( 
net045 ), .Q( DA1[10] ) ),
  DFFNA1_9 ( .QN( net55[6] ), .CKN( net059 ), .D( DAT_A[9] ), .RN( 
net045 ), .Q( DA1[9] ) ),
  DFFNA1_8 ( .QN( net55[7] ), .CKN( net059 ), .D( DAT_A[8] ), .RN( 
net045 ), .Q( DA1[8] ) ),
  DFFNA1_7 ( .QN( net55[8] ), .CKN( net059 ), .D( DAT_A[7] ), .RN( 
net045 ), .Q( DA1[7] ) ),
  DFFNA1_6 ( .QN( net55[9] ), .CKN( net059 ), .D( DAT_A[6] ), .RN( 
net045 ), .Q( DA1[6] ) ),
  DFFNA1_5 ( .QN( net55[10] ), .CKN( net059 ), .D( DAT_A[5] ), .RN( 
net045 ), .Q( DA1[5] ) ),
  DFFNA1_4 ( .QN( net55[11] ), .CKN( net059 ), .D( DAT_A[4] ), .RN( 
net045 ), .Q( DA1[4] ) ),
  DFFNA1_3 ( .QN( net55[12] ), .CKN( net059 ), .D( DAT_A[3] ), .RN( 
net045 ), .Q( DA1[3] ) ),
  DFFNA1_2 ( .QN( net55[13] ), .CKN( net059 ), .D( DAT_A[2] ), .RN( 
net045 ), .Q( DA1[2] ) ),
  DFFNA1_1 ( .QN( net55[14] ), .CKN( net059 ), .D( DAT_A[1] ), .RN( 
net045 ), .Q( DA1[1] ) ),
  DFFNA1_0 ( .QN( net55[15] ), .CKN( net059 ), .D( DAT_A[0] ), .RN( 
net045 ), .Q( DA1[0] ) );
DFFNRX1 DFFNA0_15 ( .QN( 
net0177[0] ), .CKN( net063 ), .D( DAT_A0[15] ), .RN( net081 ), .Q( 
DA0[15] ) ),
  DFFNA0_14 ( .QN( net0177[1] ), .CKN( net063 ), .D( DAT_A0[14] ), .RN( 
net081 ), .Q( DA0[14] ) ),
  DFFNA0_13 ( .QN( net0177[2] ), .CKN( net063 ), .D( DAT_A0[13] ), .RN( 
net081 ), .Q( DA0[13] ) ),
  DFFNA0_12 ( .QN( net0177[3] ), .CKN( net063 ), .D( DAT_A0[12] ), .RN( 
net081 ), .Q( DA0[12] ) ),
  DFFNA0_11 ( .QN( net0177[4] ), .CKN( net063 ), .D( DAT_A0[11] ), .RN( 
net081 ), .Q( DA0[11] ) ),
  DFFNA0_10 ( .QN( net0177[5] ), .CKN( net063 ), .D( DAT_A0[10] ), .RN( 
net081 ), .Q( DA0[10] ) ),
  DFFNA0_9 ( .QN( net0177[6] ), .CKN( net063 ), .D( DAT_A0[9] ), .RN( 
net081 ), .Q( DA0[9] ) ),
  DFFNA0_8 ( .QN( net0177[7] ), .CKN( net063 ), .D( DAT_A0[8] ), .RN( 
net081 ), .Q( DA0[8] ) ),
  DFFNA0_7 ( .QN( net0177[8] ), .CKN( net063 ), .D( DAT_A0[7] ), .RN( 
net081 ), .Q( DA0[7] ) ),
  DFFNA0_6 ( .QN( net0177[9] ), .CKN( net063 ), .D( DAT_A0[6] ), .RN( 
net081 ), .Q( DA0[6] ) ),
  DFFNA0_5 ( .QN( net0177[10] ), .CKN( net063 ), .D( DAT_A0[5] ), .RN( 
net081 ), .Q( DA0[5] ) ),
  DFFNA0_4 ( .QN( net0177[11] ), .CKN( net063 ), .D( DAT_A0[4] ), .RN( 
net081 ), .Q( DA0[4] ) ),
  DFFNA0_3 ( .QN( net0177[12] ), .CKN( net063 ), .D( DAT_A0[3] ), .RN( 
net081 ), .Q( DA0[3] ) ),
  DFFNA0_2 ( .QN( net0177[13] ), .CKN( net063 ), .D( DAT_A0[2] ), .RN( 
net081 ), .Q( DA0[2] ) ),
  DFFNA0_1 ( .QN( net0177[14] ), .CKN( net063 ), .D( DAT_A0[1] ), .RN( 
net081 ), .Q( DA0[1] ) ),
  DFFNA0_0 ( .QN( net0177[15] ), .CKN( net063 ), .D( DAT_A0[0] ), .RN( 
net081 ), .Q( DA0[0] ) );
DFFNRX1 DFFNB0_15 ( .QN( 
DB0N[15] ), .CKN( net061 ), .D( DAT_B0[15] ), .RN( net086 ), .Q( 
DB0[15] ) ),
  DFFNB0_14 ( .QN( DB0N[14] ), .CKN( net061 ), .D( DAT_B0[14] ), .RN( 
net086 ), .Q( DB0[14] ) ),
  DFFNB0_13 ( .QN( DB0N[13] ), .CKN( net061 ), .D( DAT_B0[13] ), .RN( 
net086 ), .Q( DB0[13] ) ),
  DFFNB0_12 ( .QN( DB0N[12] ), .CKN( net061 ), .D( DAT_B0[12] ), .RN( 
net086 ), .Q( DB0[12] ) ),
  DFFNB0_11 ( .QN( DB0N[11] ), .CKN( net061 ), .D( DAT_B0[11] ), .RN( 
net086 ), .Q( DB0[11] ) ),
  DFFNB0_10 ( .QN( DB0N[10] ), .CKN( net061 ), .D( DAT_B0[10] ), .RN( 
net086 ), .Q( DB0[10] ) ),
  DFFNB0_9 ( .QN( DB0N[9] ), .CKN( net061 ), .D( DAT_B0[9] ), .RN( 
net086 ), .Q( DB0[9] ) ),
  DFFNB0_8 ( .QN( DB0N[8] ), .CKN( net061 ), .D( DAT_B0[8] ), .RN( 
net086 ), .Q( DB0[8] ) ),
  DFFNB0_7 ( .QN( DB0N[7] ), .CKN( net061 ), .D( DAT_B0[7] ), .RN( 
net086 ), .Q( DB0[7] ) ),
  DFFNB0_6 ( .QN( DB0N[6] ), .CKN( net061 ), .D( DAT_B0[6] ), .RN( 
net086 ), .Q( DB0[6] ) ),
  DFFNB0_5 ( .QN( DB0N[5] ), .CKN( net061 ), .D( DAT_B0[5] ), .RN( 
net086 ), .Q( DB0[5] ) ),
  DFFNB0_4 ( .QN( DB0N[4] ), .CKN( net061 ), .D( DAT_B0[4] ), .RN( 
net086 ), .Q( DB0[4] ) ),
  DFFNB0_3 ( .QN( DB0N[3] ), .CKN( net061 ), .D( DAT_B0[3] ), .RN( 
net086 ), .Q( DB0[3] ) ),
  DFFNB0_2 ( .QN( DB0N[2] ), .CKN( net061 ), .D( DAT_B0[2] ), .RN( 
net086 ), .Q( DB0[2] ) ),
  DFFNB0_1 ( .QN( DB0N[1] ), .CKN( net061 ), .D( DAT_B0[1] ), .RN( 
net086 ), .Q( DB0[1] ) ),
  DFFNB0_0 ( .QN( DB0N[0] ), .CKN( net061 ), .D( DAT_B0[0] ), .RN( 
net086 ), .Q( DB0[0] ) );
 
INVX1 I243
 ( .Y( net045 ), .A( disable_A1 ) );
INVX1 I244
 ( .Y( net088 ), .A( disable_B1 ) );
INVX1 I245
 ( .Y( net095 ), .A( two_adc_per_ln ) );
INVX1 I246
 ( .Y( two_per_ln ), .A( net095 ) );
 
NAND2X4 I237 ( .Y( net041 ), .A( FRM_clk ), .B( net081 ) );
NAND2X4 I240 ( .Y( net0116 ), .A( FRM_clk ), .B( net086 ) );
 
MXI2X2 MUX21_15 ( .Y( DA0b[15] ), .S0( 
two_per_ln ), .A( DA0[15] ), .B( DAB0R[15] ) ),
  MUX21_14 ( .Y( DA0b[14] ), .S0( two_per_ln ), .A( DA0[14] ), .B( 
DAB0R[14] ) ),
  MUX21_13 ( .Y( DA0b[13] ), .S0( two_per_ln ), .A( DA0[13] ), .B( 
DAB0R[13] ) ),
  MUX21_12 ( .Y( DA0b[12] ), .S0( two_per_ln ), .A( DA0[12] ), .B( 
DAB0R[12] ) ),
  MUX21_11 ( .Y( DA0b[11] ), .S0( two_per_ln ), .A( DA0[11] ), .B( 
DAB0R[11] ) ),
  MUX21_10 ( .Y( DA0b[10] ), .S0( two_per_ln ), .A( DA0[10] ), .B( 
DAB0R[10] ) ),
  MUX21_9 ( .Y( DA0b[9] ), .S0( two_per_ln ), .A( DA0[9] ), .B( 
DAB0R[9] ) ),
  MUX21_8 ( .Y( DA0b[8] ), .S0( two_per_ln ), .A( DA0[8] ), .B( 
DAB0R[8] ) ),
  MUX21_7 ( .Y( DA0b[7] ), .S0( two_per_ln ), .A( DA0[7] ), .B( 
DAB0R[7] ) ),
  MUX21_6 ( .Y( DA0b[6] ), .S0( two_per_ln ), .A( DA0[6] ), .B( 
DAB0R[6] ) ),
  MUX21_5 ( .Y( DA0b[5] ), .S0( two_per_ln ), .A( DA0[5] ), .B( 
DAB0R[5] ) ),
  MUX21_4 ( .Y( DA0b[4] ), .S0( two_per_ln ), .A( DA0[4] ), .B( 
DAB0R[4] ) ),
  MUX21_3 ( .Y( DA0b[3] ), .S0( two_per_ln ), .A( DA0[3] ), .B( 
DAB0R[3] ) ),
  MUX21_2 ( .Y( DA0b[2] ), .S0( two_per_ln ), .A( DA0[2] ), .B( 
DAB0R[2] ) ),
  MUX21_1 ( .Y( DA0b[1] ), .S0( two_per_ln ), .A( DA0[1] ), .B( 
DAB0R[1] ) ),
  MUX21_0 ( .Y( DA0b[0] ), .S0( two_per_ln ), .A( DA0[0] ), .B( 
DAB0R[0] ) );
 
DLY4X1 DELAB_15 ( .Y( DAB0R[15] ), .A( 
DAB0[15] ) ),
  DELAB_14 ( .Y( DAB0R[14] ), .A( DAB0[14] ) ),
  DELAB_13 ( .Y( DAB0R[13] ), .A( DAB0[13] ) ),
  DELAB_12 ( .Y( DAB0R[12] ), .A( DAB0[12] ) ),
  DELAB_11 ( .Y( DAB0R[11] ), .A( DAB0[11] ) ),
  DELAB_10 ( .Y( DAB0R[10] ), .A( DAB0[10] ) ),
  DELAB_9 ( .Y( DAB0R[9] ), .A( DAB0[9] ) ),
  DELAB_8 ( .Y( DAB0R[8] ), .A( DAB0[8] ) ),
  DELAB_7 ( .Y( DAB0R[7] ), .A( DAB0[7] ) ),
  DELAB_6 ( .Y( DAB0R[6] ), .A( DAB0[6] ) ),
  DELAB_5 ( .Y( DAB0R[5] ), .A( DAB0[5] ) ),
  DELAB_4 ( .Y( DAB0R[4] ), .A( DAB0[4] ) ),
  DELAB_3 ( .Y( DAB0R[3] ), .A( DAB0[3] ) ),
  DELAB_2 ( .Y( DAB0R[2] ), .A( DAB0[2] ) ),
  DELAB_1 ( .Y( DAB0R[1] ), .A( DAB0[1] ) ),
  DELAB_0 ( .Y( DAB0R[0] ), .A( DAB0[0] ) );
 
DLY3X1 DLYA_15 ( .Y( DAT_A[15] ), .A( 
DATA_A[15] ) ),
  DLYA_14 ( .Y( DAT_A[14] ), .A( DATA_A[14] ) ),
  DLYA_13 ( .Y( DAT_A[13] ), .A( DATA_A[13] ) ),
  DLYA_12 ( .Y( DAT_A[12] ), .A( DATA_A[12] ) ),
  DLYA_11 ( .Y( DAT_A[11] ), .A( DATA_A[11] ) ),
  DLYA_10 ( .Y( DAT_A[10] ), .A( DATA_A[10] ) ),
  DLYA_9 ( .Y( DAT_A[9] ), .A( DATA_A[9] ) ),
  DLYA_8 ( .Y( DAT_A[8] ), .A( DATA_A[8] ) ),
  DLYA_7 ( .Y( DAT_A[7] ), .A( DATA_A[7] ) ),
  DLYA_6 ( .Y( DAT_A[6] ), .A( DATA_A[6] ) ),
  DLYA_5 ( .Y( DAT_A[5] ), .A( DATA_A[5] ) ),
  DLYA_4 ( .Y( DAT_A[4] ), .A( DATA_A[4] ) ),
  DLYA_3 ( .Y( DAT_A[3] ), .A( DATA_A[3] ) ),
  DLYA_2 ( .Y( DAT_A[2] ), .A( DATA_A[2] ) ),
  DLYA_1 ( .Y( DAT_A[1] ), .A( DATA_A[1] ) ),
  DLYA_0 ( .Y( DAT_A[0] ), .A( DATA_A[0] ) );
DLY3X1 DLYB_15 ( .Y( DAT_B[15] ), .A( 
DATA_B[15] ) ),
  DLYB_14 ( .Y( DAT_B[14] ), .A( DATA_B[14] ) ),
  DLYB_13 ( .Y( DAT_B[13] ), .A( DATA_B[13] ) ),
  DLYB_12 ( .Y( DAT_B[12] ), .A( DATA_B[12] ) ),
  DLYB_11 ( .Y( DAT_B[11] ), .A( DATA_B[11] ) ),
  DLYB_10 ( .Y( DAT_B[10] ), .A( DATA_B[10] ) ),
  DLYB_9 ( .Y( DAT_B[9] ), .A( DATA_B[9] ) ),
  DLYB_8 ( .Y( DAT_B[8] ), .A( DATA_B[8] ) ),
  DLYB_7 ( .Y( DAT_B[7] ), .A( DATA_B[7] ) ),
  DLYB_6 ( .Y( DAT_B[6] ), .A( DATA_B[6] ) ),
  DLYB_5 ( .Y( DAT_B[5] ), .A( DATA_B[5] ) ),
  DLYB_4 ( .Y( DAT_B[4] ), .A( DATA_B[4] ) ),
  DLYB_3 ( .Y( DAT_B[3] ), .A( DATA_B[3] ) ),
  DLYB_2 ( .Y( DAT_B[2] ), .A( DATA_B[2] ) ),
  DLYB_1 ( .Y( DAT_B[1] ), .A( DATA_B[1] ) ),
  DLYB_0 ( .Y( DAT_B[0] ), .A( DATA_B[0] ) );
 
NAND2X2 I198 ( .Y( net039 ), .A( FRM_clk ), .B( two_per_ln ) );
NAND2X2 I239 ( .Y( net0126 ), .A( FRM_clk ), .B( net088 ) );
NAND2X2 I238 ( .Y( net053 ), .A( FRM_clk ), .B( net045 ) );
 
INVX2 I241
 ( .Y( net081 ), .A( disable_A0 ) );
INVX2 INVDB1_15 ( .Y( DB1b[15] ), 
.A( DB1[15] ) ),
  INVDB1_14 ( .Y( DB1b[14] ), .A( DB1[14] ) ),
  INVDB1_13 ( .Y( DB1b[13] ), .A( DB1[13] ) ),
  INVDB1_12 ( .Y( DB1b[12] ), .A( DB1[12] ) ),
  INVDB1_11 ( .Y( DB1b[11] ), .A( DB1[11] ) ),
  INVDB1_10 ( .Y( DB1b[10] ), .A( DB1[10] ) ),
  INVDB1_9 ( .Y( DB1b[9] ), .A( DB1[9] ) ),
  INVDB1_8 ( .Y( DB1b[8] ), .A( DB1[8] ) ),
  INVDB1_7 ( .Y( DB1b[7] ), .A( DB1[7] ) ),
  INVDB1_6 ( .Y( DB1b[6] ), .A( DB1[6] ) ),
  INVDB1_5 ( .Y( DB1b[5] ), .A( DB1[5] ) ),
  INVDB1_4 ( .Y( DB1b[4] ), .A( DB1[4] ) ),
  INVDB1_3 ( .Y( DB1b[3] ), .A( DB1[3] ) ),
  INVDB1_2 ( .Y( DB1b[2] ), .A( DB1[2] ) ),
  INVDB1_1 ( .Y( DB1b[1] ), .A( DB1[1] ) ),
  INVDB1_0 ( .Y( DB1b[0] ), .A( DB1[0] ) );
INVX2 INVDB0_15 ( .Y( DB0b[15] ), 
.A( DB0[15] ) ),
  INVDB0_14 ( .Y( DB0b[14] ), .A( DB0[14] ) ),
  INVDB0_13 ( .Y( DB0b[13] ), .A( DB0[13] ) ),
  INVDB0_12 ( .Y( DB0b[12] ), .A( DB0[12] ) ),
  INVDB0_11 ( .Y( DB0b[11] ), .A( DB0[11] ) ),
  INVDB0_10 ( .Y( DB0b[10] ), .A( DB0[10] ) ),
  INVDB0_9 ( .Y( DB0b[9] ), .A( DB0[9] ) ),
  INVDB0_8 ( .Y( DB0b[8] ), .A( DB0[8] ) ),
  INVDB0_7 ( .Y( DB0b[7] ), .A( DB0[7] ) ),
  INVDB0_6 ( .Y( DB0b[6] ), .A( DB0[6] ) ),
  INVDB0_5 ( .Y( DB0b[5] ), .A( DB0[5] ) ),
  INVDB0_4 ( .Y( DB0b[4] ), .A( DB0[4] ) ),
  INVDB0_3 ( .Y( DB0b[3] ), .A( DB0[3] ) ),
  INVDB0_2 ( .Y( DB0b[2] ), .A( DB0[2] ) ),
  INVDB0_1 ( .Y( DB0b[1] ), .A( DB0[1] ) ),
  INVDB0_0 ( .Y( DB0b[0] ), .A( DB0[0] ) );
INVX2 INVDA1_15 ( .Y( DA1b[15] ), 
.A( DA1[15] ) ),
  INVDA1_14 ( .Y( DA1b[14] ), .A( DA1[14] ) ),
  INVDA1_13 ( .Y( DA1b[13] ), .A( DA1[13] ) ),
  INVDA1_12 ( .Y( DA1b[12] ), .A( DA1[12] ) ),
  INVDA1_11 ( .Y( DA1b[11] ), .A( DA1[11] ) ),
  INVDA1_10 ( .Y( DA1b[10] ), .A( DA1[10] ) ),
  INVDA1_9 ( .Y( DA1b[9] ), .A( DA1[9] ) ),
  INVDA1_8 ( .Y( DA1b[8] ), .A( DA1[8] ) ),
  INVDA1_7 ( .Y( DA1b[7] ), .A( DA1[7] ) ),
  INVDA1_6 ( .Y( DA1b[6] ), .A( DA1[6] ) ),
  INVDA1_5 ( .Y( DA1b[5] ), .A( DA1[5] ) ),
  INVDA1_4 ( .Y( DA1b[4] ), .A( DA1[4] ) ),
  INVDA1_3 ( .Y( DA1b[3] ), .A( DA1[3] ) ),
  INVDA1_2 ( .Y( DA1b[2] ), .A( DA1[2] ) ),
  INVDA1_1 ( .Y( DA1b[1] ), .A( DA1[1] ) ),
  INVDA1_0 ( .Y( DA1b[0] ), .A( DA1[0] ) );
INVX2 I242
 ( .Y( net086 ), .A( disable_B0 ) );
INVX2 I212
 ( .Y( net0115 ), .A( FRM_clk ) );
 
INVX4 INVDB0b_15 ( .Y( 
DATA_B0[15] ), .A( DB0b[15] ) ),
  INVDB0b_14 ( .Y( DATA_B0[14] ), .A( DB0b[14] ) ),
  INVDB0b_13 ( .Y( DATA_B0[13] ), .A( DB0b[13] ) ),
  INVDB0b_12 ( .Y( DATA_B0[12] ), .A( DB0b[12] ) ),
  INVDB0b_11 ( .Y( DATA_B0[11] ), .A( DB0b[11] ) ),
  INVDB0b_10 ( .Y( DATA_B0[10] ), .A( DB0b[10] ) ),
  INVDB0b_9 ( .Y( DATA_B0[9] ), .A( DB0b[9] ) ),
  INVDB0b_8 ( .Y( DATA_B0[8] ), .A( DB0b[8] ) ),
  INVDB0b_7 ( .Y( DATA_B0[7] ), .A( DB0b[7] ) ),
  INVDB0b_6 ( .Y( DATA_B0[6] ), .A( DB0b[6] ) ),
  INVDB0b_5 ( .Y( DATA_B0[5] ), .A( DB0b[5] ) ),
  INVDB0b_4 ( .Y( DATA_B0[4] ), .A( DB0b[4] ) ),
  INVDB0b_3 ( .Y( DATA_B0[3] ), .A( DB0b[3] ) ),
  INVDB0b_2 ( .Y( DATA_B0[2] ), .A( DB0b[2] ) ),
  INVDB0b_1 ( .Y( DATA_B0[1] ), .A( DB0b[1] ) ),
  INVDB0b_0 ( .Y( DATA_B0[0] ), .A( DB0b[0] ) );
INVX4 INVDB1b_15 ( .Y( 
DATA_B1[15] ), .A( DB1b[15] ) ),
  INVDB1b_14 ( .Y( DATA_B1[14] ), .A( DB1b[14] ) ),
  INVDB1b_13 ( .Y( DATA_B1[13] ), .A( DB1b[13] ) ),
  INVDB1b_12 ( .Y( DATA_B1[12] ), .A( DB1b[12] ) ),
  INVDB1b_11 ( .Y( DATA_B1[11] ), .A( DB1b[11] ) ),
  INVDB1b_10 ( .Y( DATA_B1[10] ), .A( DB1b[10] ) ),
  INVDB1b_9 ( .Y( DATA_B1[9] ), .A( DB1b[9] ) ),
  INVDB1b_8 ( .Y( DATA_B1[8] ), .A( DB1b[8] ) ),
  INVDB1b_7 ( .Y( DATA_B1[7] ), .A( DB1b[7] ) ),
  INVDB1b_6 ( .Y( DATA_B1[6] ), .A( DB1b[6] ) ),
  INVDB1b_5 ( .Y( DATA_B1[5] ), .A( DB1b[5] ) ),
  INVDB1b_4 ( .Y( DATA_B1[4] ), .A( DB1b[4] ) ),
  INVDB1b_3 ( .Y( DATA_B1[3] ), .A( DB1b[3] ) ),
  INVDB1b_2 ( .Y( DATA_B1[2] ), .A( DB1b[2] ) ),
  INVDB1b_1 ( .Y( DATA_B1[1] ), .A( DB1b[1] ) ),
  INVDB1b_0 ( .Y( DATA_B1[0] ), .A( DB1b[0] ) );
INVX4 INVDA1b_15 ( .Y( 
DATA_A1[15] ), .A( DA1b[15] ) ),
  INVDA1b_14 ( .Y( DATA_A1[14] ), .A( DA1b[14] ) ),
  INVDA1b_13 ( .Y( DATA_A1[13] ), .A( DA1b[13] ) ),
  INVDA1b_12 ( .Y( DATA_A1[12] ), .A( DA1b[12] ) ),
  INVDA1b_11 ( .Y( DATA_A1[11] ), .A( DA1b[11] ) ),
  INVDA1b_10 ( .Y( DATA_A1[10] ), .A( DA1b[10] ) ),
  INVDA1b_9 ( .Y( DATA_A1[9] ), .A( DA1b[9] ) ),
  INVDA1b_8 ( .Y( DATA_A1[8] ), .A( DA1b[8] ) ),
  INVDA1b_7 ( .Y( DATA_A1[7] ), .A( DA1b[7] ) ),
  INVDA1b_6 ( .Y( DATA_A1[6] ), .A( DA1b[6] ) ),
  INVDA1b_5 ( .Y( DATA_A1[5] ), .A( DA1b[5] ) ),
  INVDA1b_4 ( .Y( DATA_A1[4] ), .A( DA1b[4] ) ),
  INVDA1b_3 ( .Y( DATA_A1[3] ), .A( DA1b[3] ) ),
  INVDA1b_2 ( .Y( DATA_A1[2] ), .A( DA1b[2] ) ),
  INVDA1b_1 ( .Y( DATA_A1[1] ), .A( DA1b[1] ) ),
  INVDA1b_0 ( .Y( DATA_A1[0] ), .A( DA1b[0] ) );
INVX4 I236
 ( .Y( net055 ), .A( net0116 ) );
INVX4 INVDA0b_15 ( .Y( 
DATA_A0[15] ), .A( DA0b[15] ) ),
  INVDA0b_14 ( .Y( DATA_A0[14] ), .A( DA0b[14] ) ),
  INVDA0b_13 ( .Y( DATA_A0[13] ), .A( DA0b[13] ) ),
  INVDA0b_12 ( .Y( DATA_A0[12] ), .A( DA0b[12] ) ),
  INVDA0b_11 ( .Y( DATA_A0[11] ), .A( DA0b[11] ) ),
  INVDA0b_10 ( .Y( DATA_A0[10] ), .A( DA0b[10] ) ),
  INVDA0b_9 ( .Y( DATA_A0[9] ), .A( DA0b[9] ) ),
  INVDA0b_8 ( .Y( DATA_A0[8] ), .A( DA0b[8] ) ),
  INVDA0b_7 ( .Y( DATA_A0[7] ), .A( DA0b[7] ) ),
  INVDA0b_6 ( .Y( DATA_A0[6] ), .A( DA0b[6] ) ),
  INVDA0b_5 ( .Y( DATA_A0[5] ), .A( DA0b[5] ) ),
  INVDA0b_4 ( .Y( DATA_A0[4] ), .A( DA0b[4] ) ),
  INVDA0b_3 ( .Y( DATA_A0[3] ), .A( DA0b[3] ) ),
  INVDA0b_2 ( .Y( DATA_A0[2] ), .A( DA0b[2] ) ),
  INVDA0b_1 ( .Y( DATA_A0[1] ), .A( DA0b[1] ) ),
  INVDA0b_0 ( .Y( DATA_A0[0] ), .A( DA0b[0] ) );
INVX4 I197
 ( .Y( net051 ), .A( net039 ) );
INVX4 I235
 ( .Y( net057 ), .A( net041 ) );
INVX4 I213
 ( .Y( PLL_ref ), .A( net0115 ) );
INVX4 I231
 ( .Y( net065 ), .A( net0126 ) );
INVX4 I234
 ( .Y( net059 ), .A( net053 ) );
INVX4 I233
 ( .Y( net063 ), .A( net041 ) );
INVX4 I232
 ( .Y( net061 ), .A( net0116 ) );
 
MXI2X1 MUXAB0_15 ( .Y( DAB0[15] ), 
.S0( net051 ), .A( DAT_A0N[15] ), .B( DB0N[15] ) ),
  MUXAB0_14 ( .Y( DAB0[14] ), .S0( net051 ), .A( DAT_A0N[14] ), .B( 
DB0N[14] ) ),
  MUXAB0_13 ( .Y( DAB0[13] ), .S0( net051 ), .A( DAT_A0N[13] ), .B( 
DB0N[13] ) ),
  MUXAB0_12 ( .Y( DAB0[12] ), .S0( net051 ), .A( DAT_A0N[12] ), .B( 
DB0N[12] ) ),
  MUXAB0_11 ( .Y( DAB0[11] ), .S0( net051 ), .A( DAT_A0N[11] ), .B( 
DB0N[11] ) ),
  MUXAB0_10 ( .Y( DAB0[10] ), .S0( net051 ), .A( DAT_A0N[10] ), .B( 
DB0N[10] ) ),
  MUXAB0_9 ( .Y( DAB0[9] ), .S0( net051 ), .A( DAT_A0N[9] ), .B( 
DB0N[9] ) ),
  MUXAB0_8 ( .Y( DAB0[8] ), .S0( net051 ), .A( DAT_A0N[8] ), .B( 
DB0N[8] ) ),
  MUXAB0_7 ( .Y( DAB0[7] ), .S0( net051 ), .A( DAT_A0N[7] ), .B( 
DB0N[7] ) ),
  MUXAB0_6 ( .Y( DAB0[6] ), .S0( net051 ), .A( DAT_A0N[6] ), .B( 
DB0N[6] ) ),
  MUXAB0_5 ( .Y( DAB0[5] ), .S0( net051 ), .A( DAT_A0N[5] ), .B( 
DB0N[5] ) ),
  MUXAB0_4 ( .Y( DAB0[4] ), .S0( net051 ), .A( DAT_A0N[4] ), .B( 
DB0N[4] ) ),
  MUXAB0_3 ( .Y( DAB0[3] ), .S0( net051 ), .A( DAT_A0N[3] ), .B( 
DB0N[3] ) ),
  MUXAB0_2 ( .Y( DAB0[2] ), .S0( net051 ), .A( DAT_A0N[2] ), .B( 
DB0N[2] ) ),
  MUXAB0_1 ( .Y( DAB0[1] ), .S0( net051 ), .A( DAT_A0N[1] ), .B( 
DB0N[1] ) ),
  MUXAB0_0 ( .Y( DAB0[0] ), .S0( net051 ), .A( DAT_A0N[0] ), .B( 
DB0N[0] ) );
 
DFFX1 DFFB0_15 ( .QN( net77[0] ), .CK( 
net055 ), .D( DAT_B[15] ), .Q( DAT_B0[15] ) ),
  DFFB0_14 ( .QN( net77[1] ), .CK( net055 ), .D( DAT_B[14] ), .Q( 
DAT_B0[14] ) ),
  DFFB0_13 ( .QN( net77[2] ), .CK( net055 ), .D( DAT_B[13] ), .Q( 
DAT_B0[13] ) ),
  DFFB0_12 ( .QN( net77[3] ), .CK( net055 ), .D( DAT_B[12] ), .Q( 
DAT_B0[12] ) ),
  DFFB0_11 ( .QN( net77[4] ), .CK( net055 ), .D( DAT_B[11] ), .Q( 
DAT_B0[11] ) ),
  DFFB0_10 ( .QN( net77[5] ), .CK( net055 ), .D( DAT_B[10] ), .Q( 
DAT_B0[10] ) ),
  DFFB0_9 ( .QN( net77[6] ), .CK( net055 ), .D( DAT_B[9] ), .Q( 
DAT_B0[9] ) ),
  DFFB0_8 ( .QN( net77[7] ), .CK( net055 ), .D( DAT_B[8] ), .Q( 
DAT_B0[8] ) ),
  DFFB0_7 ( .QN( net77[8] ), .CK( net055 ), .D( DAT_B[7] ), .Q( 
DAT_B0[7] ) ),
  DFFB0_6 ( .QN( net77[9] ), .CK( net055 ), .D( DAT_B[6] ), .Q( 
DAT_B0[6] ) ),
  DFFB0_5 ( .QN( net77[10] ), .CK( net055 ), .D( DAT_B[5] ), .Q( 
DAT_B0[5] ) ),
  DFFB0_4 ( .QN( net77[11] ), .CK( net055 ), .D( DAT_B[4] ), .Q( 
DAT_B0[4] ) ),
  DFFB0_3 ( .QN( net77[12] ), .CK( net055 ), .D( DAT_B[3] ), .Q( 
DAT_B0[3] ) ),
  DFFB0_2 ( .QN( net77[13] ), .CK( net055 ), .D( DAT_B[2] ), .Q( 
DAT_B0[2] ) ),
  DFFB0_1 ( .QN( net77[14] ), .CK( net055 ), .D( DAT_B[1] ), .Q( 
DAT_B0[1] ) ),
  DFFB0_0 ( .QN( net77[15] ), .CK( net055 ), .D( DAT_B[0] ), .Q( 
DAT_B0[0] ) );
DFFX1 DFFA0_15 ( .QN( DAT_A0N[15] ), 
.CK( net057 ), .D( DAT_A[15] ), .Q( DAT_A0[15] ) ),
  DFFA0_14 ( .QN( DAT_A0N[14] ), .CK( net057 ), .D( DAT_A[14] ), .Q( 
DAT_A0[14] ) ),
  DFFA0_13 ( .QN( DAT_A0N[13] ), .CK( net057 ), .D( DAT_A[13] ), .Q( 
DAT_A0[13] ) ),
  DFFA0_12 ( .QN( DAT_A0N[12] ), .CK( net057 ), .D( DAT_A[12] ), .Q( 
DAT_A0[12] ) ),
  DFFA0_11 ( .QN( DAT_A0N[11] ), .CK( net057 ), .D( DAT_A[11] ), .Q( 
DAT_A0[11] ) ),
  DFFA0_10 ( .QN( DAT_A0N[10] ), .CK( net057 ), .D( DAT_A[10] ), .Q( 
DAT_A0[10] ) ),
  DFFA0_9 ( .QN( DAT_A0N[9] ), .CK( net057 ), .D( DAT_A[9] ), .Q( 
DAT_A0[9] ) ),
  DFFA0_8 ( .QN( DAT_A0N[8] ), .CK( net057 ), .D( DAT_A[8] ), .Q( 
DAT_A0[8] ) ),
  DFFA0_7 ( .QN( DAT_A0N[7] ), .CK( net057 ), .D( DAT_A[7] ), .Q( 
DAT_A0[7] ) ),
  DFFA0_6 ( .QN( DAT_A0N[6] ), .CK( net057 ), .D( DAT_A[6] ), .Q( 
DAT_A0[6] ) ),
  DFFA0_5 ( .QN( DAT_A0N[5] ), .CK( net057 ), .D( DAT_A[5] ), .Q( 
DAT_A0[5] ) ),
  DFFA0_4 ( .QN( DAT_A0N[4] ), .CK( net057 ), .D( DAT_A[4] ), .Q( 
DAT_A0[4] ) ),
  DFFA0_3 ( .QN( DAT_A0N[3] ), .CK( net057 ), .D( DAT_A[3] ), .Q( 
DAT_A0[3] ) ),
  DFFA0_2 ( .QN( DAT_A0N[2] ), .CK( net057 ), .D( DAT_A[2] ), .Q( 
DAT_A0[2] ) ),
  DFFA0_1 ( .QN( DAT_A0N[1] ), .CK( net057 ), .D( DAT_A[1] ), .Q( 
DAT_A0[1] ) ),
  DFFA0_0 ( .QN( DAT_A0N[0] ), .CK( net057 ), .D( DAT_A[0] ), .Q( 
DAT_A0[0] ) );

endmodule

// 
// Design File for: (tsmc18 DFFNRX1 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module DFFNRX1 (Q, QN, D, CKN, RN);
output Q, QN;
input  D, CKN, RN;
reg NOTIFIER;
supply1 xSN;

  buf   XX0 (xRN, RN);
  not     IC (clk, CKN);
  udp_dff I0 (n0, D, clk, xRN, xSN, NOTIFIER);
  and     I4 (flag, xRN, xSN);
  buf     I1 (Q, n0);
  not     I2 (QN, n0);

  specify
    specparam
    tplh$RN$Q  = 0.05,
    tphl$RN$Q  = 0.05,
    tplh$RN$QN  = 0.05,
    tphl$RN$QN  = 0.05,
    tplh$CKN$Q  = 0.05,
    tphl$CKN$Q  = 0.05,
    tplh$CKN$QN = 0.05,
    tphl$CKN$QN = 0.05,
    tsetup$D$CKN        = 0.05,
    thold$D$CKN = 0.02,
    tsetup$RN$CKN    = 0.05,
    thold$RN$CKN    = 0.02,
    tminpwl$RN     = 0.05,
    tminpwl$CKN    = 0.05,
    tminpwh$CKN    = 0.05;

    if (flag)
      (negedge CKN *> (Q    +: D)) = (tplh$CKN$Q,    tphl$CKN$Q);
    if (flag)
      (negedge CKN *> (QN -: D)) = (tplh$CKN$QN, tphl$CKN$QN);
    $setuphold(negedge CKN &&& (flag == 1), posedge D, tsetup$D$CKN, thold$D$CKN, NOTIFIER);
    $setuphold(negedge CKN &&& (flag == 1), negedge D, tsetup$D$CKN, thold$D$CKN, NOTIFIER);
      (negedge RN *> (Q +: 1'b0)) = (tphl$RN$Q);
      (negedge RN *> (QN -: 1'b0)) = (tplh$RN$QN);
    $setuphold(negedge CKN, posedge RN , tsetup$RN$CKN, thold$RN$CKN, NOTIFIER);
    $width(negedge RN , tminpwl$RN, 0, NOTIFIER);
    $width(negedge CKN &&& (flag == 1), tminpwl$CKN, 0, NOTIFIER);
    $width(posedge CKN &&& (flag == 1), tminpwh$CKN, 0, NOTIFIER); 

   endspecify
endmodule // DFFNRX1

`endcelldefine

// 
// Design File for: (tsmc18 INVX1 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module INVX1 (Y, A);
output Y;
input A;

  not I0(Y, A);
  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
  endspecify

endmodule // INVX1

`endcelldefine

// 
// Design File for: (tsmc18 NAND2X4 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module NAND2X4 (Y, A, B);
output Y;
input A, B;

  nand (Y, A, B);

  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05,
      tplh$B$Y = 0.05,
      tphl$B$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
    (B *> Y) = (tplh$B$Y, tphl$B$Y);
  endspecify

endmodule // NAND2X4

`endcelldefine

// 
// Design File for: (tsmc18 MXI2X2 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module MXI2X2 (Y, A, B, S0);
output Y;
input A, B, S0;

  udp_mux2 u0(YN, A, B, S0);
  not      u1(Y, YN);

  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05,
      tplh$B$Y = 0.05,
      tphl$B$Y = 0.05,
      tplh$S0$Y = 0.05,
      tphl$S0$Y = 0.05;
    // path delays
     if ((A == 1'b1) && (B == 1'b0))
        (S0 *> Y) = (tplh$S0$Y, tphl$S0$Y);
     if ((A == 1'b0) && (B == 1'b1))
        (S0 *> Y) = (tplh$S0$Y, tphl$S0$Y);
     (B *> Y) = (tplh$B$Y, tphl$B$Y);
     (A *> Y) = (tplh$A$Y, tphl$A$Y);
  endspecify

endmodule // MXI2X2

`endcelldefine

// 
// Design File for: (tsmc18 DLY4X1 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module DLY4X1 (Y, A);
output Y;
input A;

  buf I0(Y, A);
  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.6,
      tphl$A$Y = 0.6;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
  endspecify

endmodule // DLY4X1

`endcelldefine

// 
// Design File for: (tsmc18 DLY3X1 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module DLY3X1 (Y, A);
output Y;
input A;

  buf I0(Y, A);
  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.8,
      tphl$A$Y = 0.8;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
  endspecify

endmodule // DLY3X1

`endcelldefine

// 
// Design File for: (tsmc18 NAND2X2 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module NAND2X2 (Y, A, B);
output Y;
input A, B;

  nand (Y, A, B);

  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05,
      tplh$B$Y = 0.05,
      tphl$B$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
    (B *> Y) = (tplh$B$Y, tphl$B$Y);
  endspecify

endmodule // NAND2X2

`endcelldefine

// 
// Design File for: (tsmc18 INVX2 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module INVX2 (Y, A);
output Y;
input A;

  not I0(Y, A);
  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
  endspecify

endmodule // INVX2

`endcelldefine

// 
// Design File for: (tsmc18 INVX4 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module INVX4 (Y, A);
output Y;
input A;

  not I0(Y, A);
  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05;

    // path delays
    (A *> Y) = (tplh$A$Y, tphl$A$Y);
  endspecify

endmodule // INVX4

`endcelldefine

// 
// Design File for: (tsmc18 MXI2X1 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module MXI2X1 (Y, A, B, S0);
output Y;
input A, B, S0;

  udp_mux2 u0(YN, A, B, S0);
  not      u1(Y, YN);

  specify
    // delay parameters
    specparam
      tplh$A$Y = 0.05,
      tphl$A$Y = 0.05,
      tplh$B$Y = 0.05,
      tphl$B$Y = 0.05,
      tplh$S0$Y = 0.05,
      tphl$S0$Y = 0.05;
    // path delays
     if ((A == 1'b1) && (B == 1'b0))
        (S0 *> Y) = (tplh$S0$Y, tphl$S0$Y);
     if ((A == 1'b0) && (B == 1'b1))
        (S0 *> Y) = (tplh$S0$Y, tphl$S0$Y);
     (B *> Y) = (tplh$B$Y, tphl$B$Y);
     (A *> Y) = (tplh$A$Y, tphl$A$Y);
  endspecify

endmodule // MXI2X1

`endcelldefine

// 
// Design File for: (tsmc18 udp_mux2 verilog)
//
// Created by ihdl
primitive udp_mux2 (out, in0, in1, sel);
   output out;  
   input  in0, in1, sel;

   table

// in0 in1  sel :  out
//
   1  ?   0  :  1 ;
   0  ?   0  :  0 ;
   ?  1   1  :  1 ;
   ?  0   1  :  0 ;
   0  0   x  :  0 ;
   1  1   x  :  1 ;

   endtable
endprimitive // udp_mux2

// 
// Design File for: (tsmc18 DFFX1 verilog)
//
// Created by ihdl
`timescale 1ns/1ps

`celldefine

module DFFX1 (Q, QN, D, CK);
output Q, QN;
input  D, CK;
reg NOTIFIER;
supply1 xSN,xRN;
  buf     IC (clk, CK);
  udp_dff I0 (n0, D, clk, xRN, xSN, NOTIFIER);
  and     I4 (flag, xRN, xSN);
  buf     I1 (Q, n0);
  not     I2 (QN, n0);

  specify
    specparam
    tplh$CK$Q   = 0.05,
    tphl$CK$Q   = 0.05,
    tplh$CK$QN  = 0.05,
    tphl$CK$QN  = 0.05,
    tsetup$D$CK = 0.05,
    thold$D$CK  = 0.02,
    tminpwl$CK    = 0.05,
    tminpwh$CK    = 0.05;


    if (flag)
      (posedge CK *> (Q +: D)) = (tplh$CK$Q,    tphl$CK$Q);
    if (flag)
      (posedge CK *> (QN -: D)) = (tplh$CK$QN,   tphl$CK$QN);
    $setuphold(posedge CK &&& (flag == 1), posedge D, tsetup$D$CK, thold$D$CK, NOTIFIER);
    $setuphold(posedge CK &&& (flag == 1), negedge D, tsetup$D$CK, thold$D$CK, NOTIFIER);
    $width(negedge CK &&& (flag == 1), tminpwl$CK, 0, NOTIFIER);
    $width(posedge CK &&& (flag == 1), tminpwh$CK, 0, NOTIFIER); 

   endspecify
endmodule // DFFX1

`endcelldefine

// 
// Design File for: (tsmc18 udp_dff verilog)
//
// Created by ihdl
primitive udp_dff (out, in, clk, clr_, set_, NOTIFIER);
   output out;  
   input  in, clk, clr_, set_, NOTIFIER;
   reg    out;

   table

// in  clk  clr_   set_  NOT  : Qt : Qt+1
//
   0  r   ?   1   ?   : ?  :  0  ; // clock in 0
   1  r   1   ?   ?   : ?  :  1  ; // clock in 1
   1  *   1   ?   ?   : 1  :  1  ; // reduce pessimism
   0  *   ?   1   ?   : 0  :  0  ; // reduce pessimism
   ?  f   ?   ?   ?   : ?  :  -  ; // no changes on negedge clk
   *  b   ?   ?   ?   : ?  :  -  ; // no changes when in switches
   ?  ?   ?   0   ?   : ?  :  1  ; // set output
   ?  b   1   *   ?   : 1  :  1  ; // cover all transistions on set_
   1  x   1   *   ?   : 1  :  1  ; // cover all transistions on set_
   ?  ?   0   1   ?   : ?  :  0  ; // reset output
   ?  b   *   1   ?   : 0  :  0  ; // cover all transistions on clr_
   0  x   *   1   ?   : 0  :  0  ; // cover all transistions on clr_
   ?  ?   ?   ?   *   : ?  :  x  ; // any notifier changed

   endtable
endprimitive // udp_dff
